package com.asdf.plugins.hello;

import java.awt.Dimension;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.WindowConstants;

import com.asdf.gui.lobby.CreateGamePanel;
import com.asdf.parser.CreateGameRequest;
import com.asdf.parser.Parameters;
import com.asdf.utils.BasicElement;


/**
 * Game creation panel for HelloGame game example. It is recommended to create panels that can
 * be used by several games, instead of tailoring a single one for each. See pre created panels
 * before creating your own, might save a lot of trouble.
 * 
 * Game type is not decided in the game creation panel, but on the upper level of hierarchy.
 * This means that the panels can be used by any game, as long as the transmitted information
 * suffices for all games that use it.
 */
public class CreatePanel extends CreateGamePanel{

	private static final long serialVersionUID = -7975533206336492104L;
	private JLabel helloLabel;

	/**
	* Auto-generated main method to display this 
	* JPanel inside a new JFrame.
	*/
	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.getContentPane().add(new CreatePanel());
		frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		frame.pack();
		frame.setVisible(true);
	}
	
	/**
	 * Creates a simplex game creation panel.
	 *
	 */
	public CreatePanel() {
		super();
		initGUI();
		setInitialized(true);
	}
	
	
	/**
	 * Creates a HelloGame creation panel. The name of the classes to launch with game start
	 * are transmitted automatically when the game meta files are properly in place.
	 * 
	 * This means that you can use the same game creation panel for different games that ask
	 * for similar parameters for creation from the user. For example many poker games can 
	 * easily share their game creation panel.
	 * 
	 * This game creation panel asks for no information from the user, and doesn't suit for
	 * games that have some creation options (for example time per turn, number of players
	 * before starting game, required player level, difficulty, game name, etc..). The only thing the panel
	 * does, is listen to mouse clicks. When the mouse is clicked on the panel, it sends a
	 * game creation request to server.
	 */
	public void initGUI() {
		try {
			setPreferredSize(new Dimension(400, 300));
			{
				helloLabel = new JLabel();
				this.add(helloLabel);
				helloLabel.setText("Hello World!");
				
				
				helloLabel.setPreferredSize(new java.awt.Dimension(218, 178));
				helloLabel.addMouseListener(new MouseListener(){

					@Override
					public void mouseClicked(MouseEvent e) {
                		CreateGameRequest createReq = new CreateGameRequest();                		
                		String gameName = "Helloworld";
                		createReq.setName(gameName);
                		
                		Parameters param = new Parameters();
                		param.getAny().add(new BasicElement("nimi"));
                		
                		createReq.setParameters(param);
                        getController().requestCreateGame(createReq);
					}

					@Override
					public void mouseEntered(MouseEvent e) {
						// TODO Auto-generated method stub
						
					}

					@Override
					public void mouseExited(MouseEvent e) {
						// TODO Auto-generated method stub
						
					}

					@Override
					public void mousePressed(MouseEvent e) {
						// TODO Auto-generated method stub
						
					}

					@Override
					public void mouseReleased(MouseEvent e) {
						// TODO Auto-generated method stub
						
					}});
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
