package com.asdf.plugins.hello;

import com.asdf.common.Game;
import com.asdf.parser.GameDetailsResponse;
import com.asdf.parser.GameRequest;
import com.asdf.parser.GameResponse;
import com.asdf.parser.GenericResponse;
import com.asdf.parser.OptionalStringParameters;
import com.asdf.parser.GameDetailsResponse.Players;

public class ServerSideHelloGameLogic extends Game {

	@Override
	protected String[] columnNames() {
		return new String[]{"Jou", "Jeah", "Jaa", "123"};
	}

	/**
	 * A small description about the game to the game listing.
	 * The bare minimum of information is sent for the purposes of clarity.
	 */
	@Override
	protected com.asdf.parser.GameListResponse.Games.Game description() {

		com.asdf.parser.GameListResponse.Games.Game game = new com.asdf.parser.GameListResponse.Games.Game();
		game.setGameID(this.getId());
		game.setName(this.getName());
		game.setGameType("Example game");
		game.setMaxPlayers(1);
		game.setPlayers(0);
		
		/// Variant name is required, and affects the categories in client game tree, in which this game can be found.
		game.setGameVariant("ExampleGame");
		
		return game;
	}

	/**
	 * Sends details about this game instance to the clients.
	 * The example is not very interested in telling things about it self,
	 * as there is very little to say. So the bare minimum of information is sent.
	 */
	@Override
	protected void details(GameDetailsResponse a) {
		OptionalStringParameters b = new OptionalStringParameters();
		Players plrs = new Players();
		
		a.setColumnNames(b);
		a.setPlayers(plrs);
		a.setRules(b);
	}

	/**
	 * Kills the game program. If a game is deemed as trash (no players playing it for one minute)
	 * it is killed [note that when a game is created, it has an extra 1 minute of time when it is
	 * not checked for trashyness at all, giving it a minimum of 2 minutes life time].
	 * This method is called to give the game a chance to make any final deeds
	 * that might be necessary for proper client service.
	 */
	@Override
	public void die() {
		// I'm ready to die ANYTIME. As soon as Server deems me as trash,
		// I will die. (About 2 minutes after creation)
	}

	/**
	 * User sends a request. Reacting to those requests should be done in this method.
	 * In this example we just check if the client wants to join the game, and if so,
	 * send a response that he is welcome to start the example application. The client is not registered as a player,
	 * as this example game doesn't really have players. If it did, it would have to keep track
	 * of clients in it's own data structures and use that information in game details responses
	 * and game list responses.
	 */
	@Override
	protected void handleRequest(long sessionID, GameRequest request) {
		
		GameResponse msg = new GameResponse();
		GenericResponse gr = new GenericResponse();
		msg.setJoin(gr);
		
		gr.setSuccess(true);
		gr.setDescription("HelloGame");
		
		if (request.getJoin() != null)
			send(sessionID, msg);
		
	}

	/**
	 * If we want to react to clients' responses, this is the place.
	 * This example will not cover any specific things one might
	 * wish to do in this method, your hands are free. Try things
	 * out and build your perfect game.
	 */
	@Override
	protected void handleResponse(long sessionID, GameResponse response) {
		
	}

}
