package com.asdf.plugins.pokergames;

import java.util.Vector;
import java.util.Random;

/**
 * Represents a traditional 52 card deck with no jokers.
 * @author urtela
 *
 */

public class Deck
{
	/**
	 * Default constructor. Initialises the deck into a randomised 52 card deck. No jokers.
	 *
	 */
	public Deck()
	{
		cards = new Vector<Card>();
		newDeck();
		//shuffle();
	}

	/**
	 * Initialises this deck into a randomised 52 card deck. No jokers.
	 * Removes any possible information the deck might previously contain.
	 */
	public void newDeck()
	{
		cards.clear();
		int i, j, n = 0, suit = 0x8000;
		for ( i = 0; i < 4; i++, suit >>= 1 )
			for ( j = 0; j < 13; j++, n++ )
				cards.add(new Card( (primes[j] | (j << 8) | suit | (1 << (16+j)))) );
		shuffle();
	}

	/**
	 * Shuffles the deck.
	 *
	 */
	public void shuffle()
	{
		Random generator = new Random();

		for(int i=0; i<cards.size(); i++)
		{
			int target = generator.nextInt( cards.size() - i ) + i;
			cards.elementAt(i).swap(cards.elementAt(target));
		}
	}

	/**
	 * Removes the bottom card from the deck.
	 * @return The bottom card of the deck.
	 */
	public Card dealCard()
	{
		if(cards.size() == 0)
		{
			Card deal = null;
			return deal;
		}

		Card deal = cards.lastElement();
		cards.remove(cards.size() - 1);
		return deal;
	}

	/**
	 * Extracts known card from the deck.
	 */
	public void extractCard(Card c) {
		for (int i=0; i < cards.size(); i++) {
			if (cards.elementAt(i).rank() == c.rank() && cards.elementAt(i).suit() == c.suit()) {
				cards.removeElementAt(i);
			}
		}
	}
	
	/**
	 * Extracts known cards from the deck.
	 */
	public void extractHand(Hand h) {
		for (int i=0; i<h.size(); i++) {
			extractCard(h.getCard(i));
		}
	}
	/**
	 * Gets the card in the given index.
	 * @param index
	 * @return The card in the given index:
	 */
	public Card getCard(int index) {
		return cards.elementAt(index);
	}
	
	/**
	 * Getter method of the size
	 * @return The size of the deck.
	 */
	public int getNumberOfCards() {
		return cards.size();
	}
	
	/**
	 * Gives a string representation of the deck.
	 */
	public String toString()
	{
		String sDeck = "";
		for(int i=0; i<cards.size(); i++)
			sDeck += (cards.elementAt(i) + " ");
		return sDeck;
	}

	private Vector<Card> cards;
	public static int primes[] = { 2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41 };
	public static final int NUM_CARDS = 52;
}
