package com.asdf.plugins.pokergames;

import java.util.LinkedList;
import java.util.List;

import com.asdf.common.PokerGameTypes;
import com.asdf.plugins.pokergames.PokerGame.Blind;
import com.asdf.plugins.pokergames.texasholdem.TexasHoldemStage;
import com.asdf.plugins.pokergames.texasholdem.parser.GameModelInfoMessage;
import com.asdf.plugins.pokergames.texasholdem.parser.ParserCard;
import com.asdf.plugins.pokergames.texasholdem.parser.ParserHand;
import com.asdf.plugins.pokergames.texasholdem.parser.PlayerInfoFieldsMessage;

/**
 * Contains all the state information of the game and players
 */
public class GameModel implements GameInfo, Cloneable {

	public static final String NIL_OBJECT = "nil_obj";
	public static final String RESERVE_OBJECT = "res_obj";
	
	private GameModelInfoMessage gmi;
	private Stage generatorStage;
	
	
	/**
	 * return the minimum amount to bring to this table
	 */
	public double minBuy() {
		return this.getBigBlindSize()*20;
	}
	
	/**
	 * return the maximum amount to bring to this table
	 */
	public double maxBuy() {
		return this.getBigBlindSize()*100;
	}
	
	/**
	 * TODO find out what testing
	 * @return the biggest bet
	 */
	public double getBiggestBet() {
		double bet = 0;
		for (int i = 0; i < this.getMaxPlayers(); i++)
			if (this.getPlayer(i) != null)
				if (this.getPlayer(i).getAmountInPotThisRound() > bet)
					bet = this.getPlayer(i).getAmountInPotThisRound();
		return bet;
	}
	
	/**
	 * removes history information from the players
	 */
	public void startHand() {
		for (int i = 0; i < this.getMaxPlayers(); i++) {
			if (this.getOurPlayer(i) != null)
				this.getOurPlayer(i).setLastGameInfo((GameModel)null);
		}
	}
	
	/**
	 * 
	 * @return returns the seat that would pay smallb lind, if all players in between are active
	 */
	public int getSmallBlindSeatFromButton() {
		if (this.getCountablePlayers() > 2 || !this.isReverseBlinds()) {
			return this.nextPlayer(this.getButtonSeat());
		}
		return this.getButtonSeat();
	}
	
	/**
	 * Sets the small blind seat
	 * @param seat
	 */
	public void setSmallBlindSeat(int seat) {
		this.gmi.setSmallBlindSeat(seat);
	}
	
	/**
	 * Sets the big blind seat
	 * @param seat
	 */
	public void setBigBlindSeat(int seat) {
		this.gmi.setBigBlindSeat(seat);
	}

	/**
	 * use to move the big blind seat, when the player who should pay refuses
	 */
	public void moveBigBlindSeat() {
		if (this.getCountablePlayers() > 2) 
			this.gmi.setBigBlindSeat(this.nextPlayer(this.gmi.getBigBlindSeat()));
	}

	/**
	 * use to move the big blind and small blind seats, when the player who should pay small blind refuses
	 */
	public void moveSmallBlindSeat() {
		if (this.getCountablePlayers() > 2) {
			this.gmi.setSmallBlindSeat(this.nextPlayer(this.gmi.getSmallBlindSeat()));
			this.gmi.setBigBlindSeat(this.nextPlayer(this.gmi.getBigBlindSeat()));
		}
	}
	
	/**
	 * 
	 * @return returns the seat that would pay big blind, if all players in between are active
	 */
	public int getBigBlindSeatFromButton() {
		if (this.getCountablePlayers() > 2 || !this.isReverseBlinds()) {
			return this.nextPlayer(this.nextPlayer(this.getButtonSeat()));
		}
		return this.nextPlayer(this.getButtonSeat());
	}
	
	/**
	 * Sets the given seat to be reserved
	 * @param seat 
	 * @return true if the seat was free and was reserved
	 */
	public boolean reserveSeat(int seat) {
		if (this.isReserved(seat))
			return false;
		else this.setPlayer(seat, new OurPlayerInfo(GameModel.RESERVE_OBJECT,GameModel.RESERVE_OBJECT, 0, seat, null).getInfoFields());
		//System.out.println("laitettiin varaus");
		return true;
	}
	
	//ACCESSORS, every field can be set with set<field name capitalized> and get with get<field name capitalized>
	
	/**
	 *  returns the fields of the model, for sending in messages
	 */
	public GameModelInfoMessage getGameModelInfo() {
		return this.gmi;
	}
	
	/**
	 * 
	 * @return maxplayers the maximum number of players allowed in the table
	 */
	public int getMaxPlayers() {
		return this.gmi.getPlayers().size();
	}
	/**
	 * 
	 * @param limitStyle set Fixed, Pot, or No Limit
	 */
	public void setLimitStyle(int limitStyle) {
		this.gmi.setLimitStyle(limitStyle);
	}
	
	/**
	 * 
	 * @return is this Fixed, Pot, or No Limit
	 */
	public int getLimitStyle() {
		return this.gmi.getLimitStyle();
	}

	/**
	 * set minRaise
	 * @param minRaise 
	 */
	public void setMinRaise(double minRaise) {
		this.gmi.setMinRaise(minRaise);
	}

	/**
	 * set minPlayers
	 * @param minPlayers
	 */
	public void setMinPlayers(int minPlayers) {
		this.gmi.setMinPlayers(minPlayers);
	}
	
	/**
	 * 
	 * @return minPlayers
	 */
	public int getMinPlayers() {
		return this.gmi.getMinPlayers();
	}
	
	/**
	 * set ante
	 * @param ante
	 */
	public void setAnte(int ante) {
		this.gmi.setAnte(ante);
	}

	public void freeSeat(int seat) {
		synchronized(this) {
			if (seat < this.gmi.getPlayers().size()) {
				this.gmi.getPlayers().remove(seat);
			}
			this.gmi.getPlayers().add(seat, new OurPlayerInfo(NIL_OBJECT, NIL_OBJECT, 0, seat, null).info);
		}
	}
	
	/**
	 * Removes all the players from the gameModel, and sets the given players infos in their stead
	 * @param players
	 */
	public void setPlayers(PlayerInfoFieldsMessage[] players) {
		this.gmi.getPlayers().clear();
		for (int i = 0; i < players.length; i++)
			if (players[i] != null)
				this.gmi.getPlayers().add(players[i]);
			else
				this.gmi.getPlayers().add(new OurPlayerInfo(NIL_OBJECT, NIL_OBJECT, 0, i, null).info);
	}
	/**
	 * Sets given players info to the given seat, is seat is taken, the previous player will be removed
	 * @param seat
	 * @param player
	 */
	public void setPlayer(int seat, PlayerInfoFieldsMessage player) {
		if (seat < 0 || seat >= this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		synchronized(this) {
			if (seat < this.gmi.getPlayers().size()) {
				this.gmi.getPlayers().remove(seat);
			}
			this.gmi.getPlayers().add(seat, player);
		}
	}
	
	public void setStartPlayers(int maxPlayers) {
		for (int i = 0; i < maxPlayers; i++)
			synchronized(this) {
				if (i < this.gmi.getPlayers().size()) {
					this.gmi.getPlayers().remove(i);
				}
				this.gmi.getPlayers().add(i, new OurPlayerInfo(NIL_OBJECT, NIL_OBJECT, 0, i, null).getInfoFields());
		}
	}
	
	/**
	 * Sets given players info to the given seat, is seat is taken, the previous player will be removed
	 * @param seat
	 * @param player
	 */
	public void setPlayer(int seat, OurPlayerInfo player) {
		this.setPlayer(seat, player.getInfoFields());
	}
	
	/**
	 * @return the players infos including reserving and null objects in the game as an array
	 */
	public PlayerInfoFieldsMessage[] getPlayers() {
		PlayerInfoFieldsMessage[] fields = new PlayerInfoFieldsMessage[this.gmi.getPlayers().size()];
		this.gmi.getPlayers().toArray(fields);
		for (int i = 0; i < fields.length; i++)
			if (fields[i] != null && (fields[i].getMyName() == RESERVE_OBJECT || fields[i].getMyName() == NIL_OBJECT))
				fields[i] = null;
		return fields;
	}

	/**
	 * Set the button seat
	 * @param buttonSeat
	 */
	public void setButtonSeat(int buttonSeat) {
		this.gmi.setButtonSeat(buttonSeat);
	}

	/**
	 * set the big blind
	 * @param bigBlind
	 */
	public void setBigBlind(double bigBlind) {
		this.gmi.setBigBlind(bigBlind);
	}
	
	/**
	 * 
	 * @return the big blind
	 */
	public double getBigBlind() {
		return this.gmi.getBigBlind();
	}
	
	/**
	 * set the small blind
	 * @param smallBlind
	 */
	public void setSmallBlind(double smallBlind) {
		this.gmi.setSmallBlind(smallBlind);
	}
	
	/**
	 * 
	 * @return the small blind
	 */
	public double getSmallBlind() {
		return this.gmi.getSmallBlind();
	}	
	
	/**
	 * Set the given hand as the board cards
	 * @param boardCards
	 */
	public void setBoardCards(Hand boardCards) {
		ParserHand hand = new ParserHand();
		ParserCard card;
		for (Card card1 : boardCards.getCards()) {
			card = new ParserCard();
			card.setCard(card1.getInt());
			hand.getCards().add(card);
		}
		this.gmi.setBoardCards(hand);
	}
	
	/**
	 * 
	 * @return the board cards
 	 */
	public Hand getBoardCards() {
		Hand hand = new Hand();
		for (ParserCard card : this.gmi.getBoardCards().getCards())
			hand.addCard(new Card(card.getCard()));
		return hand;
	}
	
	/**
	 * adds the given card to be the last in the board cards
	 * @param card1
	 */
	public void addCardToBoard(Card card1) {
		ParserCard card = new ParserCard();
		card.setCard(card1.getInt());
		this.gmi.getBoardCards().getCards().add(card);
	}
	
	/**
	 * Set the current player
	 * @param currentPlayerSeat
	 */
	public void setCurrentPlayerSeat(int currentPlayerSeat) {
		this.gmi.setCurrentPlayerSeat(currentPlayerSeat);
	}
	
	/**
	 * set the game ID
	 * @param gameID
	 */
	public void setGameID(long gameID) {
		this.gmi.setGameID(gameID);
	}
	
	/**TODO
	 * Set the number of players dealt in
	 * @param playersDealtIn
	 *
	public void setPlayersDealtIn(int playersDealtIn) {
		this.gmi.setPlayersDealtIn(playersDealtIn);
	}*/
	
	/**
	 * decrease the number of players dealt in by one
	 *
	public void decreasePlayersDealtIn() {
		this.gmi.setPlayersDealtIn(this.gmi.getPlayersDealtIn()-1);
	}
	
	/**
	 * increase the number of players dealt in by one
	 *
	public void increasePlayersDealtIn() {
		this.gmi.setPlayersDealtIn(this.gmi.getPlayersDealtIn()+1);
	}
	
	/**
	 * 
	 * @return the number of players dealt in
	 *
	public int getPlayersDealtIn() {
		return this.gmi.getPlayersDealtIn();
	}*/
	
	/**
	 * set the number of raises this round
	 * @param raisesThisRound
	 */
	public void setRaisesThisRound(int raisesThisRound) {
		this.gmi.setRaisesThisRound(raisesThisRound);
	}
	
	/**
	 * 
	 * @return the number of raises this round
	 */
	public int getRaisesThisRound() {
		return this.gmi.getRaisesThisRound();
	}

	/**
	 * increase the number of raises this round by one
	 */
	public void addRaisesThisRound(){
		this.gmi.setRaisesThisRound(this.getRaisesThisRound() +1);
	}
	
	/**
	 * set the pot
	 * @param pot
	 */
	public void setPot(double pot) {
		this.gmi.setPot(pot);
	}
	
	/**
	 * 
	 * @return the pot
	 */
	public double getPot() {
		return this.gmi.getPot();
	}
	
	public void addToPot(double amount) {
		this.gmi.setPot(this.gmi.getPot()+amount);
	}
	/*
	public void setMainpot(double mainpot) {
		this.gmi.setMainpot(mainpot);
	}
	public double getMainpot() {
		return this.gmi.getMainpot();
	}
	
	/*public void setSidepots(List<Sidepot> sidepots) {	//TODO removed methods
		this.gmi.setSidepots(sidepots);
	}
	public List<Sidepot> getSidepots() {
		return this.gmi.getSidepots();
	}*/

	/**
	 * Set the stage
	 * @param stage
	 */
	public void setStage(Stage stage) {
		this.gmi.setStage(stage.index());
	}
	
	/**
	 * 
	 * @return the current stage
	 */
	public Stage Stage() {
		return this.generatorStage.indexToStage(this.gmi.getStage());
	}
	
	/**
	 * set if the game is Simulation
	 * @param isSimulation
	 */
	public void setIsSimulation(boolean isSimulation) {
		this.gmi.setSimulation(isSimulation);
	}
	
	/**
	 * 
	 * @return if the game is simulation
	 */
	public boolean getIsSimulation() {
		return this.gmi.isSimulation();
	}
	
	/**
	 * set if the game is in zipMode
	 * @param isZipMode
	 */
	public void setIsZipMode(boolean isZipMode) {
		this.gmi.setZipMode(isZipMode);
	}
	
	/**
	 * 
	 * @return if the game is in zipMode
	 */
	public boolean getIsZipMode() {
		return this.gmi.isZipMode();
	}
	
	/**
	 * set if the heads-up is in reverseblinds order, by default it is
	 * @param reverseBlinds
	 */
	public void setReverseBlinds(boolean reverseBlinds) {
		this.gmi.setReverseBlinds(reverseBlinds);
	}
	
	/**
	 * 
	 * @return if it is reverse blinds
	 */
	public boolean getReverseBlinds() {
		return this.gmi.isReverseBlinds();
	}
	
	/**
	 * Get OurPlayerInfo from player in the given seat, to update his fields
	 * Use getPlayer if you just want to ask for the fields
	 * @param seat
	 * @return OurPlayerInfo
	 */
	public OurPlayerInfo getOurPlayer(int seat) {
		PlayerInfoFieldsMessage player = this.gmi.getPlayers().get(seat);
		if (player == null || player.getMyName() == null || player.getMyName().equals(GameModel.RESERVE_OBJECT) || player.getMyName().equals(GameModel.NIL_OBJECT))
			return null;
		return new OurPlayerInfo(this, player);
	}
	
	/**
	 * decrease the number of Players to act by one
	 */
	public void decreasePlayersToAct(){
		this.gmi.setPlayersToAct(this.gmi.getPlayersToAct()-1);
	}
	
	/**
	 * increase the number of Players to act to the maximum
	 */
	public void maxPlayersToAct(){
		this.gmi.setPlayersToAct(this.getNumActivePlayersNotAllIn());
	}
	/**
	 * set the number of players to act
	 * @param playersToAct
	 */
	public void setPlayersToAct(int playersToAct){
		this.gmi.setPlayersToAct(playersToAct);
	}
	
	/**
	 * increase the number of winners by one
	 */
	public void increaseWinners() {
		//TODO this.gmi.setWinners( this.gmi.getWinners() +1);
	}
	
	/**
	 * set the number of winners to zero
	 */
	public void zeroWinners() {
		//TODO this.gmi.setWinners(0);
	}
	
	/**
	 * 
	 * @return the number of winners;
	 */
	public int getWinners() {
		return 0;
		//TODO return this.gmi.getWinners();
	}
	
	/**
	 * Constructor for a new GameModel at the beginning of a table
	 * @param stage stage of the game
	 * @param smallBlind small blind for the hand
	 * @param bigBlind Big blind for the hand
	 * @param minPlayers minimum amount of required players
	 * @param maxPlayers maximum players at the table
	 * @param gameID ID for the first hand to be played
	 */
	public GameModel(double smallBlind, double bigBlind, int minPlayers, int maxPlayers, long gameID, Stage stage) {
		this.generatorStage = stage; //TODO some one who knows the gametype to set?
		this.gmi = new GameModelInfoMessage();
		this.setSmallBlind(smallBlind);
		this.setBigBlind(bigBlind);
		this.setMinPlayers(minPlayers);
		if (maxPlayers > 10)
			this.setStartPlayers(10);
		else
			this.setStartPlayers(maxPlayers);
		//this.setPlayers(new OurPlayerInfoFields[maxPlayers]);
		//this.hasToPostBlind = new Blind[maxPlayers];
		this.setGameID(gameID);
		this.setBoardCards(new Hand());
		this.setCurrentPlayerSeat(-1);
		this.setButtonSeat(-2);
	}
	
	public GameModel(GameModelInfoMessage gmi) {
		this.generatorStage = new TexasHoldemStage();//TODO some one who knows the gametype to set?
		this.gmi = gmi;
	}
	
	/**
	 * Little helper who changes this GameModel to have the same values as the given GameModel
	 * Saves that previous state of the model to the lastgameinfo of the previous currentplayer
	 * @param newModel GameModel to imitate 
	 * @return 1 if the current player has changed
	 *	       2 if the stage has changed, 0 otherwise
	 */
	public int update(GameModel newModel) {
		int event = 0;
		GameModel previous = clone(this);
		if (this.getCurrentPlayerSeat() != newModel.getCurrentPlayerSeat())
			event = 1;
		if (this.getStage().index() != newModel.getStage().index())
			event = 2;
		if (event != 0 && previous.getCurrentPlayerSeat() >= 0 && this.getOurPlayer(previous.getCurrentPlayerSeat()) != null)
			this.getOurPlayer(previous.getCurrentPlayerSeat()).setLastGameInfo(previous);
		this.gmi = newModel.gmi;
		
		/*
		this.minRaise = newModel.minRaise;
		this.minPlayers = newModel.minPlayers;
		this.ante = newModel.ante;
		for (int i = 0; i < this.players.length; i++) {
			if (newModel.getPlayer(i) != null && newModel.getPlayer(i).getRevealedHand() != null 
					&& this.getPlayer(i) != null && this.getPlayer(i).getRevealedHand() == null)
				event = PlayerStates.playerRevealed(event, i);
		}
		this.players = newModel.players;
		this.buttonSeat = newModel.buttonSeat;
		this.bigBlind = newModel.bigBlind;
		this.smallBlind = newModel.smallBlind;
		this.boardCards = new Hand(newModel.boardCards);
		this.currentBetSize = newModel.currentBetSize;
		this.currentPlayerSeat = newModel.currentPlayerSeat;
		if (this.gameID != newModel.gameID) {
			this.gameID = newModel.gameID;
			event = PlayerStates.newHand(event);
		}
		this.playersDealtIn = newModel.playersDealtIn;
		this.raisesThisRound = newModel.raisesThisRound;
		this.pot = newModel.pot;
		this.mainpot = newModel.mainpot;
		this.sidepots = newModel.sidepots;
		if (this.stage != newModel.stage) {
			this.stage = newModel.stage;
			event = PlayerStates.newStage(event);
		}
		this.isSimulation = newModel.isSimulation;
		this.isZipMode = newModel.isZipMode;
		this.reverseBlinds  = newModel.reverseBlinds;
		return event;*/
		return event;
	}
	
	/**
	 * Little helper who creates an independent copy of given GameModel
	 * @param previous 
	 * @return the clone of given GameModel
	 */
	public static GameModel clone(GameModelInfoMessage previous) {
		GameModelInfoMessage gmim = new GameModelInfoMessage();
		gmim.setAnte(previous.getAnte());
		gmim.setBigBlind(previous.getBigBlind());
		gmim.setBoardCards(previous.getBoardCards());
		gmim.setButtonSeat(previous.getButtonSeat());
		gmim.setCurrentPlayerSeat(previous.getCurrentPlayerSeat());
		gmim.setGameID(previous.getGameID());
		gmim.setLimitStyle(previous.getLimitStyle());
		//gmim.setMainpot(previous.getMainpot());
		gmim.setMinPlayers(previous.getMinPlayers());
		gmim.setMinRaise(previous.getMinRaise());
		for (PlayerInfoFieldsMessage player : previous.getPlayers())
			gmim.getPlayers().add(player);
		//gmim.setPlayersDealtIn(previous.getPlayersDealtIn());
		gmim.setPlayersToAct(previous.getPlayersToAct());
		gmim.setPot(previous.getPot());
		gmim.setRaisesThisRound(previous.getRaisesThisRound());
		gmim.setReverseBlinds(previous.isReverseBlinds());
		gmim.setSimulation(previous.isSimulation());
		gmim.setSmallBlind(previous.getSmallBlind());
		gmim.setStage(previous.getStage());
		gmim.setZipMode(previous.isZipMode());
		gmim.setSmallBlindSeat(previous.getSmallBlindSeat());
		gmim.setBigBlindSeat(previous.getBigBlindSeat());
		return new GameModel(gmim);
		/*
		next.minRaise = previous.minRaise;
		next.ante = previous.ante;
		next.players = previous.players.clone();
		next.buttonSeat = previous.buttonSeat;
		next.boardCards = new Hand(previous.boardCards);
		next.currentBetSize = previous.currentBetSize;
		next.currentPlayerSeat = previous.currentPlayerSeat;
		next.playersDealtIn = previous.playersDealtIn;
		next.raisesThisRound = 0;
		next.pot = previous.pot;
		next.mainpot = previous.mainpot;
		next.sidepots = previous.sidepots;
		next.isSimulation = previous.isSimulation;
		next.isZipMode = previous.isZipMode;
		next.stage = previous.stage;
		next.reverseBlinds = previous.reverseBlinds;
		return next;
		*/
	}
	public static GameModel clone(GameModel previous) {
		return clone(previous.gmi);
	}
	
	public static GameModel cloneWithoutPreviousStates(GameModel previous) {
		GameModel model = clone(previous);
		for (int i = 0; i < model.getMaxPlayers(); i++) {
			if (model.getOurPlayer(i) != null)
				model.getOurPlayer(i).setLastGameInfo((GameModel)null);
		}
		return model;
	}
	
	
	/**
	 * Little helper to update a GameModel for the next round of the hand
	 * @param nextStage the stage to go to
	 * @return GameModel for the beginning of next round
	 *
	public GameModel nextRound(Stage nextStage) {
		this.raisesThisRound = 0;
		this.stage = nextStage;
		return this;
	}*/
	
	/**
	 * Little helper to create a next hand with the same players and values as the previous one
	 * @param previous GameModel to have the next hand for
	 * @param gameID ID for the next hand
	 * @return new GameModel with the same values as the previous one
	 *
	public static GameModel nextHand(GameModel previous, long gameID) {
		GameModel next = new GameModel(previous.smallBlind, previous.bigBlind, previous.minPlayers, previous.players.length, previous.gameID);
		next.minRaise = previous.minRaise;
		next.ante = previous.ante;
		next.players = previous.players;
		//next.buttonSeat = previous.nextPlayer(previous.buttonSeat);
		next.boardCards = new Hand(previous.boardCards);
		next.currentBetSize = previous.currentBetSize;
		next.currentPlayerSeat = previous.currentPlayerSeat;
		next.playersDealtIn = 0;
		for (int i = 0; i < next.players.length; i++)
			if (next.getPlayer(i) != null)
				next.playersDealtIn++;
		next.raisesThisRound = 0;
		next.pot = previous.pot;
		next.mainpot = previous.mainpot;
		next.sidepots = previous.sidepots;
		next.isSimulation = previous.isSimulation;
		next.isZipMode = previous.isZipMode;
		//next.stage = Stage.WAIT_PLAYERS;
		next.reverseBlinds = previous.reverseBlinds;
		return next;
	}
	*/
	
	/**
	 * @return the number of players in the table
	 */
	public int getSittingPlayers() {
		int num = 0;
		for (int i = 0; i < this.getMaxPlayers(); i++) {
			if (this.getPlayer(i) != null)
				num++;
		}
		return num;
	}
	
	//INTERFACE METHODS
	@Override
	/**
	 * From GameInfo, see if the player in the given seat is able to raise
	 * Calls the PlayerInfo's corresponding method.
	 * @return true if the player has enough chips to raise after calling
	 * @param seat position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 */
	public boolean canRaise(int seat) {
		if (seat<0 || seat>=this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		if (this.getPlayer(seat) == null)
			throw new EmptySeatException(seat);
		return this.getPlayer(seat).hasEnoughToRaise();
	}

	@Override
	/**
	 * from GameInfo, get a player's amount to call
	 * Calls the PlayerInfo's corresponding method.
	 * @return the amount given player has to call
	 * @param seat position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 */
	public double getAmountToCall(int seat) {
		if (seat<0 || seat>=this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		if (this.getPlayer(seat) == null)
			throw new EmptySeatException(seat);
		return this.getPlayer(seat).getAmountToCall();
	}

	@Override
	/**
	 * from GameInfo, get the amount of the ante, if any
	 * @return the value of ante
	 */
	public double getAnte() {
		return this.gmi.getAnte();
	}

	@Override
	/**
	 * from GameInfo, get a player's bankroll
	 * Calls the PlayerInfo's corresponding method.
	 * @param seat position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 * @return the given player's bankroll
	 */
	public double getBankRoll(int seat) {
		if (seat<0 || seat>=this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		if (this.getPlayer(seat) == null)
			throw new EmptySeatException(seat);
		return this.getPlayer(seat).getBankRoll();
	}

	//@Override
	/**
	 * from GameInfo, get the maximum amount the player in the given seat can risk in this hand.
	 * Calls the PlayerInfo's corresponding method.
	 * @param seat position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 *
	public double getBankRollAtRisk(int seat) {
		if (seat<0 || seat>=this.getMaxPlayers())
			throw new InvalidSeatException();
		if (this.getPlayer(seat) == null)
			throw new EmptySeatException();
		return this.getPlayer(seat).getBankRollAtRisk();
	}*/

	@Override
	/**
	 * from GameInfo, get a player's amount to call in bets
	 * @param seat position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 */
	public double getBetsToCall(int seat) {
		if (seat<0 || seat>=this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		if (this.getPlayer(seat) == null)
			throw new EmptySeatException(seat);
		return this.getAmountToCall(seat)/this.getCurrentBetSize();
	}

	@Override
	/**
	 * from GameInfo, get the big blind seat
	 * @return the big blind seat
	 */
	public int getBigBlindSeat() {
		return this.gmi.getBigBlindSeat();
	}

	@Override
	/**
	 * from GameInfo, get the amount of the big blind.
	 * @return the amount of the big blind
	 */
	public double getBigBlindSize() {
		return this.getBigBlind();
	}

	@Override
	/**
	 * from GameInfo, get the board cards
	 * @return new hand with the board cards
	 */
	public Hand getBoard() {
		return new Hand(this.getBoardCards());
	}

	@Override
	/**
	 * from GameInfo, get the seat the button is on (could be dead button)
	 * @return button seat
	 */
	public int getButtonSeat() {
		return this.gmi.getButtonSeat();
	}

	@Override
	/**
	 * from GameInfo, get the amount of the betting increment (for fixed-limit)
	 * @return current bet size, nothing relevant if not fixed-limit
	 */
	public double getCurrentBetSize() {
		return this.getMinRaise();
	}

	@Override
	/**
	 * from GameInfo, get the seat for the current player to act
	 * @return the seat for the current player to act
	 */
	public int getCurrentPlayerSeat() {
		return this.gmi.getCurrentPlayerSeat();
	}

	@Override
	/**
	 * from GameInfo, get the amount in the pot that a player is eligible to win.
	 * @param seat position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 * @return the amount the player is eligible to win
	 */
	public double getEligiblePot(int seat) {
		if (seat<0 || seat>=this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		if (this.getPlayer(seat) == null)
			throw new EmptySeatException(seat);
		if (!this.getPlayer(seat).isActive())
			return 0;
		double amount = 0;
		for (int i = 0; i < this.getMaxPlayers(); i++)
			if (this.getPlayer(i) != null)
				if (this.getPlayer(i).getAmountInPot() <= this.getPlayer(seat).getAmountInPot())
					amount += this.getPlayer(i).getAmountInPot();
				else
					amount += this.getPlayer(seat).getAmountInPot();
		return amount;
	}

	@Override
	/**
	 * from GameInfo, the game identifier for this table
	 * @return the gameID
	 */
	public long getGameID() {
		return this.gmi.getGameID();
	}

	//@Override
	/**
	 * from GameInfo, get the Log Directory used by all players for keeping logs
	 * @return the log directory
	 * TODO what? how do we log?
	 *
	public String getLogDirectory() {
		return null;
	}*/

	@Override
	/**
	 * from GameInfo, get the size of the main pot.
	 * @return the main pot
	 */
	public double getMainPotSize() {
		return this.getPot();
	}

	@Override
	/**
	 * from GameInfo, get the smallest legal raise
	 * @return the minimum raise
	 */
	public double getMinRaise() {
		return this.gmi.getMinRaise();
	}

	@Override
	/**
	 * from GameInfo, get the number of players still active (not folded) in the hand
	 * @return the number of active players
	 */
	public int getNumActivePlayers() {
		int number = 0;
		for (int i = 0; i < this.getMaxPlayers(); i++) 
			if (this.getPlayer(i) != null && this.getPlayer(i).isActive())
				number++;
		return number;
	}

	@Override
	/**
	 * from GameInfo, get the number of active players that are not all-in
	 * @return the number of players not all-in
	 */
	public int getNumActivePlayersNotAllIn() {
		int number = 0;
		for (int i = 0; i < this.getMaxPlayers(); i++) 
			if (this.getPlayer(i) != null && this.getPlayer(i).isActive() && !this.getPlayer(i).isAllIn())
				number++;
		return number;
	}

	@Override
	/**
	 * from GameInfo, get the number of players dealt into the hand
	 * @return the number of players dealt in to the hand
	 */
	public int getNumPlayers() {
		int num = 0;
		for (int i = 0; i < this.getMaxPlayers(); i++)
			if (this.getPlayer(i) != null && this.getPlayer(i).inGame())
				num++;
		return num;
	}

	@Override
	/**
	 * from GameInfo, get the number of bets and raises made during the current betting round
	 * @return the number of raises made during current round
	 */
	public int getNumRaises() {
		return this.getRaisesThisRound();
	}

	@Override
	/**
	 * from GameInfo, the number of seats at the table
	 * @return the number of seats
	 */
	public int getNumSeats() {
		return this.getMaxPlayers();
	}

	//@Override
	/**
	 * from GameInfo, get the number of side pots
	 * @return the number of side pots
	 */
	/*public int getNumSidePots() { //TODO removed
		return this.getSidepots().toArray().length;
	}*/

	@Override
	/**
	 * from GameInfo, get the number of players still to act in this betting round if there are no further raises.
	 * @return the number of players to act
	 */
	public int getNumToAct() {
		return this.gmi.getPlayersToAct();
	}

	@Override
	/**
	 * from GameInfo, get the number of players that were awarded a pot in this hand
	 * @return the number of winners
	 */
	public int getNumWinners() {
		return this.getWinners();
	}

	@Override
	/**
	 * from GameInfo, get the number of players in the hand that are all-in
	 * @the number of players all-in
	 */
	public int getNumberOfAllInPlayers() {
		int number = 0;
		for (int i = 0; i < this.getMaxPlayers(); i++) 
			if (this.getPlayer(i) != null && this.getPlayer(i).isActive() && this.getPlayer(i).isAllIn())
				number++;
		return number;
	}

	@Override
	/**
	 * from GameInfo, get state information for a player in a given seat
	 * @param seat position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @return PlayerInfo for given player, or null if seat is empty
	 */
	public PlayerInfo getPlayer(int seat) {
		if (seat<0 || seat>=this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		
		if(this == null) {
			throw new RuntimeException("JUMALAUTA NYT LASKI TUMMUU\"!!");
		}
		
		if (/*this.gmi == null || 
				this.gmi.getPlayers() == null || 
				this.gmi.getPlayers().get(seat) == null ||
				this.gmi.getPlayers().get(seat).getMyName() == null ||*/
				this.gmi.getPlayers().get(seat).getMyName().equals(GameModel.RESERVE_OBJECT) || 
				this.gmi.getPlayers().get(seat).getMyName().equals(GameModel.NIL_OBJECT))
			return null;
		return new OurPlayerInfo(this, this.gmi.getPlayers().get(seat));
	}

	@Override
	/**
	 * from GameInfo, get state information for a player with a given name
	 * @param name the name of desired player
	 * @return PlayerInfo for given player, or null if not found
	 */
	public PlayerInfo getPlayer(String name) {
		for (int i = 0; i < this.getMaxPlayers(); i++)
			if (this.getPlayer(i) != null && this.getPlayer(i).getName().equals(name))
				return this.getPlayer(i);
		return null;
	}

	@Override
	/**
	 * from GameInfo, get the name of the player in a given seat
	 * @param seat position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 * @return the name of the player in the seat
	 */
	public String getPlayerName(int seat) {
		if (seat<0 || seat>=this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		if (this.getPlayer(seat) == null)
			throw new EmptySeatException(seat);
		return this.getPlayer(seat).getName();
	}

	@Override
	/**
	 * from GameInfo, get the seat number for a player by name
	 * @return the seat the player is in or -1 if not found
	 */
	public int getPlayerSeat(String name) {
		for (int i = 0; i < this.getMaxPlayers(); i++)
			if (this.getPlayer(i) != null && this.getPlayer(i).getName().equals(name))
				return i;
		return -1;
	}

	@Override
	/**
	 * from GameInfo, get a list of all players eligible to win a pot with the given minimum stakes
	 * @param amount the amount a player must have in the pot to be eligible to win it
	 * @return a list of all players eligible to win the pot
	 */
	public List<PlayerInfo> getPlayersInPot(double amount) {
		List<PlayerInfo> winners = new LinkedList<PlayerInfo>();
		for (int i = 0; i < this.getMaxPlayers(); i++)
			if (this.getPlayer(i) != null && this.getPlayer(i).isActive() && (this.getPlayer(i).getAmountInPot() >= amount))
					winners.add(this.getPlayer(i));
		return winners;
	}

	//@Override
	/**
	 * from GameInfo, Get the amount raked from pot so far, not implemented
	 * @return 0
	 * TODO do we need these?
	public double getRake() {
		return 0;
	}
	*/
	//@Override
	/*
	 * from GameInfo, get the size of the side pot amount.
	 * @throws IndexOutOfBoundsException
	 * @param i the side pot index [0..n-1]
	 * @return the size of the ith side pot.
	 */
	/*public double getSidePotSize(int i) {
		if (i >= this.getNumSidePots())
			throw new IndexOutOfBoundsException();
		return this.getSidepots().get(i).getAmount();
	}*/

	@Override
	/**
	 * from GameInfo, get the small blind seat (could be dead small blind)
	 * @return the small blind seat
	 */
	public int getSmallBlindSeat() {
		return this.gmi.getSmallBlindSeat();
	}

	@Override
	/**
	 * from GameInfo, get the amount of the small blind
	 * @return small blind size
	 */
	public double getSmallBlindSize() {
		return this.getSmallBlind();
	}

	@Override
	/**
	 * from GameInfo, get the current game stage
	 * @return the current stage
	 */
	public Stage getStage() {
		return this.generatorStage.indexToStage(this.gmi.getStage());
	}

	@Override
	/**
	 * from GameInfo, The stakes are the amount each player must have invested in the pot to stay active in the hand.
	 * @return the stakes
	 */
	public double getStakes() {
		double amount = 0;
		for (int i = 0; i < this.getMaxPlayers(); i++)
			if (amount < this.getPlayer(i).getAmountInPot())
				amount = this.getPlayer(i).getAmountInPot();
		return amount;
	}

	@Override
	/**
	 * from GameInfo, get the total size of the pot including the main pot and all side pots.
	 * @return total pot size
	 * 
	 */
	public double getTotalPotSize() {
		return this.getPot();
	}

	@Override
	/**
	 * from GameInfo, get the number of players who have not acted yet this round
	 * @return the number of players not yet acted 
	 */
	public int getUnacted() {
		return this.gmi.getPlayersToAct();
	}

	@Override
	/**
	 * from GameInfo, See if the player in the given seat is dealt into the hand (not sitting out or empty).
	 * @param seat position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @return true if the player is dealt in
	 */
	public boolean inGame(int seat) {
		if (seat<0 || seat >= this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		if (this.getPlayer(seat) != null && this.getPlayer(seat).inGame())
			return true;
		return false;
	}

	@Override
	/**
	 * from GameInfo, see if the player in the given seat is still active in the hand
	 * @param seat position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 * @return true if player is active
	 */
	public boolean isActive(int seat) {
		if (seat<0 || seat >= this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		if (this.getPlayer(seat) == null)
			throw new EmptySeatException(seat);
		return this.getPlayer(seat).isActive();
	}

	@Override
	/**
	 * from GameInfo, see if a player has invested anything in the current round
	 * @param seat position of the player
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @throws EmptySeatException, if the given seat is empty
	 * @return true if the player has invested
	 */
	public boolean isCommitted(int seat) {
		if (seat<0 || seat >= this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		if (this.getPlayer(seat) == null)
			throw new EmptySeatException(seat);
		return this.getPlayer(seat).isCommitted();
	}

	@Override
	/**
	 * from GameInfo, determine if the game is fixed-limit
	 * @return true if the game is fixed-limit
	 */
	public boolean isFixedLimit() {
		return PokerGameTypes.values()[this.getLimitStyle()].isFixedLimit();
	}

	//@Override
	/**
	 * from GameInfo, See if it is currently flop
	 * @return true if it is flop
	 */
	/*
	public boolean isFlop() {
		return (this.stage == Stage.FLOP || this.stage == Stage.BET_ROUND_FLOP);
	}
	*/
	
	//@Override
	/**
	 * from GameInfo, See if the hand is over
	 * @return true if the hand is over
	 */
	/*public boolean isGameOver() {
		// TODO Hand can be over when all others fold, or after showdown?
		return this.stage == Stage.SHOWDOWN || this.getNumActivePlayers() == 0;
	}
	*/

	@Override
	/**
	 * from GameInfo, determine if the game is no-limit
	 * @return true if the game is no-limit
	 */
	public boolean isNoLimit() {
		return PokerGameTypes.values()[this.getLimitStyle()].isNoLimit();
	}

	//@Override
	/**
	 * from GameInfo, See if it is currently the post-flop
	 * @return true if it is post-flop
	 */
	/*public boolean isPostFlop() {
		return !this.isPreFlop();
	}*/

	@Override
	/**
	 * from GameInfo, determine if the game is pot-limit
	 * @return true if the game is pot-limit
	 */
	public boolean isPotLimit() {
		return PokerGameTypes.values()[this.getLimitStyle()].isPotLimit();
	}

	//@Override
	/**
	 * from GameInfo, See if it is currently the pre-flop
	 * @return true if it is pre-flop
	 */
	/*
	public boolean isPreFlop() {
		return this.stage.ordinal() < Stage.FLOP.ordinal();
	}
	*/
	@Override
	/**
	 * from GameInfo, In reverse blinds mode, if the game is Heads-up, the small blind is placed on the button.
	 * @return if we are using the reverse blinds rule for heads up
	 */
	public boolean isReverseBlinds() {
		return true;
		//TODO return this.getReverseBlinds();
	}

	//@Override
	/**
	 * from GameInfo, see if it is currently the river round
	 * @return true if it is the river
	 */
	/*
	public boolean isRiver() {
		return this.stage == Stage.RIVER || this.stage == Stage.BET_ROUND_RIVER;
	}
	*/
	@Override
	/**
	 * from GameInfo, determine if the game is in simulation mode. Simulations are high speed games used to determine approximate values of particular hand scenarios In simulation mode, players may choose to play faster, using cruder approximations, and should avoid all use of disk.
	 * @return true if game is in simulation mode
	 */
	public boolean isSimulation() {
		return this.gmi.isSimulation();
	}

	//@Override
	/**
	 * from GameInfo, see if it is currently the turn round
	 * @return true if it is the turn
	 */
	/*
	public boolean isTurn() {
		return (this.stage == Stage.TURN || this.stage == Stage.BET_ROUND_TURN);
	}
	*/
	@Override
	/**
	 * from GameInfo, we should play as fast and crudely as possible because the user has requested we zip to the end of the hand.
	 * @return true is it is Zip mode
	 */
	public boolean isZipMode() {
		return this.gmi.isZipMode();
	}

	@Override
	/**
	 * from GameInfo, get the seat number of the next active player at the table (skips empty seats and folded players)
	 * @param seat position
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @return the seat number of the next active player
	 */
	public int nextActivePlayer(int seat) {
		if (seat<0 || seat >= this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		if (this.getNumActivePlayers() < 2)
			return -1;
		int nextSeat = seat;
		do {
			nextSeat = this.nextSeat(nextSeat);
		} while (this.getPlayer(nextSeat) == null || !this.getPlayer(nextSeat).isActive() || seat == nextSeat);
		return nextSeat;
	}

	@Override
	/**
	 * from GameInfo, get the seat number of the next player at the table (skips empty seats)
	 * @param seat position
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @return seat number of the next player
	 */
	public int nextPlayer(int seat) {
		if (seat<0 || seat >= this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		int nextSeat = seat;
		do {
			nextSeat = this.nextSeat(nextSeat);
		} while (this.getPlayer(nextSeat) == null);
		return nextSeat;
	}

	@Override
	/**
	 * from GameInfo, get the seat number of the next seat clockwise at the table
	 * @param seat position
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @return the next seat number
	 */
	public int nextSeat(int seat) {
		if (seat<0 || seat >= this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		int nextSeat = seat+1;
		if (nextSeat == this.getMaxPlayers())
			nextSeat = 0;
		return nextSeat;
	}

	@Override
	/**
	 * from GameInfo, get the seat number of the previous player at the table (skips empty seats)
	 * @param seat position
	 * @throws InvalidSeatException, if the table does not have such a seat
	 * @return seat number of previous player
	 */
	public int previousPlayer(int seat) {
		if (seat<0 || seat >= this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		int previousSeat = seat;
		do {
			if (previousSeat == 0)
				previousSeat = this.getMaxPlayers();
			previousSeat--;
		} while (this.getPlayer(previousSeat) == null);
		return previousSeat;
	}
	
	public void setNextPlayer() {
		this.setCurrentPlayerSeat(this.nextPlayer(this.getCurrentPlayerSeat()));
	}
	
	public void setNextActivePlayer() {
		this.setCurrentPlayerSeat(this.nextActivePlayer(this.getCurrentPlayerSeat()));
	}
	
	public PlayerInfo getCurrentPlayer() {
		if (this.getCurrentPlayerSeat() < 0 || this.getCurrentPlayerSeat() >= this.getMaxPlayers())
			return null;
		return this.getPlayer(this.getCurrentPlayerSeat());
	}
	
	
	//TODO what counts for heads-up? maxplayers? sitting? active?
	public int getCountablePlayers() {
		return this.getSittingPlayers();
	}
	
	public void activatePlayer(int player) {
		this.getOurPlayer(player).setState(PlayerStates.activated(this.getOurPlayer(player).getState()));
	}

	@Override
	public Blind hasToPostBlind(int seat) {
		return this.getPlayer(seat).hasToPostBlind();
	}

	@Override
	public boolean isReserved(int seat) {
		if (seat<0 || seat >= this.getMaxPlayers())
			throw new InvalidSeatException(seat);
		return this.gmi.getPlayers().get(seat) != null && !this.gmi.getPlayers().get(seat).getMyName().equals(NIL_OBJECT);
	}

/*	private double highest(int[] list) {
		int max = 0;
		for (int i = 0; i < list.length; i++)
			if (max < list[i])
				max = list[i];
		return max;
	}
	*/
	public static  void main(String[] args) {
		GameModel game = new GameModel(1 /*small blind*/, 2 /*big blind*/, 2 /*min players*/, 10 /*maxplayers*/, 0 /*id*/, new TexasHoldemStage());
		PlayerInfoFieldsMessage[] players;//= game.getPlayers();
		
		game.reserveSeat(5);

			
		OurPlayerInfo ville = new OurPlayerInfo("Ville", "Ville",10000, 1, game);
		OurPlayerInfo kalle = new OurPlayerInfo("Kalle", "Kalle", 100, 2, game);
		game.setPlayer(2, kalle);
		game.setPlayer(1, ville);
		game.setPlayer(6, new OurPlayerInfo("Liisa", "Liisa",10000, 6, game));
		players= game.getPlayers();
		game.setButtonSeat(2);
		System.out.println(game.getBigBlindSeat());
		System.out.println(game.getSmallBlindSeat());
		for (int i = 0; i < players.length; i++)
			if (players[i] != null)
				System.out.println(players[i].getMyName());
			else 
				System.out.println("null");
		ville.setState(PlayerStates.sitOut(0));
		for (int i = 0; i < game.getMaxPlayers(); i++)
			if (game.getPlayer(i) != null && !game.getPlayer(i).isSittingOut()) {
					game.getOurPlayer(i).setState(PlayerStates.activated(game.getOurPlayer(i).getState()));
					//this.gameModel.increasePlayersDealtIn();
			}
		kalle.setBankRoll(0);
		//System.out.println(game.getNumActivePlayersNotAllIn());
		//System.out.println(game.getPlayer(6).getBankRoll());
		game.getOurPlayer(6).setBankRoll(2);
		//System.out.println(game.getPlayer(6).getBankRoll());
		//System.out.println(game.getPlayer("Liisa").getBankRoll());
		//System.out.println(game.getPlayerName(2));
		//System.out.println(game.getPlayerSeat("Liisa"));
		kalle.setInPot(100);
		//System.out.println(game.getPlayersInPot(101).size());
		//System.out.println(game.inGame(3));
		game.setPlayer(0, new OurPlayerInfo("Ilkka", "Ilkka", 100, 0, game));
		game.getOurPlayer(0).setState(PlayerStates.activated(0));
		//System.out.println(game.nextActivePlayer(9));
		System.out.println(game.nextPlayer(6));
		System.out.println(game.nextSeat(2));
		System.out.println(game.previousPlayer(1));
		ville.setBankRoll(0);
		game.getOurPlayer(0).setBankRoll(0);
		//kalle.setState(PlayerStates.folded(ville.getState()));
		//System.out.println(game.getNumberOfAllInPlayers());
		game.getOurPlayer(6).setInPot(10);
		System.out.println(game.getEligiblePot(2));
	}


}
