package com.asdf.plugins.pokergames;

import java.util.ArrayList;

import com.asdf.plugins.pokergames.texasholdem.parser.ParserCard;
import com.asdf.plugins.pokergames.texasholdem.parser.ParserHand;

/**
 * container for cards
 * @author rimpila
 *
 */

public class Hand {

	private ArrayList<Card> cards;
	
	/**
	 * constructor for an empty hand
	 */
	public Hand() {
		cards = new ArrayList<Card>();
	}
	
	/**
	 * 
	 * @param crd card asked for
	 * @return true id hand contains the asked card
	 */
	public boolean containsCard(Card crd) {
		for(int i=0; i<this.size(); i++)
			if(crd.getInt() == this.getCard(i).getInt())
				return true;
		return false;
	}
	
	/**
	 * constructor
	 * @param info
	 */
	public Hand(ParserHand info){
		this();
		if (info == null || info.getCards() == null)
			return;
		for (ParserCard card : info.getCards())
			this.addCard(new Card(card.getCard()));
	}
	
	/**
	 * Constructor for copying a hand
	 * @param hand
	 */
	public Hand(Hand hand){
		cards = new ArrayList<Card>();
		for (Card card : hand.getCards())
			cards.add(card);
	}
	
	/**
	 * add given card to the hand
	 * @param card1 card to add
	 */
	public void addCard(Card card1) {
		cards.add(card1);
	}
	
	/**
	 * 
	 * @param pos
	 * @return card in the given position in the hand
	 */
	public Card getCard(int pos) {
		return this.cards.get(pos);
	}
	
	/**
	 * 
	 * @return number of cards in the hand
	 */
	public int size(){
		return this.cards.size();
	}
	
	/**
	 * 
	 * @return list of cards
	 */
	public ArrayList<Card> getCards(){
		return this.cards;
	}
	/**
	 * Put the given list of cards to be this hand
	 * @param cards
	 */
	public void setCards(ArrayList<Card> cards){
		this.cards = cards;
	}
	
	public void removeCard() {
		removeCard(size()-1);
	}
	
	/**
	 * remove card from the given index
	 * @param index
	 */
	public void removeCard (int index) {
		this.cards.remove(index);
	}
	
}
