package com.asdf.plugins.pokergames;

import com.asdf.common.GameController;
import com.asdf.common.Action;

/**
 * class implementing Player interface for human players to send action choices through
 * @author rimpila
 *
 */
public class OurPlayer implements Player {
	
	protected PokerGameController gameController; //set by constructor, to get GameModel
	//protected GameInfo gameInfo;
	protected String myName;
	protected int mySeat;
	protected Hand myHand;
	
	//TODO
	public OurPlayer(String name, int mySeat ,PokerGameController gameController) {
		this.mySeat = mySeat;
		this.myName = name;
		this.gameController = gameController;
	}
	/*
	public void setGameModel(GameModel update){
		this.gameInfo = update;
	}
	*/
	//TODO remove?
	public void setGameController(PokerGameController gameController) {
		this.gameController = gameController;
	}
	public GameController getGameController() {
		return this.gameController;
	}
	
	public Hand getHand() {
		return this.myHand;
	}
	
	
	//INTERFACE
	@Override
	
	/**
	 * @param pos (int) the position of the player who made the action
	 * @param action (Action) the action made
	 * This should be called when any one makes an action, 
	 * but Ourplayer will not probably be using this
	 */
	public void actionEvent(int pos, Action action) {
		;
		//may override to get each action made by players
	}

	@Override
	/**
	 * From GameObserver, called when hole cards are dealt.
	 */
	public void dealHoleCardsEvent() {
			;
	}

	
	@Override
	/**
	 * From GameObserver, called when the hand is over.
	 * Resets the Player fields concerning one hand to
	 */
	public void gameOverEvent() {
		this.myHand = null;
	}

	@Override
	/**
	 * @param arg0 (GameInfo) all the public information of the starting hand
	 * (i.e. our GameModel object)
	 * From GameObserver, called at the start of hand. 
	 * Sets the fields to start values.
	 */
	public void gameStartEvent(GameInfo arg0, int seat) {
		/*
		this.gameInfo = arg0;
		this.bankRollAtStartOfHand = this.bankRoll;
		this.inPotThisRound = 0;
		this.inPot = 0;
		this.lastAction = -1;
		this.holeCards = null;
		this.myState = PlayerStates.inGame(myState);
		*/

	}

	@Override
	/**
	 * From GameObserver, called whenever an action is completed, and the GameModel has been updated
	 */

	public void gameStateChanged() {
			;
	}

	@Override
	/**
	 * @return Action generated from the GUI given action value (int)
	 * This is called when it is the players turn to act.
	 * Gets the desired action from the GUI, 
	 * may sleep the thread while waiting to get the values, so should be called in a separate thread
	 * and updates the fields of OurPlayer accordingly
	 * at this point asks also for the amount, but that is only needed for Raise, other values
	 * can be calculated
	 */
	
	
	public Action getAction() {
		//this.lastGameInfo = GameModel.clone(this.gameController.newGameModel());
		//this.gameController.setWaitingObject();
		//System.out.println("Halutaan action");
		int myAction;
		double amount;
		this.gameController.waitForNextAction();

		synchronized(this) {
			myAction = this.gameController.getNextAction();
			amount = this.gameController.getNextAmount();
			this.gameController.zeroActions();
		}
		//System.out.println("Saatiin action" + myAction);
		switch (myAction) {
		case Action.ALLIN_PASS:
			return Action.allInPassAction();
		case Action.BET:
			amount = this.gameController.gameInfo.getCurrentBetSize();
			/*this.bankRoll -= amount;
			this.inPot += amount;
			this.inPotThisRound += amount;
			this.hasActedThisRound = true;
			this.myState = PlayerStates.committed(myState);*/
			return Action.betAction(amount);
		case Action.BIG_BLIND:
			//amount = this.gameController.gameInfo.getBigBlindSize();
			/*this.bankRoll -= amount;
			this.inPot += amount;
			this.inPotThisRound += amount;*/
			return Action.bigBlindAction();
		case Action.CALL:
			//amount = this.gameController.gameInfo.getPlayer(this.mySeat).getAmountCallable();
			/*this.bankRoll -= amount;
			this.inPot += amount;
			this.inPotThisRound += amount;
			this.lastAction = myAction;
			this.hasActedThisRound = true;
			this.myState = PlayerStates.committed(myState);*/
			return Action.callAction();
		case Action.CHECK:
			//this.hasActedThisRound = true;
			return Action.checkAction();
		case Action.FOLD:
			/*this.myState = PlayerStates.folded(myState);
			this.lastAction = myAction;
			this.hasActedThisRound = true;*/
			return Action.foldAction();
		case Action.MUCK:
			/*this.myState = PlayerStates.folded(myState);
			this.holeCards = new Hand();*/
			return Action.muckAction();
		case Action.POST_ANTE:
			amount = this.gameController.gameInfo.getAnte();
			/*this.bankRoll -= amount;
			this.inPot += amount;
			this.inPotThisRound += amount;*/
			return Action.postAnte(amount);
		case Action.POST_BLIND:
		case Action.POST_DEAD_BLIND:
			//if (this.mySeat == this.gameController.gameInfo.getBigBlindSeat())
			//	amount = this.gameController.gameInfo.getBigBlindSize();
			//else  
			//	amount = this.gameController.gameInfo.getPlayer(this.mySeat).hasToPostBlind().toPay(this.gameController.gameInfo.getSmallBlindSize(), this.gameController.gameInfo.getBigBlindSize());
			//if (this.mySeat == this.gameController.gameInfo.getSmallBlindSeat() && amount < this.gameController.gameInfo.getSmallBlindSize())
			//	amount = this.gameController.gameInfo.getSmallBlindSize();
			/*this.bankRoll -= amount;
			this.inPot += amount;
			this.inPotThisRound += amount;*/
			return Action.postBlindAction();
		case Action.RAISE:
			/*this.bankRoll -= amount;
			this.inPot += amount;
			this.inPotThisRound += amount;
			this.lastAction = myAction;
			this.hasActedThisRound = true;
			this.myState = PlayerStates.committed(myState);*/
			return Action.raiseAction(amount);
		case Action.SIT_OUT:
			//this.myState = PlayerStates.sitOut(myState);
			return Action.sitout();
		case Action.SMALL_BLIND:
			//amount = this.gameController.gameInfo.getSmallBlindSize();
			/*this.bankRoll -= amount;
			this.inPot += amount;
			this.inPotThisRound += amount;*/
			return Action.postBlindAction();
		case Action.BACK:
			return Action.backAction();
		case Action.DISCARD:
			int[] discarded = new int[1]; //TODO add functionality to controller for getting discards...
			return Action.discardAction(discarded);
		//case Action.JOIN_GAME:
		//case Action.LEAVE_GAME:
		//case Action.RESERVE_SEAT:
		case Action.REVEAL:
			return Action.revealAction();
		case Action.SKIP_TURN:
			return Action.skipTurnAction();
			
		}
		
		return null;
	}

	//@Override
	/*
	 * from Player, receive your hole cards
	 * @param card1 first hole card
	 * @param card2 second hole card
	 * @param your seat in this round
	 */
	/*public void holeCards(Card card1, Card card2, int seat) {
		mySeat = seat;
		myHand = new Hand();
		myHand.addCard(card1);
		myHand.addCard(card2);
		System.out.println("OurPlayer sai kortit");
		//myState = PlayerStates.activated(myState);
	}*/
	
	@Override
	/**
	 * from Player, receive your hand
	 * @param hand 
	 * @param your seat in this round
	 */
	public void receiveHand(Hand hand, int seat) {
		myHand = new Hand(hand);
		//myState = PlayerStates.activated(myState);
	}

	
	

	@Override
	/**
	 * from GameObserver, Player pos has shown two cards.
	 * @param pos the showing player
	 * @param c1 first card
	 * @param c2 second shown card
	 */
	public void showdownEvent(int pos, Card c1, Card c2) {
			;
	}

	@Override
	/**
	 * from GameObserver, a new stage (betting round) has begun. A player may override this method to receive the event The stage is a constant such as Holdem.PREFLOP, Holdem.FLOP, etc...
	 * @param the stage
	 */
	public void stageEvent(int arg0) {
		/*this.hasActedThisRound = false;
		this.inPotThisRound = 0;*/

	}

	@Override
	/**
	 * from GameObserver, a player at seat has won amount with the hand handName
	 * @param seat the player
	 * @param amount the amount won
	 * @param hand the winning hand
	 */
	public void winEvent(int seat, double amount, Hand hand) {

	}	
	
	public static void main(String[] args) {
		//OurPlayer kalle = new OurPlayer("Kalle", 500, null/*new Kokeilu()*/);
		//OurPlayer ville = new OurPlayer("Ville", 500, null);
/*		System.out.println("kalle BankRoll: " + kalle.bankRoll);
		System.out.println("kalle:" + kalle.bankRollAtStartOfHand);
		System.out.println("kalle" + kalle.hasActedThisRound);
		System.out.println("kalle" + kalle.inPot);
		System.out.println("kalle" + kalle.inPotThisRound);
		System.out.println("kalle" + kalle.lastAction);
		System.out.println("kalle" + kalle.myName);
		System.out.println("kalle" + kalle.mySeat);
		System.out.println("kalle" + kalle.myState);*/
		//System.out.println("kalle" + kalle.getAmountCallable());
		/*System.out.println("kalle amountInPot: " + kalle.getAmountInPot());
		System.out.println("kalle AmountInPotThisRound: " + kalle.getAmountInPotThisRound());
		System.out.println("kalle" + kalle.getAmountRaiseable());
		System.out.println("kalle" + kalle.getAmountRaiseable());
		System.out.println("kalle" + kalle.getAmountToCall());
		System.out.println("kalle" + kalle.getBankRoll());
		System.out.println("kalle" + kalle.getBankRollAtRisk());
		System.out.println("kalle" + kalle.getBankRollAtStartOfHand());
		System.out.println("kalle" + kalle.getBankRollInSmallBets());
		System.out.println("kalle" + kalle.getHasActedThisRound());
		System.out.println("kalle" + kalle.getInPot());
		System.out.println("kalle" + kalle.getInPotThisRound());
		System.out.println("kalle" + kalle.getLastAction());
		System.out.println("kalle" + kalle.getName());
		System.out.println("kalle" + kalle.getNetGain());
		double amountToRaise = 10;
		System.out.println("kalle" + kalle.getRaiseAmount(amountToRaise));
		System.out.println("kalle" + kalle.getSeat());
		System.out.println("kalle" + kalle.getState());
		System.out.println("kalle" + kalle.hasActedThisRound());
		System.out.println("kalle" + kalle.getInPot());
		System.out.println("kalle" + kalle.hasEnoughToRaise());
		System.out.println("kalle" + kalle.getInPotThisRound());
		System.out.println("kalle" + kalle.inGame());
		System.out.println("kalle" + kalle.isActive());
		System.out.println("kalle" + kalle.isAllIn());
		System.out.println("kalle" + kalle.isButton());
		System.out.println("kalle" + kalle.isCommitted());
		System.out.println("kalle" + kalle.isFolded());
		System.out.println("kalle" + kalle.isSittingOut());
		System.out.println("kalle" + kalle.gameController);
		System.out.println("kalle" + kalle.gameInfo);
		System.out.println("kalle" + kalle.holeCards);
		System.out.println("kalle" + kalle.lastGameInfo);
		System.out.println("kalle" + kalle.gameInfo());
		System.out.println("kalle" + kalle.getAction());
		System.out.println("kalle" + kalle.getGameController());
		System.out.println("kalle" + kalle.getGameInfo());
		System.out.println("kalle" + kalle.getHoleCards());
		System.out.println("kalle" + kalle.getRevealedHand());
		int pos = 1; Action action = new Action(pos, amountToRaise, amountToRaise);
		kalle.actionEvent(pos, action);
		kalle.dealHoleCardsEvent();
		kalle.gameOverEvent();
		kalle.gameStartEvent(null);
		kalle.gameStateChanged();
		Card card1 = new Card(1,1), card2 = new Card(2, 2); int seat = 1;
		kalle.holeCards(card1, card2, seat);
		kalle.showdownEvent(pos, card1, card2);
		int arg0 = 0;
		kalle.stageEvent(arg0);
		double amount = 100; String hand = "1";
		kalle.winEvent(seat, amount, hand);
		*/
	}
	
}
