package com.asdf.plugins.pokergames;

import com.asdf.plugins.pokergames.PokerGame.Blind;
import com.asdf.plugins.pokergames.texasholdem.parser.GameModelInfoMessage;
import com.asdf.plugins.pokergames.texasholdem.parser.ParserBlind;
import com.asdf.plugins.pokergames.texasholdem.parser.ParserCard;
import com.asdf.plugins.pokergames.texasholdem.parser.ParserHand;
import com.asdf.plugins.pokergames.texasholdem.parser.PlayerInfoFieldsMessage;

/**
 * Our implementation for PlayerInfo
 * @author rimpila
 *
 */
public class OurPlayerInfo implements PlayerInfo {

	/*
	protected String myName; //set by constructor
	protected int mySeat; //set by something
	protected Hand holeCards = null; //updated when player reveals, otherwise set to blank cards	
	protected double bankRoll = 0; //set with creator, updated with wins and actions
	protected GameInfo lastGameInfo; //set with actions something?		
	protected int myState = 0; //updated with actions, starts by being in game, and active
	protected double inPot = 0; //updated with actions
	protected double bankRollAtStartOfHand = 0; //updated with actions and zeroed at beginning
	protected double inPotThisRound = 0; //updated with actions	
	protected int lastAction; //updated with actions
	protected boolean hasActedThisRound; //updated with actions
	
	private Blind hasToPostBlind;
	*/
	
	protected GameInfo gameInfo; //set by set	
	protected PlayerInfoFieldsMessage info;	
	
	//ACCESSORS, every field can be set with set<field name capitalized> and get with get<field name capitalized>
	//EXCEPT gameInfo, as getGameInfo is defined in the PlayerInfo interface to be the state of the players last action
	
	public PlayerInfoFieldsMessage getInfoFields() {
		return this.info;
	}
	
	public void setGameInfo(GameInfo gameInfo) {
		this.gameInfo = gameInfo;
	}
	
	//not "get" because it is used in the interface PlayerInfo for a different purpose
	public GameInfo gameInfo() {
		return this.gameInfo;
	}
	
	public void setLastGameInfo(GameModelInfoMessage info) {
		this.info.setLastGameInfo(info);
	}
	public void setLastGameInfo(GameModel info) {
		if (info != null)
			this.info.setLastGameInfo(info.getGameModelInfo());
		else
			this.info.setLastGameInfo((GameModelInfoMessage)null);
	}
	
	public void setAccountName(String accountName){
		info.setAccountName(accountName);
	}
	public void setName(String myName) {
		this.info.setMyName(myName);
	}

	public void setSeat(int seat) {
		this.info.setMySeat(seat);
	}

	public void setState(int state) {
		this.info.setMyState(state);
	}
	public int getState() {
		return this.info.getMyState();
	}

	public void setBankRoll(double bankRoll) {
		this.info.setBankRoll(bankRoll);
	}

	public void increaseBankRoll(double amount) {
		this.info.setBankRoll(this.getBankRoll()+amount);
	}
	
	public void decreaseBankRoll(double amount) {
		if(this.getBankRoll() - amount < -0.1)
			throw new RuntimeException("Paid more than has money!! @ TexasHoldem.OurPlayerInfo.decreaseBankRoll");
		this.info.setBankRoll(this.getBankRoll()-amount);
	}
	
	public void setInPot(double inPot) {
		this.info.setInPot(inPot);
	}
	public void increaseInPot(double amount) {
		this.info.setInPot(this.getInPot()+amount);
	}
	
	public double getInPot() {
		return this.info.getInPot();
	}
	
	public void setBankRollAtStartOfHand(double bankRollAtStartOfHand) {
		this.info.setBankRollAtStartOfHand(bankRollAtStartOfHand);
	}
	
	public void setInPotThisRound(double inPotThisRound) {
		this.info.setInPotThisRound(inPotThisRound);
	}
	public void increaseInPotThisRound(double amount) {
		this.info.setInPotThisRound(this.getInPotThisRound()+ amount);
	}
	public double getInPotThisRound() {
		return this.info.getInPotThisRound();
	}
	
	public void setHoleCards(Hand holeCards) {
		if (holeCards != null) {
			ParserHand hand = new ParserHand();
			ParserCard card;
			
			for (Card card1 : holeCards.getCards()) {
				card = new ParserCard();
				card.setCard(card1.getInt());
				hand.getCards().add(card);
			}
			this.info.setHoleCards(hand);
		} else
			this.info.setHoleCards(null);
	}
	public Hand getHoleCards() {
		if (this.info.getHoleCards() == null)
			return null;
		Hand hand = new Hand();
		for (ParserCard card : this.info.getHoleCards().getCards())
			hand.addCard(new Card(card.getCard()));
		return hand;
		//TODO return new Hand(this.info.getHoleCards());
	}
	
	public void setLastAction(int lastAction) {
		this.info.setLastAction(lastAction);
	}
	
	public void setHasActedThisRound(boolean hasActedThisRound) {
		this.info.setHasActedThisRound(hasActedThisRound);
	}
	public boolean getHasActedThisRound() {
		return this.info.isHasActedThisRound();
	}
	public Blind hasToPostBlind() {
		return Blind.values()[this.info.getHasToPostBlind().ordinal()];
	}
	public void setHasToPostBlind(Blind amount) {
		this.info.setHasToPostBlind(com.asdf.plugins.pokergames.texasholdem.parser.ParserBlind.fromValue(amount.toString()));
	}
	
	
	//CONSTRUCTORS
	
	public OurPlayerInfo(String name, String AccountName, double bankRoll, int mySeat, GameInfo gameInfo2) {
		if(name == null)
			name = GameModel.NIL_OBJECT;
		this.info = new PlayerInfoFieldsMessage();
		this.info.setMyName(name);
		this.info.setAccountName(AccountName);
		this.info.setBankRoll(bankRoll);
		this.info.setMySeat(mySeat);
		this.gameInfo = gameInfo2;
		this.info.setHasToPostBlind(ParserBlind.fromValue("NO_BLINDS"));
		this.setHoleCards(null);
	}
	
	
	
	public OurPlayerInfo(GameInfo gameInfo, PlayerInfoFieldsMessage info) {
		this.gameInfo = gameInfo;
		this.info = info;

	}

	/**
	 * for cloning players for deep copies
	 * @param previous
	 */
	public OurPlayerInfo(OurPlayerInfo previous) {
		if (previous == null)
			return;
		this.info = previous.info;
		this.gameInfo = previous.gameInfo;
	}

	/**
	 * Returns the account name of this player.
	 * @return Account name.
	 */
	@Override
	public String getAccountName() {
		return info.getAccountName();
	}
	
	@Override
	/**
	 * from PlayerInfo, get the amount of the current bet that the player can call
	 * @return how much the player can pay of the amount he should call
	 */
	public double getAmountCallable() {
		return ((this.getBankRoll() < this.getAmountToCall()) ?
				this.getBankRoll(): this.getAmountToCall());
	}

	@Override
	/**
	 * from PlayerInfo, obtain the amount the player has invested in the pot.
	 * @return the amount the player has in the pot of the current hand
	 */
	public double getAmountInPot() {
		return this.getInPot();
	}

	@Override
	/**
	 * from PlayerInfo, obtain the amount the player has invested in the pot this round
	 * @return the amount the player has invested in the pot this round
	 */
	public double getAmountInPotThisRound() {
		return this.getInPotThisRound();
	}

	@Override
	/**
	 * from PlayerInfo, get the maximum amount the player can raise.
	 * @return the amount the player will be able to raise,
	 * thus, either the upper limit for raises, or the total bankroll
	 */
	public double getAmountRaiseable() {
		if (this.gameInfo.isNoLimit()) {
			return this.getBankRoll()-this.getAmountToCall();
		}
		else if (this.gameInfo.isPotLimit()) {
			return (this.getBankRoll()-this.getAmountToCall() < this.gameInfo.getMainPotSize() ?
					this.getBankRoll()-this.getAmountToCall() : this.gameInfo.getMainPotSize());
		} else {
			if (this.gameInfo.getNumRaises() >= 4)
				return 0;
			else
			return (this.getBankRoll()-this.getAmountToCall() < this.gameInfo.getCurrentBetSize() ?
					this.getBankRoll()-this.getAmountToCall() : this.gameInfo.getCurrentBetSize());
		}
	}

	@Override
	/**
	 * from PlayerInfo, determine the amount a player must pay to stay in the game
	 * @return the difference between the player who has bet most in the pot, and this players amount in the pot
	 */
	public double getAmountToCall() {
		int previous = info.getMySeat();
		double toCall = 0;
		double helper = 0;
		if (this.gameInfo.getNumActivePlayers() <= 1) {
			return 0;
		}
		while ((previous = this.gameInfo.previousPlayer(previous)) != info.getMySeat()) {
			if ((this.gameInfo.getPlayer(previous)) != null)
				if ((helper = this.gameInfo.getPlayer(previous).getAmountInPot() - this.getAmountInPot()) > toCall)
					toCall = helper;
		}
		return toCall;
	}

	@Override
	/**
	 * from PlayerInfo, Get the current bankroll
	 * @return the amount of money the player has in his disposal in the table
	 */
	public double getBankRoll() {
		return this.info.getBankRoll();
	}

	@Override
	/**
	 * from PlayerInfo, get the player's bankroll at the start of the hand (i.e., before any actions including antes and blinds).
	 * @return the amount the player had in his bankroll before any antes, blinds or bets
	 */
	public double getBankRollAtStartOfHand() {
		return this.info.getBankRollAtStartOfHand();
	}

	@Override
	/**
	 * from PlayerInfo, get the current bankroll as a multiple of the big blind
	 * @return the players bankroll divided by the size of the big blind
	 */
	public double getBankRollInSmallBets() {
		return this.getBankRoll()/this.gameInfo.getBigBlindSize();
	}

	@Override
	/**
	 * from PlayerInfo, get the context of the last action made by this player.
	 * @return the GameInfo saved when doing latest action 
	 */
	public GameInfo getGameInfo() {
		return new GameModel(this.info.getLastGameInfo());
	}

	@Override
	/**
	 * from PlayerInfo, a single integer code for the last action made
	 * @return one of [Holdem.FOLD, Holdem.CALL, Holdem.RAISE]
	 */
	public int getLastAction() {
		return this.info.getLastAction();
	}

	@Override
	/**
	 * from PlayerInfo, obtain the player's name
	 * @return the name of the player
	 */
	public String getName() {
		return info.getMyName();
	}

	@Override
	/**
	 * from PlayerInfo, get the net amount won or lost since the start of the current hand
	 * @return the difference between current bankroll and the bankroll at the start of hand
	 */
	public double getNetGain() {
		return this.getBankRoll()-this.getBankRollAtStartOfHand();
	}

	@Override
	/**
	 * from PlayerInfo, return the amount the player can raise, given the desired amount to raise.
	 * @param the amount desired to raise
	 * @return the amount player is able to raise given the desired
	 */
	public double getRaiseAmount(double amountToRaise) {
		return (this.getAmountRaiseable() < amountToRaise ? this.getAmountRaiseable() : amountToRaise);
	}

	@Override
	/**
	 * from PlayerInfo, obtain the hand revealed by this player
	 * @return the players hole cards, if he has revealed them, otherwise 2 invalid cards, if he has two, otherwise empty hand
	 */
	public Hand getRevealedHand() {
		if (this.getHoleCards() == null)
			return null;
		return new Hand(this.getHoleCards());
	}

	@Override
	/**
	 * from PlayerInfo, get the player's seat number in the GameInfo
	 * @return the players seat number
	 */
	public int getSeat() {
		return this.info.getMySeat();
	}

	@Override
	/**
	 * from PlayerInfo, check if a player has made at least one action this round
	 * @return true if the player has acted this round 
	 */
	public boolean hasActedThisRound() {
		return this.getHasActedThisRound();
	}

	@Override
	/**
	 * from PlayerInfo, check if the player has enough chips to complete a raise
	 * @return true if the player has enough for minimum raise after paying what he has to call
	 */
	public boolean hasEnoughToRaise() {
		return (this.getBankRoll()-this.getAmountCallable() > this.gameInfo.getMinRaise());
	}
	
	@Override
	/**
	 * from PlayerInfo, see if the player is dealt into the current game
	 * @return true if the player is dealt in
	 */
	public boolean inGame() {
		return (PlayerStates.isInGame(this.getState()));
	}
	
	@Override
	/**
	 *  from PlayerInfo, check if the player is still active in the hand
	 *  @return true if the player is active in current hand
	 */
	public boolean isActive() {
		return (PlayerStates.isActive(this.getState()));
	}

	@Override
	/**
	 * from PlayerInfo, check if the player is all-in
	 * @return true if the player is all-in
	 */
	public boolean isAllIn() {
		return (this.isActive() && this.getBankRoll() <= 0);
	}

	@Override
	/**
	 * from PlayerInfo, check if the player is currently the button
	 * @return true if the player is the button
	 */
	public boolean isButton() {
		return (this.gameInfo.getButtonSeat() == this.getSeat());
	}
	
	@Override
	/**
	 * from PlayerInfo, check if the player has voluntarily committed this round
	 * @return true if the player has committed
	 */
	public boolean isCommitted() {
		return (PlayerStates.isCommitted(this.getState()));
	}

	@Override
	/**
	 * from PlayerInfo, check if the player has folded
	 * @return true if the player has folded
	 */
	public boolean isFolded() {
		return (PlayerStates.isFolded(this.getState()));
	}

	@Override
	/**
	 * from PlayerInfo, check if the player is sitting out
	 * @return true if the player is sitting out
	 */
	public boolean isSittingOut() {
		return (PlayerStates.isSittingOut(this.getState()));
	}

	
	public static void main(String[] args) {
		GameModel game = new GameModel(1 /*small blind*/, 2 /*big blind*/, 2 /*min players*/, 10 /*maxplayers*/, 0 /*id*/, null);
		OurPlayerInfo ville = new OurPlayerInfo("Ville","Ville", 10000, 1, game);
		OurPlayerInfo kalle = new OurPlayerInfo("Kalle","Kalle", 100, 2, game);
		
		game.setPlayer(1, ville);
		game.setPlayer(2, kalle);
		game.setPlayer(3, new OurPlayerInfo("Liisa","Liisa", 1000, 3, game));
		//System.out.println(kalle.getState());
		game.activatePlayer(1);
		//System.out.println(kalle.getState());
		game.activatePlayer(2);
		//System.out.println(kalle.getState());
		kalle.setInPot(200);
		ville.setInPot(50);
		game.getOurPlayer(3).setInPot(500);
		System.out.println(ville.getAmountToCall());
		System.out.println(ville.getAmountCallable());
		game.setPot(250);
		game.setLimitStyle(0);
		System.out.println(ville.getAmountRaiseable());
		game.setMinRaise(10);
		//System.out.println(ville.getBankRollAtRisk());
		//System.out.println(ville.getBankRollInSmallBets());
		ville.setBankRollAtStartOfHand(1000);
		//System.out.println(ville.getNetGain());
		//System.out.println(ville.getRaiseAmount(10000000));
		//ville.setBankRoll(100);
		//kalle.setInPot(100000000);
		//System.out.println(ville.hasEnoughToRaise());
		//ville.setBankRoll(0);
		//System.out.println(ville.isAllIn());
		//System.out.println(ville.isButton());
		game.setButtonSeat(1);
		//System.out.println(ville.isButton());
	}
	

}
