package com.asdf.plugins.pokergames;

import com.asdf.common.Action;

/**
 * Interface for players and bots to implement for receiving updates and to be asked for 
 * actions. Method gameStartEvent is called at the beginning of hand, and it includes all the public info
 * of the hand in progress and seat for the player. The bot also reaches the public information as kept by the server through the info.
 * 
 * @author rimpila
 *
 */
public interface Player {

	/**
	 * @return Action the players next action, with desired amount
	 * This is called when it is the players turn to act.
	 */
		public Action getAction() ;
			
			 
		
		/*
		 * from Player, receive your hole cards
		 * @param card1 first hole card
		 * @param card2 second hole card
		 * @param your seat in this round
		 */		
		//public void holeCards(Card card1, Card card2, int seat) ;
			

		/**
		 * @param pos (int) the position of the player who made the action
		 * @param action (Action) the action made
		 * This should be called when any one makes an action
		 */
		public void actionEvent(int pos, Action action) ;
			

		

		/**
		 * From GameObserver, called when hole cards are dealt.
		 */
		public void dealHoleCardsEvent() ;
			

		

		/**
		 * Called when the hand is over.
		 */
		public void gameOverEvent() ;
			

		

		/**
		 * @param arg0 (GameInfo) all the public information of the starting hand
		 * (i.e. our GameModel object)
		 * Called at the start of hand.
		 */
		public void gameStartEvent(GameInfo arg0, int seat) ;
			

		

		/**
		 * From GameObserver, called whenever an action is completed, and the GameModel has been updated
		 */
		public void gameStateChanged() ;
			

		

		/**
		 * from GameObserver, Player pos has shown two cards. 
		 * @param arg0 the showing player
		 * @param arg1 first card
		 * @param arg2 second shown card
		 */
		public void showdownEvent(int arg0, Card arg1, Card arg2) ;
			

		

		/**
		 * a new stage has begun. A player may override this method to receive the event 
		 * for texas hold'em the stage is a constant from TexasHoldemEnumStage
		 * @param stage
		 */
		public void stageEvent(int stage) ;
			

		

		/**
		 * from GameObserver, a player at seat has won amount
		 * @param seat the player
		 * @param amount the amount won
		 */
		public void winEvent(int seat, double amount, Hand hand) ;




		/**
		 * Receive your hand, and your seat, this is called when the player is dealt his hand
		 * @param hand 
		 * @param seat your seat in this round
		 */
		void receiveHand(Hand hand, int seat);
		
}
			

		

	

