package com.asdf.plugins.pokergames;

/**Note: this class has a natural ordering that is inconsistent with equals.
 * Class for collecting information of the winnings for each player*/

public class PlayerPosition implements Comparable<PlayerPosition>{

	private int seat;
	private double amountInPot;
	private BestHand handValue;
	private int winnings = 0;
	
	/**
	 * Constructor
	 * @param seat seat of the player
	 * @param amountInPot amount the player had in the pot
	 * @param handValue hand value as calculated by handEval
	 */
	public PlayerPosition(int seat, double amountInPot, BestHand handValue) {
		this.seat = seat;
		this.amountInPot = amountInPot;
		this.handValue = handValue;
	}

	@Override
	/**
	 * Compare to another playerPosition, first according to hand values, then amount in pot,
	 * and last with seats, to make equals consistent
	 */
	public int compareTo(PlayerPosition o) {
		if (this.handValue.value > o.handValue.value) //hand has better value
			return 1;
		if (this.handValue.value < o.handValue.value) //hand has worse value
			return -1;
		if (this.amountInPot > o.amountInPot) //hands are equal, but the amount in pot is less
			return 1;						  //the player with less in pot will be handled first
		if (this.amountInPot < o.amountInPot)
			return -1;
		if (this.seat > o.seat)
			return 1;
		if (this.seat < o.seat)
			return -1;
		return 0;
	}

	/**
	 * @return the seat
	 */
	public int getSeat() {
		return seat;
	}

	/**
	 * @return the amountInPot
	 */
	public double getAmountInPot() {
		return amountInPot;
	}

	/**
	 * @return the handValue
	 */
	public int getHandValue() {
		if (this.handValue != null)
			return handValue.value;
		return -1;
	}
	
	/**
	 * 
	 * @return integer table of five cards, out of seven, the positions of the cards that make the best possible hand
	 */
	public int[] getWinningCardsPositions() {
		if (this.handValue != null)
			return this.handValue.cards;
		return null;
	}
	
	/**
	 * adds  given amount to the player's winnings
	 * @param amount
	 */
	public void addWinnings(double amount) {
		this.winnings += amount;
	}
	
	/**
	 * 
	 * @return the winnings
	 */
	public double getWinnings() {
		return this.winnings;
	}
	
	
}
