package com.asdf.plugins.pokergames;

import java.io.IOException;
import java.util.List;

import javax.swing.SwingUtilities;
import javax.xml.bind.JAXBException;

import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import com.asdf.client.ConnectionHandler;
import com.asdf.common.GameController;
import com.asdf.common.AccountType;
import com.asdf.common.Action;
import com.asdf.parser.ChatMessage;
import com.asdf.parser.Custom;
import com.asdf.parser.GameRequest;
import com.asdf.parser.GameResponse;
import com.asdf.parser.Message;
import com.asdf.parser.OptionalParameters;
import com.asdf.parser.Request;
import com.asdf.parser.Response;
import com.asdf.parser.StateUpdate;
import com.asdf.plugins.pokergames.gameview.GameView;
import com.asdf.plugins.pokergames.texasholdem.parser.ActionMessage;
import com.asdf.plugins.pokergames.texasholdem.parser.TexasMessage;
import com.asdf.utils.XMLTools;


/**
 * Provides control for receiving game updates and forwarding them to the players and GUI
 * and sending actions to server
 */
	
public abstract class PokerGameController extends GameController{
	
	public GameInfo gameInfo; //to give references to
	protected GameModel gameModel; //to update as needed, same as gameInfo
	GameModel newGameModel; //where the new model received from server is stored	
	
	private XMLTools xmltoolsTex;
	private XMLTools xmltoolsMsg;
	

	ConnectionHandler connection;	
	protected Player[] players;
	private OurPlayer[] ourPlayers;
	int mainPlayerSeat = -1;
	private int nextAction = -1;
	private double nextAmount = 0;
	private boolean hasNextAction = false;
	protected GameView gameView;
	//private boolean[][] options; //0 for sitting out, 1 for big blind, 2 for small blind, 3 for autopostblind
								 //4 for muck winning hands, 5 for muck losing hands;
	private boolean reserving = false;
	
	protected int ACTION_WAIT = 15000;
	protected String accountName;
	protected Hand[] superUserHands;
	protected boolean firstHand = true;
	protected boolean windowOpen = true;
	private boolean gameOver;
	
	/**
	 * Getter for myself.
	 * @return me.
	 */
	public PokerGameController getThis() {
		return this;
	}
	
	/**
	 * Returns the name of the account for the user
	 */
	public String getAccountName() {
		return this.accountName;
	}
	
	/**
	 * Getter for players
	 * @return All players participating in the game.
	 */
	public Player[] getPlayers() {
		return this.players;
	}
	
	/**
	 * Getter for the main seat player.
	 * @return the human controlled player's seat.
	 */
	public int getOurPlayer() {
		return this.mainPlayerSeat;
	}
	

	/**
	 * Getter for current game state
	 * @return Model of current game state
	 */
	public GameInfo getGameInfo() {
		return this.gameModel;
	}
	
	/**
	 * Waits for the next action.
	 */
	public synchronized void waitForNextAction() {
		if (!this.getHasNextAction()) {
			try {
				this.wait(this.ACTION_WAIT+1000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Setter for has an action to process.
	 * @param has with value 'true' awakens waiting position to start handling actions.
	 */
	public synchronized void setHasNextAction(boolean has) {
		this.hasNextAction = has;
		this.notify(); //TODO something to kill the rest waiting
	}
	
	public synchronized boolean getHasNextAction() {
		return this.hasNextAction;
	}
	
	/**
	 * Setter for the next action to be processed.
	 * @param nextAction Integer description of the action.
	 * @param amount Cash amounts related to action, meaning depends on the value of action.
	 */
	public void setNextAction(int nextAction, double amount) {
		this.setNextAmount(amount);
		this.setNextAction(nextAction);
		
	}
	
	/**
	 * Setter for action.
	 * @param nextAction Integer description of the action.
	 */
	public void setNextAction(int nextAction) {
		this.nextAction = nextAction;
		this.setHasNextAction(true);
		/*if (this.hasObjectWaiting);
			this.notify();*/
	}
	
	/**
	 * Setter for nextAmount
	 * @param amount Cash amounts related to nextAction, meaning depends on the value of action.
	 */
	public void setNextAmount(double amount) {
		this.nextAmount = amount;
	}
	
	
	/**
	 * Clears all current action information.
	 */
	
	public void zeroActions(){
		this.nextAction = -1;
		this.nextAmount = 0;
		//this.hasObjectWaiting = false;
		this.setHasNextAction(false);

	}
	
	/**
	 * Getter for next action.
	 * @return integer description of the next action.
	 */
	public int getNextAction(){
		/*if (this.options[1][this.gameInfo.getCurrentPlayerSeat()])
			return Action.BIG_BLIND;
		if (this.options[2][this.gameInfo.getCurrentPlayerSeat()])
			return Action.SMALL_BLIND;*/
		return this.nextAction;
	}
	
	/**
	 * Getter for next amount.
	 * @return Cash amounts related to the action. Meaning depends on the value of action.
	 */
	
	public double getNextAmount() {
		return this.nextAmount;
	}
		
	/**
	 * Updates game information for a GUI instance
	 */
	public void updateGUI() {
		if (this.gameView != null) {
			/*Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	gameView.update(getGameInfo());		    
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);*/
//			Runnable doWorkRunnable = new Runnable() {
//				public void run() {
					gameView.update(getGameInfo());
//				}
//			};
//			try {
//	            SwingUtilities.invokeAndWait(doWorkRunnable);
//	        }
//	        catch (Exception e) {
//	            e.printStackTrace();
//	        }

		}
	}
	
	/**
	 * Updates game information and turn related information. Called only on inter-player turn changes.
	 */
	public void updateGUIBigger() {
		if (this.gameView != null) {
			/*Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	gameView.update(getThis(), playerCanSit());		    
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);*/
//			Runnable doWorkRunnable = new Runnable() {
//				public void run() {
					gameView.update(getThis(), playerCanSit());
//				}
//			};
//			try {
//	            SwingUtilities.invokeAndWait(doWorkRunnable);
//	        }
//	        catch (Exception e) {
//	            e.printStackTrace();
//	        }
		}
	}
	
	/**
	 * Calls GUI to update available seat positions
	 */
	public void updateSeatAvailable() {
		if (this.gameView != null) {
			/*Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	gameView.update(playerCanSit(), getThis());		    
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);*/
			if (!SwingUtilities.isEventDispatchThread()) {
				Runnable doWorkRunnable = new Runnable() {
					public void run() {
						gameView.update(playerCanSit(), getThis());
					}
				};
				try {
		            SwingUtilities.invokeAndWait(doWorkRunnable);
		        }
		        catch (Exception e) {
		            e.printStackTrace();
		        }
			}
			else gameView.update(playerCanSit(), getThis());
		}
	}
	
	/**
	 * Checks if a player can sit on this table.
	 * @return false if there already is a human in the table
	 */
	protected boolean playerCanSit() {
		if (this.accountName == null || this.accountName.length() <= 0 )
			return false;
		for (int i = 0; i < this.ourPlayers.length; i++)
			if (this.ourPlayers[i] != null)
				return false;
		return !this.reserving;
	}
	
	public PokerGameController() {}

	
	/**
	 * Updates the game model given the new game model
	 * If the current player is one in this controller, starts a thread to get it's action
	 * @param newModel A new game model.
	 */
	
	public void updateGameModel(GameModel newModel) {
		this.newGameModel = newModel;
		if (this.hasNextAction) //should clear lingering actions, that for some reason were not sent
			this.zeroActions();
		int changeTurn = this.gameModel.update(newModel);
		for (int i = 0; i < this.players.length; i++) 
			if (this.players[i] != null) {
				if (this.gameModel.getPlayer(i) != null) {
					this.players[i].gameStateChanged();
					if (changeTurn == 2) 
						this.players[i].stageEvent(this.gameInfo.getStage().index());
				} else if (!this.windowOpen) //window has been closed, and the player is no longer in the model, so it has been removed from somewhere else
					this.players[i] = this.ourPlayers[i] = null;
			}
		if (!SwingUtilities.isEventDispatchThread()) {
			Runnable doWorkRunnable = new Runnable() {
				public void run() {
					updateGUI();
				}
			};
			try {
				SwingUtilities.invokeAndWait(doWorkRunnable);
			}
			catch (Exception e) {
				e.printStackTrace();
			}
		}
		else updateGUI();
		
		if (changeTurn != 0) {
			if (!SwingUtilities.isEventDispatchThread()) {
				Runnable doWorkRunnable2 = new Runnable() {
					public void run() {
						updateGUIBigger();
					}
				};
				try {
					SwingUtilities.invokeAndWait(doWorkRunnable2);
				}
				catch (Exception e) {
					e.printStackTrace();
				}
			}
			else updateGUIBigger();
		}
		if (this.gameInfo.getCurrentPlayerSeat() >= 0 && this.players[this.gameInfo.getCurrentPlayerSeat()] != null) {
			new Thread (new ActionGetter(this.players[this.gameInfo.getCurrentPlayerSeat()], this, this.gameInfo.getCurrentPlayerSeat())).start();
		}
		//this.gameModel.setCurrentPlayerSeat(this.gameInfo.getCurrentPlayerSeat()+1);
		//System.out.println("stage: "+ this.gameInfo.getStage().index());
		//System.out.println("bet_size: "+ this.gameInfo.getCurrentBetSize());	
		//System.out.println("bet_size2: "+ this.gameModel.getMinRaise());
		//System.out.println("stage2: "+ this.gameModel.getStage().index());
	}

	/**
	 * A player sits to the table, creates a new OurPlayer and sends the info to the Server
	 * Should be called only after the player has received confirmation of a successful reservation
	 * @param seatNumber the seat to sit at
	 * @param plrName name of the player
	 * @param cash amount of money brought to table
	 */
	public void addPlayer(int seatNumber, String plrName, double cash) {
		if (seatNumber < 0 || seatNumber >= this.players.length)
			throw new InvalidSeatException(seatNumber);
		if (this.players[seatNumber] != null || this.gameInfo.getPlayer(seatNumber) != null)
			throw new FullSeatException(seatNumber);
		OurPlayer newPlr = new OurPlayer(plrName, seatNumber, this);
		players[seatNumber] = ourPlayers[seatNumber] = newPlr;
		this.mainPlayerSeat = seatNumber;
		this.reserving = false;
		this.sendAction(Action.joinAction(plrName, seatNumber, cash), seatNumber);
	}
	
	/**
	 * Adding a bot to the table, sends the info to the server
	 * Should be called only after the seat has been successfully reserved
	 * @param seatNumber the seat to sit the bot at
	 * @param name name of the bot
	 * @param bot object implementing the Player interface for the bot
	 * @param cash amount of money brought to table
	 */
	public void addBot(int seatNumber, String name, Player bot, double cash) {
		if (seatNumber < 0 || seatNumber >= this.players.length)
			throw new InvalidSeatException(seatNumber);
		if (this.players[seatNumber] != null || this.gameInfo.getPlayer(seatNumber) != null)
			throw new FullSeatException(seatNumber);
		//OurPlayer newPlr = new OurPlayer(plrName, seatNumber, this);
		players[seatNumber] = bot;
		//System.out.println("Added bot to seat: "+seatNumber);
		//this.mainPlayerSeat = seatNumber;
		this.standing();
		this.sendAction(Action.joinAction(name, seatNumber, cash), seatNumber);
	}
	
	/**
	 * Removes player from given seat and sends the action to leave to the server
	 * Always supposed to close the window
	 * @param seatNumber
	 */
	public void removePlayer(int seatNumber) {
		this.removePlayer(seatNumber, true);
	}
	
	/**
	 * Removes player from given seat and sends the action to leave to the server
	 * @param seatNumber
	 * @param isLeavingTable when the player leaving also closes the window
	 */
	public void removePlayer(int seatNumber, boolean isLeavingTable) {
		if (seatNumber >= 0 && seatNumber < this.players.length && this.players[seatNumber] != null)
			this.players[seatNumber] = this.ourPlayers[seatNumber] = null;
		if (this.mainPlayerSeat == seatNumber) {
			this.mainPlayerSeat = -1;
			this.reserving = false;
		}
		double amount;
		if (isLeavingTable) {
			amount = -2;
			this.windowOpen = false;
			this.gameView = null;
		}
		else
			amount = 2;
		this.sendAction(Action.leaveGameAction(seatNumber, amount), seatNumber);  
	}
	
	/**
	 * Receives notification from GUI for a player wishing to come back from sit out
	 * and forwards it to the server
	 * @param seatNumber seat of the returning player
	 */
	public void playerBack(int seatNumber) {
		this.sendAction(Action.backAction(), seatNumber);
	}
	
	
	/**
	 * called when a player wants to reserve a seat, adds a reserving object to the model (only for internal purposes)
	 * and sends a reserve action to server
	 * @param seat the seat to reserve
	 * @return true if seat was free in the local model, this is useless, though
	 */
	public boolean reserveSeat(int seat) {
		if (!this.gameModel.reserveSeat(seat))
			return false;
		this.sendAction(Action.reserveAction(seat), seat);
		this.reserving = true;
		return true;
	}
	
	/**
	 * called to update the available seats, after not sitting down after reserving a seat, or only adding a bot
	 */
	public void standing(){
		this.reserving = false;
		this.updateSeatAvailable();
	}
	
	
	/**
	 * Retrieves visible hands. This includes the players own hand and the hands that are revealed.
	 * @param seat The user's seat.
	 * @return An array of hands.
	 */
	public Hand[] getVisibleHands(int seat) {
		Hand[] hands = new Hand[this.gameModel.getMaxPlayers()];
		for (int i = 0; i < hands.length; i++)
			if (hands[i] != null && this.gameModel.getPlayer(i) != null)
				hands[i] = this.gameModel.getPlayer(i).getRevealedHand();
		if (this.ourPlayers[seat] != null)
			hands[seat] = this.ourPlayers[seat].getHand();
		return hands;
	}
	
	/**
	 * Receives any win events. Makes it easier for bots to see what is happening, as results are processed for them.
	 * @param seat Seat of current player.
	 * @param amount Amount won.
	 */
	protected abstract void receiveWinEvent(int seat, double amount, int[] winningCardPositions);
	
	/**
	 * Receives action events and notifies all player in the table
	 * @param seat the Acting players seat
	 * @param amount amount relevant to the action
	 * @param index action index from class Action
	 */
	protected void receiveActionEvent(int seat, double amount, int index) {
		for (int i = 0; i < this.players.length; i++) 
			if (this.players[i] != null)
				this.players[i].actionEvent(seat, new Action(index, amount));
	}
	
	/**
	 * Receives notification for start of the hand, and passes it on to Player objects
	 * and removes previous game states from player's structure
	 */
	protected void receiveGameStartEvent() {
		this.firstHand = true;
		this.gameModel.startHand();
		for (int i = 0; i < this.players.length; i++) 
			if (this.players[i] != null)
				this.players[i].gameStartEvent(this.gameInfo, i);
	}
	
	/**
	 * Receives notification for end of the hand, and passes it on to Player objects
	 */
	protected void receiveGameOverEvent() {
		this.gameOver = true;
		for (int i = 0; i < this.players.length; i++) 
			if (this.players[i] != null)
				this.players[i].gameOverEvent();
	}
	
	/**
	 * Receives notification for showdown, and and calls players with each revealed hand
	 */
	protected void receiveShowdownEvent() {
		for (int i = 0; i < this.players.length; i++) 
			if (this.players[i] != null)
				for (int j = 0; j > this.players.length; j++)
					if (this.gameInfo.getPlayer(j) != null && 
							this.gameInfo.getPlayer(j).getRevealedHand() != null && 
							this.gameInfo.getPlayer(j).getRevealedHand().size() > 1 &&
							this.gameInfo.getPlayer(j).getRevealedHand().getCard(0).isValid() &&
							this.gameInfo.getPlayer(j).getRevealedHand().getCard(1).isValid())
						this.players[i].showdownEvent(j, this.gameInfo.getPlayer(j).getRevealedHand().getCard(0), this.gameInfo.getPlayer(j).getRevealedHand().getCard(1));
	}
	
	/**
	 * Receives notification for dealing of the hole cards, and passes it on to Player objects
	 */
	protected void receiveHoleCardsEvent() {
		for (int i = 0; i < this.players.length; i++) 
			if (this.players[i] != null)
				this.players[i].dealHoleCardsEvent();
	}
	
	
	
	/**
	 * Sends an action message to server.
	 * 
	 * @param action Action of current player.
	 * @param currentPlayer Seat of the current player.
	 */
	public void sendAction(Action action, int currentPlayer) {
		//this.gameView.addChatMessage("sending action "+action.index);
		Message msg = new Message();
    	Response rsp = new Response();
    	GameResponse game = new GameResponse();    	
    	Custom custom = new Custom();
    	    	
    	TexasMessage tex = new TexasMessage();
    	
    	ActionMessage actmsg = new ActionMessage();
    	actmsg.setIndex(action.getActionIndex());
    	actmsg.setAmount(action.getAmount());
    	actmsg.setSeat(currentPlayer);
    	actmsg.setDiscards(action.discards());
    	actmsg.setName(action.name());
    	
    	tex.setAction(actmsg);   	
    	Element tElem = null;
		try {
			tElem = xmltoolsTex.marshallObjectToElement(tex);
		} catch (JAXBException e1) {
			e1.printStackTrace();
		}
    	
    	custom.setAny(tElem);
    	game.setCustom(custom);
    	game.setGameID(getGameID());
    	
    	rsp.setGame(game);
    	msg.setResponse(rsp);
    	
    	msg.setSessionID(getID());
    	    	
    	try {
			connection.sendMessage(msg);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (JAXBException e) {

			e.printStackTrace();
		}


	}
	
	/**
	 * Send voice data to server. Voice is
	 * wrapped in GZIP compressed Base-64 encoded string.
	 * @param data Voice data
	 */
	public void sendVoice(byte[] data) {
//		Message msg = new Message();
//		Request rsp = new Request();
//    	GameRequest game = new GameRequest();
//    	String encodedData = Base64.encodeBytes(data, Base64.GZIP);
//    	game.setVoice(encodedData);
//    	game.setGameID(getGameID());
//    	rsp.setGame(game);
//    	msg.setRequest(rsp);
//    	msg.setSessionID(getID());
//    	
//    	try {
//			connection.sendMessage(msg);
//		} catch (IOException e) {
//			e.printStackTrace();
//		} catch (JAXBException e) {
//			e.printStackTrace();
//		}
//    	
	}
	
	/**
	 * Sends given chat to the server for distribution
	 * @param name The sender's name
	 * @param message 
	 */
	public void sendChat(String name, String message){
		Message msg = new Message();
    	Request rsp = new Request();
    	GameRequest game = new GameRequest();
    	ChatMessage chat = new ChatMessage();
    
    	chat.setMessage(message);
    	chat.setSender(name);
    
    	game.setChat(chat);
    	game.setGameID(getGameID());
    	
    	rsp.setGame(game);
    	msg.setRequest(rsp);
    	msg.setSessionID(getID());
    	
    	try {
			connection.sendMessage(msg);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (JAXBException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Receives a hand for a given seat.
	 * @param hand The hand received.
	 * @param seat The seat for which the hand was received.
	 */
	public void receiveHand(Hand hand, int seat) {
		if (this.players[seat] != null) {
			this.players[seat].receiveHand(hand, seat);
		//if (isSuperuser) TODO
			this.superUserHands[seat] = hand;
		if (this.ourPlayers[seat] != null && this.gameView != null) {
			Runnable doWorkRunnable = new HandReceived(this.gameView, seat, this);
			SwingUtilities.invokeLater(doWorkRunnable);
			}
		}
	}
	
	/**
	 * Receives confirmation for a seat reservation. Calls GUI to update view.
	 * @param seat Seat which is reserved.
	 */
	public void receiveConfirmationForReservation(int seat) {
		if(seat >= 0 && this.gameView != null){
			Runnable doWorkRunnable = new SeatConfirmation(this.gameView, seat, this, this.gameInfo);
			SwingUtilities.invokeLater(doWorkRunnable);
		}else{
			this.reserving = false;
		}
	}
	
	/**
	 * Pushes a newly received chat message into the GUI structures.
	 * @param sender Name of the client who sent the message.
	 * @param message The chat message.
	 */
	protected void receiveChat(String sender, String message) {
		if (this.gameView != null) {
			final String writing;
			if (sender == null || sender.length() < 1){
				//System.out.println("ei lahettajaa");
				writing = message;
			}
			else
				writing = "<" + sender + "> "+message;
			Runnable doWorkRunnable = new Runnable() {
			    public void run() { 
			    	gameView.addChatMessage(writing);    
			    }
			};
			SwingUtilities.invokeLater(doWorkRunnable);
		}
	}
	
	/**
	 * Gets chat messages from the GUI to be send forwards to the server
	 * If the player is playing, uses the name given, otherwise uses accountname if logged in,
	 * or Guest
	 * @param message
	 */
	public void sendChat(String message) {
		if (this.getOurPlayer() >= 0 && this.gameModel.getPlayer(this.getOurPlayer()) != null)
			this.sendChat(this.gameModel.getPlayer(this.getOurPlayer()).getName(), message);
		else if (this.accountName != null && this.accountName.length() > 0 )
			this.sendChat(this.accountName, message);
		else
			this.sendChat("Guest", message);
		
	}

	/**
	 * initialized the game controller for use.
	 * Sets the connection handler.
	 */
	@Override

	
	public void init(GameResponse response, String accountName, AccountType accountType) {		
		
		//System.err.println("Init called");
		this.accountName = accountName;
		//this.gameInfo = this.gameModel = new GameModel();
		TexasMessage tex = null;
		
		connection = getConnectionHandler();
		
		xmltoolsMsg = new XMLTools();
		try {
			xmltoolsTex = new XMLTools(
					"com.asdf.plugins.pokergames.texasholdem.parser", 
					"../Protocol/texas_message.xsd");
		} catch (SAXException e) {
			throw new RuntimeException(e);
		} catch (JAXBException e) {
			throw new RuntimeException(e);
		}
		
		
		
		if(response != null && response.getJoin().getParameters() != null) {
			List<Element> pars = response.getJoin().getParameters().getAny();
			try {
				tex = (TexasMessage)xmltoolsTex.unmarshallNode(
						pars.get(0));
			} catch (DOMException e) {
				throw new RuntimeException(e);
			} catch (JAXBException e) {
				throw new RuntimeException(e);
			}
			try{
				ACTION_WAIT = new Integer(pars.get(1).getAttribute("timer"));
			}catch(NumberFormatException e){
				ACTION_WAIT = 15000;
			}
		}
		//else
			//System.out.println("Init response was null");
		
		this.gameModel = new GameModel(tex.getGameInfo());
		this.gameInfo = this.gameModel;
		int maxPlayers = gameModel.getMaxPlayers();
		this.players = new Player[maxPlayers];
		this.ourPlayers = new OurPlayer[maxPlayers];
		//if (isSuperUser()) TODO
			this.superUserHands = new Hand[maxPlayers];

		//TODO random ? this.id = id;
		//System.out.println("My session id is : " + getID());
		
		setGameID(response.getGameID());
		
		
        Message msg = new Message();
        Request req = new Request();
        GameRequest game = new GameRequest();
        OptionalParameters oPars = new OptionalParameters();
        game.setJoin(oPars);
        game.setGameID(response.getGameID());
        msg.setRequest(req);
        msg.setSessionID(getID());
        req.setGame(game);
		
		try {
			connection.send(xmltoolsMsg.marshallObject(msg));
		} catch (IOException e) {
			throw new RuntimeException(e);
		} catch (JAXBException e) {
			throw new RuntimeException(e);
		}
		
	}

	

	/**
	 * Handles a response sent by players.
	 * 
	 */
	@Override
	public void handleResponse(Response rsp) { //TODO Incomplete
		
		if(rsp == null)
			return;
		
		GameResponse resp = rsp.getGame();
		
		if(resp != null){
			if(resp.getCustom() != null){
				
				//System.out.println("Received action.");
										
				TexasMessage texmex = null;
				try {
					texmex = (TexasMessage) xmltoolsTex.unmarshallNode(
							resp.getCustom().getAny());
				} catch (DOMException e) {
					e.printStackTrace();
				} catch (JAXBException e) {
					e.printStackTrace();
				}
				
				if(texmex.getAction() != null){
					
					Action action = null;
					ActionMessage act = texmex.getAction();
					switch(act.getIndex()){
					case Action.DISCARD:									
											
						int[] array = new int[act.getDiscarded().size()];					
						int j = 0;
						for(Integer i : act.getDiscarded()){
							array[j] = i;
							j++;
						}
						action = Action.discardAction(array);
						
						break;
					case Action.RESERVE_SEAT:					
						action = Action.reserveAction(act.getSeat());
						action.name = act.getName();
						action.seat = act.getSeat();
						this.receiveConfirmationForReservation(act.getSeat());
						break;
					default:
						action = new Action(act.getIndex(), act.getAmount());
						break;
					}
					//TODO do something with the action
				}else{
					//System.out.println("It was wrong! So wrong! Calling ghostbusters!");						
				}
			}else if(resp.getCreate() != null){
				// Should not get this kind of message...
			}else if(resp.getDelete() != null){
				// Should not get this kind of message...
			}else if(resp.getJoin() != null){
				// Should not get this kind of message...
			}else if(resp.getPart() != null){
				// Should not get this kind of message...				
			}else if(resp.getEvent() != null){
				//Should not get this kind of message...
				
			}
		}
			
	}

	@Override
	public void handleRequest(Request req) {
	}

	@Override
	public void handleStateUpdate(StateUpdate stateUp) {
		if (stateUp != null && stateUp.getGame() != null) {
//			if (stateUp.getGame().getVoice() != null) {
//				byte[] decodedData = Base64.decode(stateUp.getGame().getVoice(), Base64.GZIP);
//				SoundManager.playCustomData(decodedData);
//			}
			if (stateUp.getGame().getChat() != null) {
				receiveChat(stateUp.getGame().getChat().getSender(), stateUp
						.getGame().getChat().getMessage());
			}
			if (stateUp.getGame().getCustom() != null) {
				TexasMessage txmsg = null;
				try {
					txmsg = (TexasMessage) xmltoolsTex.unmarshallNode(stateUp
							.getGame().getCustom().getAny());
				} catch (DOMException e) {
					e.printStackTrace();
				} catch (JAXBException e) {
					e.printStackTrace();
				}

				if (txmsg.getAction() != null) {
					this.receiveDiscard(txmsg.getAction().getSeat(), txmsg
							.getAction().getDiscards());
				} else if (txmsg.getEvent() != null) {
					if(txmsg.getEvent().getType() == com.asdf.common.Event.WIN_EVENT.ordinal()){
						int [] positions = new int[txmsg.getEvent().getCardPositions().size()];
						for (int i = 0; i < positions.length; i++)
							positions[i] = txmsg.getEvent().getCardPositions().get(i);
						this.receiveWinEvent(txmsg.getEvent().getSeat(), txmsg
							.getEvent().getAmount(), positions);
					} else if (txmsg.getEvent().getType() == com.asdf.common.Event.GAMEOVER_EVENT.ordinal()) {
						if (this.gameView != null)
							this.gameView.closeWindow();
						//System.exit(0);
						///TODO: This game controller should DIEEE!!!!
					} else if (txmsg.getEvent().getType() == com.asdf.common.Event.GAMESTART_EVENT.ordinal()) {
						this.receiveGameStartEvent();
					} else if (txmsg.getEvent().getType() == com.asdf.common.Event.HANDOVER_EVENT.ordinal()) {
						this.receiveGameOverEvent();
					} else if (txmsg.getEvent().getType() == com.asdf.common.Event.HOLECARDS_EVENT.ordinal()) {
						this.receiveHoleCardsEvent();
					} else if (txmsg.getEvent().getType() == com.asdf.common.Event.SHOWDOWN_EVENT.ordinal()) {
						this.receiveShowdownEvent();
					} else if (txmsg.getEvent().getType() == com.asdf.common.Event.LEAVE_EVENT.ordinal()) {
						this.removePlayer(txmsg.getEvent().getSeat(), false);
					}	else if (txmsg.getEvent().getType() == com.asdf.common.Event.WIN_EVENT.ordinal()) {
						int[] tmp = new int[txmsg.getEvent().getCardPositions().size()];
						for(int i = 0; i < txmsg.getEvent().getCardPositions().size(); i++)
							tmp[i] = txmsg.getEvent().getCardPositions().get(i);		
						this.receiveWinEvent(txmsg.getEvent().getSeat(), txmsg.getEvent().getAmount(), tmp);						
					} else if (txmsg.getEvent().getType() == com.asdf.common.Event.ACTION_EVENT.ordinal()) {
						this.receiveActionEvent(txmsg.getEvent().getAction().getSeat(), txmsg.getEvent().getAction().getAmount(), 
								txmsg.getEvent().getAction().getIndex());
					}
				} else if (txmsg.getGameInfo() != null) {
					this.updateGameModel(new GameModel(txmsg.getGameInfo()));
				} else if (txmsg.getHand() != null) {
					this.receiveHand(new Hand(txmsg.getHand()), txmsg.getHand().getSeat());
				}
			}
		}
	}

	private void receiveDiscard(Integer seat, Integer discards) {
		// TODO Auto-generated method stub
		
	}

}

/**
 * Delivers actions to game controller for processing.
 * @author rimpila
 *
 */
class ActionGetter implements Runnable {
	
	private Player player;
	private int playerSeat;
	private PokerGameController gc;
	
	/**
	 * Constructor.
	 * @param player Player information object.
	 * @param gc Texas Hold'em game controller.
	 * @param currentPlayer Local player id.
	 */
	public ActionGetter(Player player, PokerGameController gc, int currentPlayer) {
		this.player = player;
		this.gc = gc;
		this.playerSeat = currentPlayer;
	}

	/**
	 * Reads an action from local player and gives it to game controller for processing.
	 */
	public void run() {
		Action action = this.player.getAction();
		if (action != null) //{
			this.gc.sendAction(action, this.playerSeat);

	}
	
	
}

/**
 * Seat confirmation class. Handles reserving seats and the actual sitting-downs.
 * @author rimpila
 *
 */
class SeatConfirmation implements Runnable {

	private GameView gameView;
	private int playerSeat;
	private PokerGameController gc;
	private GameInfo gameInfo;
	
	/**
	 * Constructor
	 * @param gameView GUI.
	 * @param seat Local player seat number.
	 * @param gc Texas game controller instance
	 * @param gi Game information instance
	 */
	public SeatConfirmation(GameView gameView, int seat, PokerGameController gc, GameInfo gi) {
		this.gameInfo = gi;
		this.playerSeat = seat;
		this.gameView = gameView;
		this.gc = gc;
	}
	
	/**
	 * Initiates a join specific pop-up window
	 */
	@Override
	public void run() {
		if (this.gameView != null) {
			this.gameView.setJoinPopUp(gc.accountName, playerSeat, gameInfo.minBuy(), gameInfo.maxBuy(), gc);
		}
	}
	
}

/**
 * Hand receiving class.
 * @author rimpila
 *
 */
class HandReceived implements Runnable {

	private GameView gameView;
	private PokerGameController gc;
	private int playerSeat;
	
	public HandReceived(GameView gameView, int seat, PokerGameController gc) {
		this.playerSeat = seat;
		this.gameView = gameView;
		this.gc = gc;
	}
	
	/**
	 * Updates local game view.
	 */
	@Override
	public void run() {
		if (this.gameView != null) {
			this.gameView.update(gc.getVisibleHands(playerSeat));	
		}
	}
	
}