package com.asdf.plugins.pokergames;

/**
 * Interface for any one wanting to implement a different Poker game using ready-made
 * PokerGame class
 * @author rimpila
 *
 */
public interface Stage {
	
	/**
	 * Function for transforming the stage into an integer for messages
	 * @return integer unique to the current stage
	 */
	public int index();
	/**
	 * Parameter concerning this stage, for stages with cards being dealt, the number of cards to deal
	 * @return integer
	 */
	public int param();
	/**
	 * Integer for which case to run in the Switch in PokerGame method updateStage(int Stage)
	 * @return int
	 */
	public int correspondingCase();	
	/**
	 * Returns the stage following the current stage
	 * @return Stage
	 */
	public Stage next();
	/**
	 * Stage that should be run when there are one of less players still active
	 * Probably should correspond to case PokerGame.LESS_THAN_TWO_PLAYERS
	 * @return Stage
	 */
	public Stage lessThanTwoPlayers();
	/**
	 * Stage that should be run first
	 * @return the Stage
	 */
	public Stage start();
	/**
	 * Returns the Stage that this integer represents
	 * this is the inverse of index()
	 * @param index
	 * @return the Stage
	 */
	public Stage indexToStage(int index);
	
}
