package com.asdf.plugins.pokergames;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;

/**
 * Class containing player statistics.
 */
public class StatisticsPlayer {
	private String name;
	private String account;
	private short position;
	private StringBuilder preflop = new StringBuilder();
	private StringBuilder flop = new StringBuilder();
	private StringBuilder turn = new StringBuilder();
	private StringBuilder river = new StringBuilder();
	private double moneyIn;
	private double winnings;
	private String[] cards;
	
	public StatisticsPlayer(PlayerInfo player) {
		this.name = player.getName();
		this.account = player.getAccountName();
		this.position = (short)player.getSeat();
		this.cards = new String[2];
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("<StatisticsPlayer");
		sb.append(" name=" + name);
		sb.append(" account=" + account);
		sb.append(" position=" + position);
		sb.append(" preflop=" + preflop);
		sb.append(" flop=" + flop);
		sb.append(" turn=" + turn);
		sb.append(" river=" + river);
		sb.append(" moneyIn=" + moneyIn);
		sb.append(" winnings=" + winnings);
		sb.append(" cards=");
		for(String card : cards)
			sb.append("(" + card + ")");
		sb.append(">");
		return sb.toString();
	}

	/**
	 * @return the cards
	 */
	public String[] getCards() {
		return cards;
	}

	/**
	 * @param cards the cards to set
	 */
	public void setCards(String[] cards) {
		this.cards = cards;
	}

	/**
	 * @return the flop
	 */
	public String getFlop() {
		return flop.toString();
	}

	/**
	 * @param flop the flop to set
	 */
	public void appendFlop(String flop) {
		this.flop.append(flop);
	}

	/**
	 * @return the moneyIn
	 */
	public double getMoneyIn() {
		return moneyIn;
	}

	/**
	 * @param moneyIn the moneyIn to set
	 */
	public void setMoneyIn(double moneyIn) {
		this.moneyIn = moneyIn;
	}

	/**
	 * @return the preflop
	 */
	public String getPreflop() {
		return preflop.toString();
	}

	/**
	 * @param preflop the preflop to set
	 */
	public void appendPreflop(String preflop) {
		this.preflop.append(preflop);
	}

	/**
	 * @return the river
	 */
	public String getRiver() {
		return river.toString();
	}

	/**
	 * @param river the river to set
	 */
	public void appendRiver(String river) {
		this.river.append(river);
	}

	/**
	 * @return the turn
	 */
	public String getTurn() {
		return turn.toString();
	}

	/**
	 * @param turn the turn to set
	 */
	public void appendTurn(String turn) {
		this.turn.append(turn);
	}

	/**
	 * @return the winnings
	 */
	public double getWinnings() {
		return winnings;
	}

	/**
	 * @param winnings the winnings to set
	 */
	public void setWinnings(double winnings) {
		this.winnings = winnings;
	}
	
	/**
	 * Store collected player data to database. Database table for
	 * player is defined as following:
	 * CREATE TABLE player (
	 * 	hand_id integer NOT NULL,
	 * 	account varchar(256) NOT NULL,
	 * 	name varchar(256) NOT NULL,
	 * 	position smallint NOT NULL,
	 * 	flop varchar(256),
	 * 	turn varchar(256),
	 * 	river varchar(256),
	 * 	money_in decimal,
	 * 	winnings decimal,
	 * 	card1 char(3),
	 * 	card2 char(3),
	 * 	PRIMARY KEY (hand_id, account, name)
	 * );
	 * @param dbConnection
	 * @param handID
	 * @throws SQLException 
	 */
	public void storeToDb(Connection dbConnection, int handID) throws SQLException {
		PreparedStatement ps = dbConnection.prepareStatement(
			"INSERT INTO player VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
		ps.setInt(1, handID);
		ps.setString(2, account);
		ps.setString(3, name);
		ps.setInt(4, position);
		ps.setString(5, flop.toString());
		ps.setString(6, turn.toString());
		ps.setString(7, river.toString());
		ps.setDouble(8, moneyIn);
		ps.setDouble(9, winnings);
		for(int i = 0; i < 2; i++) {
			if (i < cards.length)
				ps.setString(10+i, cards[i]);
			else
				ps.setNull(10+i, Types.CHAR);
				
		}
		ps.execute();
	}
}
