package com.asdf.plugins.pokergames.gameview;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Timer;
import java.util.TimerTask;
import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import com.asdf.client.SoundManager;
import com.asdf.common.Action;
import com.asdf.common.PokerGameTypes;
import com.asdf.plugins.pokergames.GameInfo;
import com.asdf.plugins.pokergames.Hand;
import com.asdf.plugins.pokergames.PlayerInfo;
import com.asdf.plugins.pokergames.PokerGame;
import com.asdf.plugins.pokergames.PokerGameController;

/**
 * multiuse pokertable GUI frame.
 * 
 * 
 * @author ASDF
 *
 */

public class GameView extends JFrame /*implements Runnable*/{ 

	private static final long serialVersionUID = 1L;
	public static final int cardsizeX = 75;
	public static final int cardsizeY = 104;	
	public static Hashtable<String, BufferedImage> images = null;
	private LinkedList<GameViewSeat> seats = new LinkedList<GameViewSeat>();
	private JLabel background = new JLabel("");
	private GameViewSitDownPopUp joinpopup = null; 
	private JPanel quickset;
	private GameViewSeat timerseat;
	private GameViewChatPanel chatpanel;
	private GameViewCheckBoxPanel checkpanel;
	private GameViewButtonPanel buttonpanel;
	private GameViewTableCards tablecards;
	private JLayeredPane view;
	private String back = "background";
	private PokerGameTypes gametype; 
	private Timer resized = null;
	private Timer turntimer = null;
	private final int maxsizeY = 625;
	private final int maxsizeX = 900;
	private final int playareamax = 525;
	private double timerwait,timerstep,timermax;
	private int sizeX = 900; // x-axis size
	private int sizeY = 625; // y-axis size
	private int sizeP = 525; // playarea-size
	public double xratio = 100.0; // x-axis ratio
	public double pratio = 100.0; // playarea ratio
	
	
	/**
	 * Loads images that need resize in gameview as BufferedImage
	 * 
	 */
	
	private BufferedImage loadfile (String s){
		try {
			InputStream is = getClass().getResourceAsStream(s);
			if (is==null)
				return ImageIO.read(new File(s));
			else 
				return ImageIO.read(is);
			} catch (IOException e) {
				System.out.println("Picture "+s+"is missing");
				return null;
		}
	}
	
	/**
	 * Places images that game uses to static hashtable as BufferedImages
	 * to be used later as scaled ImageIcon instance.
	 * 
	 */
	
	private void loadimages() {
		if (images == null) {
		images = new Hashtable<String, BufferedImage>();
			for (int i=1;i<14;i++){
				images.put("C"+i,loadfile("images/C"+i+".png"));
				images.put("D"+i,loadfile("images/D"+i+".png"));
				images.put("H"+i,loadfile("images/H"+i+".png"));
				images.put("S"+i,loadfile("images/S"+i+".png"));
			}
			for (int i=1;i<9;i++)
				images.put("chip"+i,loadfile("images/chip"+i+".png"));
			images.put("chip", loadfile("images/chip.png"));
			images.put("small", loadfile("images/small.png"));
			images.put("dealer", loadfile("images/dealer.png"));
			images.put("reserved", loadfile("images/reserved.png"));
			images.put("background", loadfile("images/background.png"));
			images.put("background2", loadfile("images/background2.png"));
			images.put("background3", loadfile("images/background3.png"));
			images.put("background4", loadfile("images/background4.png"));
			images.put("backside", loadfile("images/backside.png"));
		}
	}
	
	/**
	 * Generates ImageIcons from stored BufferedImages. ImageIcons are scaled
	 * instances and their relative size to original pictures is given as x,y parameters.
	 * 
	 * @param key - Key to BufferedImage hashtable to get right picture. 
	 * @param xratio - Width percentage
	 * @param yratio - Height percentage
	 * @return generated ImageIcon
	 */
	
	public static ImageIcon getIconFromBufferedimage(String key,double xratio, double yratio){
		if (xratio==0 || yratio ==0) return null;
		BufferedImage b=images.get(key);
		int sizey=b.getHeight();
		int sizex=b.getWidth();
		if (xratio==1.0 && yratio==1.0)
			return new ImageIcon(b.getScaledInstance(sizex, sizey, BufferedImage.SCALE_FAST));
		sizex = (int) (b.getWidth() * xratio);
		sizey = (int) (b.getHeight() * yratio);
		if (sizex != 0 && sizey != 0) 
			return new ImageIcon(b.getScaledInstance(sizex, sizey, BufferedImage.SCALE_SMOOTH));
		return null;
	}
	
	/**
	 * Generates new gameview with given card amount, maxplayers, gametype and turn timer delays.
	 *  
	 * @param maxcards 
	 * @param maxplayers
	 * @param gametype_
	 * @param timer_wait
	 */
	
	public GameView(int maxcards,int maxplayers, PokerGameTypes gametype_, int timer_wait) {
		gametype = gametype_;
		loadimages();
		timerwait = timer_wait;
		
		this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		this.setMaximizedBounds(new Rectangle(0,0,maxsizeX,maxsizeY));
		this.setPreferredSize(new Dimension(maxsizeX,maxsizeY));
		this.setSize(1200,1000);
		this.setLayout(new BorderLayout(2,2));
		this.setResizable(true);
		
		view = this.getLayeredPane();
		
		chatpanel = new GameViewChatPanel();
		checkpanel = new GameViewCheckBoxPanel(this);
		buttonpanel = new GameViewButtonPanel(gametype,view);
		tablecards = new GameViewTableCards(view);
		quickset = new GameViewQuickSet(this);
		JPanel backselect = new GameViewBackGroundSelector(this);
		view.add(backselect);
		view.add(quickset);
		view.setLayer(quickset, 60000);
		
		this.add(background,BorderLayout.NORTH);
		this.add(checkpanel,BorderLayout.WEST);
		this.add(buttonpanel,BorderLayout.EAST);
		this.add(chatpanel,BorderLayout.CENTER);
		this.addWindowListener(new WindowAdapter() {
		      public void windowClosing(WindowEvent w) {
		    	  closeWindow();
		      }});
		
		int Pos[][] = initSeats(gametype,maxplayers);
		
		for (int i=0; i < Pos.length;i++)
			seats.add(new GameViewSeat(view,Pos[i][0],Pos[i][1],
					maxcards,seats,i,gametype.isDiscardable()));
	
		this.pack();
		this.setVisible(true);
		resize();
		
	    this.addComponentListener(new java.awt.event.ComponentAdapter() {
		      public void componentResized(ComponentEvent e) {
		    	  synchronized(this){
			    	  if(resized!=null)resized.cancel();
			    	  resized = new Timer();
		    	  }
		          resized.schedule(new TimerTask() {
		        	  public void run(){
		        		  javax.swing.SwingUtilities.invokeLater(new Runnable() {
		        			    public void run() {
		        			        resize();
		        			    }
		        		  });

		        		 }}, 500);}
		});

	}
	
	public void closeWindow(){
		buttonpanel.sendActionLeaveToController(true);
		this.dispose();
	}
	
	/**
	 * Sets player seats around table according to amount and gametype
	 * returns int array of that contains percentage values of each seat x,y position in gamearea.
	 *  
	 * @param game
	 * @param maxplayers
	 * @return
	 */
	
	private int[][] initSeats(PokerGameTypes game, int maxplayers){
		int[][] Pos = null;
		if (gametype.isPokerGame())
				switch(maxplayers){
					case 10: Pos = new int[][]{
							{37,14},{63,14},{88,23},{93,50},{88,77},
							{63,86},{37,86},{12,77},{7,50},{12,23},};break;
					case 9: Pos = new int[][]{
							{37,14},{63,14},{91,23},{93,50},{91,77},
							{50,86},{9,77},{7,50},{9,23},};break;
					case 8: Pos = new int[][]{
							{37,14},{63,14},{91,29},{91,71},
							{63,86},{37,86},{9,71},{9,29}};break;
					case 7: Pos = new int[][]{
							{37,14},{63,14},{91,29},{91,71},
							{50,86},{9,71},{9,29}};break;
					case 6: Pos = new int[][]{
							{32,14},{68,14},{93,50},
							{68,86},{32,86},{7,50}};break;
					case 5: Pos = new int[][]{
							{27,14},{73,14},{93,60},{50,86},{7,60}};break;
					case 4: Pos = new int[][]{
							{50,14},{93,50},{50,86},{7,50}};break;
					case 3: Pos = new int[][]{
							{50,14},{93,70},{7,70}};break;
					case 2: Pos = new int[][]{
							{50,14},{50,86}};break;
				}
		return Pos;
	}
	
	/**
	 *  resizes all gameview components.
	 *   
	 */
	
	private void resize() {
		sizeX = this.getContentPane().getWidth();
		sizeY = this.getContentPane().getHeight();
		if (sizeX == 0 || sizeY == 0) return;
		sizeP = sizeY - 100; // gamearea w/o bottompanel
		xratio = sizeX / ((1.0) * maxsizeX);
		pratio = sizeP / ((1.0) * playareamax);
		if (xratio>0.7 && pratio>0.7){quickset.setBounds(sizeX-56,2,54,48);
			quickset.setMinimumSize(new Dimension(54,48));
		} else quickset.setBounds(sizeX-26,2,24,24);
		
		buttonpanel.setBetMenuSize(sizeX, sizeP);
		buttonpanel.setPreferredSize(new Dimension(230, 100));
		
		tablecards.bounds(sizeX, sizeP, xratio, pratio);
		
		background.setIcon(getIconFromBufferedimage(back,xratio,pratio));
		
		for (GameViewSeat seat : seats)
			seat.bounds(sizeX, sizeP, xratio, pratio);
		
		view.updateUI();
	}
	
	/**
	 * Method that shows winning cards by setting other cards as disabled.  
	 */
	
	public void updatePreWin(){
		for (GameViewSeat seat : seats)
				seat.setCardsDisable();
		tablecards.enableCards(false);
	}
	
	/**
	 * @param winseat - Position of winner //TODO should be like int array in case of draw?
	 * @param wincards - Int array of card positions
	 * @param cardsingame - Amount of cards that determine winning hand.
	 */
	public void updateWin(int winseat,int[] wincards,int cardsingame){
		boolean[] b= new boolean[cardsingame];
		
		for (int i=0;i<cardsingame;i++)
			b[i]=false;
		
		for (int i=0; i<wincards.length; i++)
			b[wincards[i]]=true;
		
		for (GameViewSeat seat : seats)
			if (seat.getSeatNro() == winseat){
				int i;
				for(i = 0; i < seat.getCardCount(); i++){
					if (b[i]) seat.enableCard(i, true);
				}
				for(int j = i; j < b.length; j++){
					if (b[j]) tablecards.enableCard(j-seat.getCardCount(), true);
				}
			}
	}
	
	/**
	 * Updates client gameview GUI to show given gameview situation. 
	 *   
	 * @param gm
	 */
	
	public void update(GameInfo gm){
		
		for (GameViewSeat seat : seats){
			synchronized(this){
			  	int nro = seat.getSeatNro();

			   	//setting dealer
		       	seat.setDealer(nro == gm.getButtonSeat());
		       	
		       	//showing reserved seats
		       	seat.setReserved(gm.isReserved(nro) && gm.getPlayer(nro)==null);
		       	buttonpanel.setBetAndPot(gm.getCurrentBetSize(), gm.getTotalPotSize());
		    	
		    	PlayerInfo p = gm.getPlayer(nro);
		        if (p==null){
		        		//hide player information if there is no player in seat
		        	if (timerseat != null && nro == timerseat.getSeatNro()){
		        		timerseat.setTimerVisible(false);
		        		if(turntimer !=null)turntimer.cancel();
		        	}
		        	seat.hideAll();
		        	seat.setBet(false,0,xratio,pratio);
		        	continue;
		        }
			
		    // set bet    
		    seat.setBet((p.getAmountInPotThisRound()!=0),p.getAmountInPotThisRound(),xratio,pratio);
			
	        String action = null;
	        if (p.isSittingOut()){
	        	seat.setSeat(false, p.getAccountName(), p.getName(), p.getBankRoll(), "Sit Out");
	        	seat.hideCards();
	        	continue;
	        } else if (p.isFolded()){
	        	seat.setSeat(false, p.getAccountName(), p.getName(), p.getBankRoll(), "Folded");
	        	if (seat.isCardsKnown())seat.enableCards(false);
	        		else seat.setCardsDisable();
	        	continue;
	        } else if (p.isAllIn()) {
	        	action="All-in";
	        } else if (!p.hasActedThisRound()) action = "";
	        else switch(p.getLastAction()){
	        	case Action.RAISE:
	        	case Action.BET:
	        		action="Raise";break;
	        	case Action.FOLD:
			       	seat.setCardsDisable();
	        		action="Fold";break;
	        	case Action.CALL:
	        		action="Called";break;
	        	case Action.CHECK:
	        		action="check";break;
	        }
	        
	        seat.setSeat(p.isActive(), p.getAccountName(), p.getName(), p.getBankRoll(), action);
	        
		    // show revealed hands
		    Hand hand = p.getRevealedHand(); 
		       	
		    if (hand != null && !seat.isCardsKnown())
		    	if (hand.size()==0)seat.setcards(true);
		    		// in holdem we show only small cards instead of big cards if cards are hidden
				    else if (gametype.isHoldem() &&!hand.getCard(0).isValid()) seat.setcards(true);
				    	else {
				    		//draw back of cards and so on...
				    		String[] cards = new String[hand.size()];
				    		for (int i=0;i< hand.size();i++)
				    			if(hand.getCard(i).isValid()) cards[i]= hand.getCard(i).toString();
				    				else cards[i]= "backside";
				    		seat.setCards(true,cards,xratio,pratio);
				    	} 
			}
		}
		tablecards.setbet("POT:$"+gm.getTotalPotSize());
		Hand hand = gm.getBoard();
		
		if (tablecards.getVisibleCards() < hand.size() && !checkpanel.isAudioDisabled())
			SoundManager.playCardsRubbing();
		
		if (hand != null){
			String[] cards = new String[hand.size()];
				for (int i=0;i< hand.size();i++)
					if(hand.getCard(i).isValid())
						cards[i]= hand.getCard(i).toString();
    					else cards[i]= "backside";
			tablecards.setCards(cards,xratio,pratio);
		}
	}
	
	/**
	 * Changes background image of GUI to BufferedImage that is found with given key
	 * from BufferedImage hashtable.  
	 */
	
	public void setBackGround(String key){
		back = key;
		background.setIcon(getIconFromBufferedimage(back,xratio,pratio));
		view.updateUI();
	}
	
	/**
	 * Gives players their private cards. 
	 * Superuser gets all cards here.
	 * 
	 * @param hands array of strings that represent pokercards known.
	 */
	
	public void update(Hand[] hands){
		String[] cards;
		Hand hand;
		synchronized (this) {
			if (!checkpanel.isAudioDisabled())
				SoundManager.playCardsRubbing();
			for (GameViewSeat seat : seats) {
				hand = hands[seat.getSeatNro()];
				if (hand == null || hand.size() == 0) {
					// no hand
					seat.setCardsKnown(false);
					seat.setcards(false);
				} else if (!hand.getCard(0).isValid()) 
					// no information about hand, draw small cards
					seat.setcards(true);
				 	else {
						seat.setCardsKnown(true);
						cards = new String[hand.size()];
						for (int i = 0; i < hand.size(); i++)
							cards[i] = hand.getCard(i).toString();
						this.addChatMessage("** Dealing holecards (" + cards[0] + "," + cards[1]+")");
						seat.setCards(true, cards,xratio,pratio);
				 	}
			}
		}
	}
	
	/**
	 * Draws available seats. 
	 * @param gc Controller that takes responces to actions.
	 * @param cansit Draw or not to draw available seat.
	 */

	public void update(boolean cansit, PokerGameController gc){

		GameInfo gm = gc.getGameInfo();
		for (GameViewSeat seat : seats)
			seat.setAvailable(!gm.isReserved(seat.getSeatNro()) && cansit, gc);
	}
	
	/**
	 * Update method that handles given gamesituation from controller.
	 * Gives player buttons needed in current game situation.
	 * Draws possible waitgrid on other players turn.
	 * Draws timer to activeplayer if betround.
	 * Also shows available seats to player that can sit down.
	 * 
	 * @param gc
	 * @param cansit - draw available seats  
	 */
	
	public void update(PokerGameController gc, boolean cansit){
		
		GameInfo gm = gc.getGameInfo();
		
		for (GameViewSeat seat : seats){
			seat.setGameViewAccountName(gc.getAccountName());
			seat.setAvailable(!gm.isReserved(seat.getSeatNro()) && cansit, gc);
		}
		
		buttonpanel.setController(gc);
		chatpanel.setController(gc);
		
		int current = gm.getCurrentPlayerSeat();
		
		if (gm.getStage().correspondingCase() == PokerGame.BET_ROUND){
				for (GameViewSeat seat : seats)
					if (current == seat.getSeatNro())
						this.setTimer(timerwait/100, seat);
						
			} else {
				synchronized(this){
					if (turntimer!=null)
						turntimer.cancel();
					for (GameViewSeat seat : seats)
						seat.setTimerVisible(false);
				}	
			}
		int nro = gc.getOurPlayer();

		
		if (nro < 0){
			this.setTitle(""+gametype.toString().toLowerCase()+" "+gm.getSmallBlindSize()+"/"+gm.getBigBlindSize());
			return; 
		}
		this.setTitle(""+gametype.toString().toLowerCase()+" "+gm.getSmallBlindSize()+"/"+gm.getBigBlindSize()+":"+gm.getPlayerName(nro));
		if (gm.getPlayer(nro).isSittingOut()) {
			buttonpanel.setSitOutButton();
			return;
		} else if (gm.getStage().correspondingCase() == PokerGame.SHOWDOWN &&
				checkpanel.isSitOut())
					buttonpanel.sendSitOutToController(checkpanel);

		if (nro == current) {
			this.myTurn(gm, gc, nro);
			
		} else if (gm.getPlayer(nro).isActive() &&
		  		   gm.getStage().correspondingCase() == PokerGame.BET_ROUND &&
		  		   !gm.getPlayer(nro).isAllIn()) {
			buttonpanel.updateWaitGrid( gm.getMinRaise(),
											gm.getAmountToCall(nro),
											gm.getBankRoll(nro));
			buttonpanel.setWaitGrid( gm.getAmountToCall(nro),
										 gm.getPlayer(nro).getAmountRaiseable(),
										 gm.getMinRaise(),
										 gm.getBankRoll(nro));
		} else buttonpanel.setAllHidden();
		
	}
	
	/**
	 * Draws buttons to current game situation in this gameview   
	 */
	
	private void myTurn(GameInfo gm,PokerGameController gc, int nro){
		
		if (!checkpanel.isAudioDisabled())
			SoundManager.playYourTurn();
		this.toFront();
	
		//BLINDROUND
		if (gm.getStage().correspondingCase() == PokerGame.BLIND_ROUND) {
			if (checkpanel.isAutoPostBlind()) {
				gc.setNextAction(com.asdf.common.Action.POST_BLIND);
				this.addChatMessage("autopost laitettiin");
			}
			else
				buttonpanel.setPostBlindButtons();
		}

		//MUCKROUND
		else if (gm.getStage().correspondingCase() == PokerGame.MUCK_ROUND) {
			if (checkpanel.isMuckLosingHands())
				gc.setNextAction(com.asdf.common.Action.MUCK);
			else
				gc.setNextAction(com.asdf.common.Action.REVEAL);

		}

		//LESS THAN 2 PLAYERS
		else if (gm.getStage().correspondingCase() == PokerGame.LESS_THAN_TWO_PLAYERS) {
			if (gm.getCurrentPlayerSeat() == nro) 
				if (checkpanel.isMuckWinningHands())
					gc.setNextAction(com.asdf.common.Action.MUCK);
				else
					buttonpanel.setMuckButtons();
		}
		
		//BETROUND
		else if (gm.getStage().correspondingCase() == PokerGame.BET_ROUND) {

			buttonpanel.updateWaitGrid( gm.getMinRaise(),
					gm.getAmountToCall(nro),
					gm.getBankRoll(nro));				

			buttonpanel.setPokerButtons( gm.getAmountToCall(nro), 
										 gm.getTotalPotSize(),
										 gm.getBankRoll(nro),
										 gm.getPlayer(nro).getAmountRaiseable(), 
										 gm.getMinRaise());
		}
		// insert discardround here.
		else {
			System.out.println("DiscardRound");
		}
	}
	
	/**
	 * Creates popup-window that allows one to sit down in given seat.
	 * 
	 * @param accountname_ Account name
	 * @param seat_nro Number of seat. 
	 * @param min_money Minimum money that one can bring to table.
	 * @param max_money Maxumum money that one can bring to table.
	 * @param pgc PokerGameController of one that is going to join. 
	 */
	
	public void setJoinPopUp(String accountname_, int seat_nro, double min_money,
			double max_money, PokerGameController pgc) {
		
		Point center = new Point(0, 0);
		for (GameViewSeat seat : seats)
			if (seat.getSeatNro() == seat_nro)
				center = seat.getCenter();
		center.x += this.getBounds().x;
		center.y += this.getBounds().y;

		if (joinpopup != null) joinpopup.close(); // only one popup for game.
		
		joinpopup = new GameViewSitDownPopUp(accountname_, seat_nro, min_money,
				max_money, pgc, center);
		
		if (!checkpanel.isAudioDisabled())
			SoundManager.playHumming();
	}
	
	/**
	 * Forwards messages to ChatPanel
	 *  
	 * @param message - given message.
	 */
	
	public void addChatMessage(String message){
		chatpanel.addChatMessage(message);
	}
	
	/**
	 * Sets turntimer for given seat.
	 * 
	 */
	
	public void setTimer(double time,GameViewSeat activeseat){
		for (GameViewSeat seat : seats) 
				seat.setTimerVisible(false);
		activeseat.setTimerVisible(true);
		timerseat = activeseat;
		final GameViewSeat seat = activeseat;
		timermax = time;
		timerstep = time;
		synchronized(this){
			if (turntimer != null)
				turntimer.cancel();
			turntimer = new Timer();
			turntimer.scheduleAtFixedRate(new TimerTask() {
				public void run() {
					//30.6.2008:
					javax.swing.SwingUtilities.invokeLater(new Runnable() {
					    public void run() {
					        decreaseTimer(seat);
					    }
					});
					//decreaseTimer(seat);
				}
			}, 0, 100);
		}
	}
	
	/**
	 * Scheduled decrease of timer.
	 * @param seat
	 */
	
	public void decreaseTimer(GameViewSeat seat){
		timerstep--;
		if (timerstep !=0)
			seat.drawTimer(timerstep, timermax);
		else {		
			turntimer.cancel();
			if (!checkpanel.isAudioDisabled())
				SoundManager.playTimesUp();
			seat.setTimerVisible(false);
		}
	}
}

	
