package com.asdf.plugins.pokergames.gameview;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import com.asdf.common.PokerGameTypes;


public class GameViewBetmenu extends JPanel implements ActionListener, ChangeListener, MouseListener, KeyListener {
	
	private static final long serialVersionUID = 1L;
	private final Border raisedBorder = BorderFactory.createRaisedBevelBorder();
	private final Border grayBorder = BorderFactory.createLineBorder(Color.darkGray);
	String[] speedpotnames = null;
	double[] speedpotvalues = null;
	
	private JLabel[] speedbet;
	private JTextField betinput;
	private JSlider betslide;
	private JLabel bet;
	private GameViewGameButton raise;
	private double pot, max, min, bank;
	private PokerGameTypes gametype;
	private boolean betNotRaise;
	
	/**
	 * Creates betmenu for pokergames that gives slider+some predefined bet options
	 * and inputfield to enter bet actions.
	 * 
	 * @param view JlayeredPane that this betmenu will be added.
	 * @param gb Raisebutton in given game that betmenu will affect.
	 * @param gametype_ Gametype of given game, Determines whatkind of betmenu we do. 
	 */
	
	public GameViewBetmenu (JLayeredPane view, GameViewGameButton gb, PokerGameTypes gametype_) {
		gametype = gametype_;
		if (gametype.isFixedLimit()) return;
		if (gametype.isNoLimit()){
			speedpotnames = new String[] {"Min","2 Min","3 Min", "5 Min","Max"};
			speedpotvalues = new double[] {0,2,3,4,0};
		} else if (gametype.isPotLimit()){
			speedpotnames = new String[] {"Min","1/3 pot","1/2 pot", "2/3 pot","Pot"};
			speedpotvalues = new double[] {0,1.0/3,0.5,2.0/3,0};
		}
		speedbet = new JLabel[speedpotnames.length];
		raise = gb;
		JPanel betview3 = new JPanel();
	  	JPanel betview2 = new JPanel();
		JPanel betview1 = new JPanel();
		view.add(this);
		this.setLayout(new BorderLayout());
		this.add(betview1,BorderLayout.NORTH);		
		this.add(betview2,BorderLayout.CENTER);
		this.add(betview3,BorderLayout.SOUTH);
		betview1.setPreferredSize(new Dimension(300,15));
		betview1.setMaximumSize(new Dimension(300,15));
		betview2.setPreferredSize(new Dimension(300,15));
		betview2.setMaximumSize(new Dimension(300,15));
		betview3.setPreferredSize(new Dimension(300,8));
		betview3.setMaximumSize(new Dimension(300,8));
		betview1.setLayout(new GridLayout());
		betview2.setLayout(new BoxLayout(betview2, BoxLayout.X_AXIS));
		betview3.setLayout(new GridLayout(2,1));
		this.setOpaque(true);	
		this.setBorder(raisedBorder);
		betview2.add(betinput = new JTextField(3));
		betview2.add(bet = new JLabel("  BET  "));
		bet.setFont(new Font("verdana", Font.PLAIN, 10));
		betview3.add(betslide = new JSlider(JSlider.HORIZONTAL));
		betslide.setSize(1,1);
    	betslide.setPaintTicks(true);
    	
		for (int i=0;i < speedpotnames.length;i++){
			betview1.add(speedbet[i] = new JLabel(speedpotnames[i], JLabel.CENTER));
			speedbet[i].setOpaque(true);	
		  	speedbet[i].setBorder(raisedBorder);
		  	speedbet[i].addMouseListener(this);
			speedbet[i].setFont(new Font("verdana", Font.PLAIN, 10));}
		
		view.setLayer(this,55500);
		this.setVisible(false);
		betinput.addActionListener(this);
		betinput.addKeyListener(this);
	  	betslide.addChangeListener(this);
	  	this.betNotRaise = true;
	}
	
	/**
	 * Verifies that given JTextField contains double.
	 * 
	 * @param input Field that one verifies. 
	 * @return Was it double or not?
	 */
	
	public boolean verify(JTextField input) {		
        String text = input.getText().trim();
        if (text.length()==0)return false;
        double i;

        try {
            if(text == null || text.equalsIgnoreCase(""))
                i = 0;
            else
                i = new Double(text);            
         } catch (NumberFormatException e){
        	 return false;
         }
         if(i >= 0) return true;
         return false;
     }
	
	/**
	 * Method that sets betmenu visible, and sets given min/max and so on.
	 * 
	 * @param bnk_ How much money this player still has. 
	 * @param min_ Minraise.
	 * @param max_ Maxraise.
	 * @param pot_ Current pot.
	 * @param bORr Is it bet or raise.
	 */
	
	public void update (double bnk_, double min_, double max_, double pot_, boolean bORr){
		bank = bnk_;
		pot = pot_;
		
		betNotRaise = bORr;
		
		betslide.setMaximum((int) (max = max_));
		betslide.setMinimum((int) (min = min_));
		betslide.setValue((int) min);
		this.setVisible(true);
	}
	
	/**
	 * "Highlites" selected predefined button given as parameter.
	 * 
	 * @param jlabel
	 */
	
	private void setActiveColor(JLabel jlabel){
		for (int i=0;i<speedbet.length;i++)
			if (speedbet[i]==jlabel){
				speedbet[i].setBackground(Color.DARK_GRAY);
				speedbet[i].setForeground(Color.white);
				speedbet[i].setBorder(grayBorder);
			} else {
				speedbet[i].setBackground(Color.LIGHT_GRAY);
				speedbet[i].setForeground(Color.black);
				speedbet[i].setBorder(raisedBorder);
			}
	}
	
	/**
	 * Activates raise button. 
	 */
	
	public void actionPerformed(ActionEvent e) {
		if (!verify(betinput)) return;
		raise.setAmount(Double.valueOf(betinput.getText()));
		raise.doClick();
	}
	
	public void mouseClicked(MouseEvent e) {
		Object source = e.getSource();
	    for(int i=0;i<speedbet.length;i++)
        	if (source==speedbet[i]){
        		if (source==speedbet[0]) // Min button.
                	betslide.setValue((betslide.getMinimum()));
        		else if (source==speedbet[speedbet.length-1]) // Max button.
        			betslide.setValue((betslide.getMaximum()));
        		else if (gametype.isNoLimit())
        			betslide.setValue((int)(min*speedpotvalues[i]));
        		else if (gametype.isPotLimit())
        			betslide.setValue((int)(pot*speedpotvalues[i]));
        		betslide.removeChangeListener(this);
        		this.setActiveColor(speedbet[i]);
        		betslide.addChangeListener(this);
        	}
    }
	
	public void mouseEntered(MouseEvent e) { }

	public void mouseExited(MouseEvent e) { }

	public void mousePressed(MouseEvent e) { }

	public void mouseReleased(MouseEvent e) { }

	public void keyPressed(KeyEvent e) { }

	public void keyReleased(KeyEvent e) {
		if(verify(betinput)){
			this.setActiveColor(null);
			betinput.setForeground(Color.black);
			double bet = Double.valueOf(betinput.getText());
			String s = "";
			if (betNotRaise) {
				s = "Bet(";
			}
			else s = "Raise(";
			if (bet < min) bet = min;
			if (bet > max) bet = max;
			if (bet >= bank) s = "All-in(";
			raise.setText(s+bet+")");
			raise.setAmount(bet);
		} else betinput.setForeground(Color.red);
	}
	public void keyTyped(KeyEvent e) {}

	public void stateChanged(ChangeEvent e) {
		this.setActiveColor(null);
		betinput.setForeground(Color.black);
		betinput.setText(""+betslide.getValue());
		String s = "";
		if (betNotRaise) {
			s = "Bet(";
		}
		else s = "Raise(";
		if (betslide.getValue()==bank) s = "All-in(";
		raise.setText(s+betslide.getValue()+")");
		raise.setAmount(betslide.getValue());
	}
}

