package com.asdf.plugins.pokergames.gameview;
import com.asdf.common.Action;
import com.asdf.common.PokerGameTypes;
import com.asdf.plugins.pokergames.PokerGameController;

import java.awt.BorderLayout;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.border.Border;


public class GameViewButtonPanel extends JPanel implements ActionListener{
	
	private static final long serialVersionUID = 1L;
	private final int CHECK = 0;
	private final int FOLD = 1;
	private final int CALL = 2;
	private final int RAISE = 3;
	
	private final int BETBUTTONS = 0;
	private final int POSTBLIND = 1;
	private final int MUCKCARDS = 2;
	private final int SITOUT = 3;
	private final int DISCARD = 4;
	
	private final int W_CHECK_FOLD = 0;
	private final int W_FOLD = 1;
	private final int W_CHECK = 2; 
	private final int W_CALL_ANY = 3;
	private final int W_RAISE_ANY = 4;
	private final int W_CALL = 5;
	private final int W_RAISE = 6;
	
	private PokerGameController gc = null;
	private final Border grayBorder = BorderFactory.createLineBorder(Color.darkGray);
	private JLayeredPane view;
	private JPanel waitgrid = new JPanel();
	private JLabel bet = new JLabel("0",JLabel.CENTER);
	private JLabel pot = new JLabel("0",JLabel.CENTER);
	private JPanel[] buttonpanel;
	private GameViewBetmenu betmenu = null;
	private GameViewGameButton[] actionButton;
	private JCheckBox[] waitGridBox;
	private JCheckBox discard = new JCheckBox("Discard");
	private JPanel grid = new JPanel();
	private double lastbet=0;
	public PokerGameTypes gametype;
	
	/**
	 * sets Panels for pokerbuttons for following situations: 
	 * BETBUTTONS,POSTBLIND,MUCKCARDS,SITOUT,DISCARD.
	 */
	
	private void initPokerButtons(){
		ImageIcon image;
		
		URL url = getClass().getResource("images/gamebutton.png");
		if (url == null)
			image = new ImageIcon("images/gamebutton.png");
		else
			image = new ImageIcon(url);		
		
		String[] actionname = new String[] {"Check","Fold","Call","Raise",
				"Post blind","Wait","Muck cards","Show cards","Im back","Discard"};
		int[] actions = new int[] {Action.CHECK, Action.FOLD, Action.CALL,
				Action.RAISE, Action.POST_BLIND, Action.SKIP_TURN,
				Action.MUCK,Action.REVEAL,Action.BACK,Action.DISCARD};
	
		actionButton = new GameViewGameButton[actionname.length];
	
		for(int i=0;i<actionname.length;i++)
			actionButton[i] = new GameViewGameButton(actionname[i],actions[i],Color.black,image,this);
	
		buttonpanel = new JPanel[5];				
		for (int i=0;i<buttonpanel.length;i++) buttonpanel[i]= new JPanel();
		
		buttonpanel[BETBUTTONS].setLayout(new GridLayout(2,2));
		buttonpanel[POSTBLIND].setLayout(new GridLayout());
		buttonpanel[MUCKCARDS].setLayout(new GridLayout());
		buttonpanel[DISCARD].setLayout(new GridLayout());
		
		buttonpanel[SITOUT].setLayout(new GridLayout());
		buttonpanel[BETBUTTONS].add(actionButton[0]);
		buttonpanel[BETBUTTONS].add(actionButton[1]);
		buttonpanel[BETBUTTONS].add(actionButton[2]);
		buttonpanel[BETBUTTONS].add(actionButton[3]);
		buttonpanel[POSTBLIND].add(actionButton[4]);
		buttonpanel[POSTBLIND].add(actionButton[5]);
		buttonpanel[MUCKCARDS].add(actionButton[6]);
		buttonpanel[MUCKCARDS].add(actionButton[7]);
		buttonpanel[SITOUT].add(actionButton[8]);
		buttonpanel[DISCARD].add(actionButton[9]);

		if (gametype.isPotLimit() || gametype.isNoLimit())
			betmenu = new GameViewBetmenu(view,actionButton[3],gametype);
		
	}
	
	/**
	 * sets Panel of checkboxes for actions that pokerplayer can can predefine when its not his turn 
	 */
			
	private void initPokerGrid(){
		waitgrid.setLayout(new GridLayout (4,2));
		waitGridBox = new JCheckBox[8];
		int i=0;
		waitgrid.setVisible(true);
		waitgrid.add(waitGridBox[i++] = new JCheckBox ("Check/Fold"));
		waitgrid.add(waitGridBox[i++] = new JCheckBox ("Fold"));
		waitgrid.add(waitGridBox[i++] = new JCheckBox ("Check"));
		if (gametype.isDiscardable()) waitgrid.add(discard);
			else waitgrid.add(new JLabel(""));
		waitgrid.add(waitGridBox[i++] = new JCheckBox ("Call any"));
		waitgrid.add(waitGridBox[i++] = new JCheckBox ("Raise any"));
		waitgrid.add(waitGridBox[i++] = new JCheckBox ("Call"));
		waitgrid.add(waitGridBox[i++] = new JCheckBox ("Raise"));
		
		discard.setFont(new Font("verdana", Font.PLAIN, 9));
		for (JCheckBox jbox : waitGridBox)
			if (jbox != null){
				jbox.setFont(new Font("verdana", Font.PLAIN, 9));
				jbox.addActionListener(this);
			}
	}
	
	/**
	 * Creates button panel of given gametype (poker only atm)
	 * 
	 * @param game GameType
	 * @param table JLayeredpane to attach possible betmenu.
	 */
	
	public GameViewButtonPanel(PokerGameTypes game, JLayeredPane table){
		gametype= game;
		view = table;
		if (game.isPokerGame()){
			initPokerButtons();
			initPokerGrid();
	
			GridLayout temp = new GridLayout(2,2);
			temp.setHgap(2);
			JPanel potbet = new JPanel();
			potbet.setLayout(temp);
			potbet.setPreferredSize(new Dimension(100,25));
			
			for (JLabel jlabel : new JLabel[]{new JLabel("BET",JLabel.CENTER),new JLabel("POT",JLabel.CENTER),bet,pot}){
				if (jlabel != bet && jlabel != pot){
					jlabel.setForeground(Color.white);
					jlabel.setBackground(Color.black);
				} else jlabel.setBorder(grayBorder);
				jlabel.setOpaque(true);
				jlabel.setFont(new Font("verdana", Font.PLAIN, 10));
				potbet.add(jlabel);
			}
			
			grid.setLayout(new GridLayout());
			grid.setPreferredSize(new Dimension(100,75));
			
			this.setLayout(new BorderLayout(2,2));
			this.add(potbet,BorderLayout.NORTH);
			this.add(grid,BorderLayout.CENTER);
		}
	}

	public void updateX() {
		betmenu.updateUI();
		updateUI();
	}

	/**
	 * Moves betmenu (if any) to southeast corner of given cordinates.
	 *   
	 */
	
	public void setBetMenuSize(int x,int y){
		if (betmenu!=null)
			betmenu.setBounds(x-300,y-45,300,45);
	}
	
	/**
	 * Sets controller to this buttonpanel and clears GUI.
	 *   
	 */
	
	public void setController(PokerGameController pgc){
		synchronized(this){
			this.setAllHidden();
			gc=pgc;
		}
	}
	
	/**
	 * Sends sitout action to controller and clears autositout checkbox for comeback
	 *  
	 * @param gcbp Panel where autositout checkbox is located.
	 */
	
	public void sendSitOutToController(GameViewCheckBoxPanel gcbp){
		synchronized(this){
			if (gc == null) return;
			gc.sendAction(Action.sitOutAction(), gc.getOurPlayer());
			gcbp.setSitout(false);
		}
	}
	
	/**
	 * Sends raiseaction to controller with given amount of money.
	 * 
	 * @param amount Amount of money raised.
	 */
	
	public void sendRaiseToController(double amount){
		synchronized(this){
			if (gc == null)
				return;
			gc.setNextAction(Action.RAISE, amount);
			this.setAllHidden();
		}
	}
	
	/**
	 * Sends leaveaction to controller.
	 * 
	 * @param closetable Do we close remove controller too. 
	 */
	
	public void sendActionLeaveToController(boolean closetable){
		synchronized(this){
			if (gc == null)
				return;
				gc.removePlayer(gc.getOurPlayer(),closetable);
		}
	}
	
	/**
	 * Sends action to controller. 
	 * 
	 * @param action Action that we want to send.
	 */
	
	public void sendActionToController(int action){
		synchronized(this){
			this.setAllHidden();
			if (gc == null)
				return;
			if (action==com.asdf.common.Action.BACK)
				gc.playerBack(gc.getOurPlayer());
			else 
				gc.setNextAction(action);
		}
	}
	
	/**
	 * hides all components from button panel.
	 * except pot+bet.
	 */
	
	public void setAllHidden(){
		if (betmenu!=null)betmenu.setVisible(false);
		grid.removeAll();
		this.updateUI();
	}
	
	/**
	 * Sets values to bot and bet
	 *  
	 * @param bet_ Current bet value.
	 * @param pot_ Current pot value.
	 */
	
	public void setBetAndPot(double bet_,double pot_){
		bet.setText(""+bet_);
		pot.setText(""+pot_);
	}
	
	/**
	 * Sets discard button panel. Contains button "DISCARD".
	 *
	 */
	
	public void setDiscardButton(){
		this.setAllHidden();
		grid.add(buttonpanel[DISCARD]);
	}
	
	/**
	 * Sets sitout button panel. Contains button "Im back".
	 *
	 */
	
	public void setSitOutButton(){
		this.setAllHidden();
		grid.add(buttonpanel[SITOUT]);
	}
	
	/**
	 * Sets post blind buttons. Contains buttons "post" and "wait". 
	 *
	 */
	
	public void setPostBlindButtons() {
		this.setAllHidden();
		grid.add(buttonpanel[POSTBLIND]);
	}
	
	/**
	 * Sets muck buttons. Contains buttons "muck cards" and "show cards".
	 *
	 */
	
	public void setMuckButtons(){
		this.setAllHidden();
		grid.add(buttonpanel[MUCKCARDS]);
	}
	
	/**
	 * Sets actionbuttons CHECK, FOLD, CALL, RAISE and activates betmenu.
	 * activated buttons and menu are calculated from gamesituation 
	 * and only buttons that are valid are shown
	 * 
	 * @param amounttocall How much player has to pay to call.
	 * @param pot Pot size.
	 * @param bank Players bankroll.
	 * @param amountraisable Howmuch is it possible for this player to raise.
	 * @param minraise Minimum raise.
	 */
	
	public void setPokerButtons(double amounttocall,
			double pot,
			double bank,
			double amountraisable,
			double minraise){
		
		grid.removeAll();
		
		// Check if there is possible predefined speedpot action. 
		for (JCheckBox jbox : waitGridBox)
			if (jbox!=null && jbox.isSelected()){
				if (jbox==waitGridBox[W_CHECK] || jbox == waitGridBox[W_CHECK_FOLD])
					this.sendActionToController(Action.CHECK);
				if (jbox==waitGridBox[W_FOLD])
					this.sendActionToController(Action.FOLD);
				if (jbox==waitGridBox[W_RAISE] || jbox == waitGridBox[W_RAISE_ANY])
					this.sendRaiseToController(minraise);
				if (jbox==waitGridBox[W_CALL] || jbox == waitGridBox[W_CALL_ANY])
					this.sendActionToController(Action.CALL);
				jbox.setSelected(false);
				waitGridBox[W_RAISE_ANY].setVisible(false);
				return;
			}
		
		grid.add(buttonpanel[this.BETBUTTONS]);
		
		actionButton[CHECK].setVisible(amounttocall==0);
		actionButton[FOLD].setVisible(true);
		actionButton[CALL].setVisible(true);
		
		if (amounttocall >= bank) {
			actionButton[CALL].setText("all-in("+bank+")"); 
			actionButton[RAISE].setVisible(false);
			return;
		}
		actionButton[CALL].setVisible(amounttocall!=0);
		actionButton[CALL].setText("Call("+amounttocall+")");
		
		if (amountraisable==0){
			actionButton[RAISE].setVisible(false);
			return;
		}
		
		if (minraise > amountraisable) {
			actionButton[RAISE].setAmount(bank);
			actionButton[RAISE].setText("all-in("+bank+")");
			return;
		}
		
		actionButton[RAISE].setVisible(true);
		actionButton[RAISE].setAmount(minraise);
		
		boolean isBet = false;
		if (amounttocall == 0) {
			actionButton[RAISE].setText("Bet("+minraise+")");
			isBet = true;
		}
		else actionButton[RAISE].setText("Raise("+minraise+")");
		if (betmenu!=null) betmenu.update(bank,minraise,amountraisable,pot,isBet);
	}
	/**
	 * Unvalidate options from grid that are no longer
     * valid because of changed gamesituation.  
	 *   
	 * @param minraise Minimumraise.
	 * @param amttocall Amount to call.
	 * @param bank Players Bankroll.
	 */
	public void updateWaitGrid(double minraise, double amttocall, double bank){
		if (waitGridBox[W_CHECK_FOLD].isSelected() && amttocall > 0)
				waitGridBox[W_FOLD].setSelected(true);
		if (amttocall > 0){
			waitGridBox[W_CHECK_FOLD].setSelected(false);
			waitGridBox[W_CHECK].setSelected(false);
			waitGridBox[W_CALL].setVisible(true);
			} else waitGridBox[W_CALL].setVisible(false);
		if ((waitGridBox[W_CALL].isSelected() && amttocall != lastbet) || amttocall == 0)
				waitGridBox[W_CALL].setSelected(false);
		if (waitGridBox[W_RAISE].isSelected() && amttocall != lastbet)
				waitGridBox[W_RAISE].setSelected(false);
		if (waitGridBox[W_RAISE_ANY] != null && waitGridBox[W_RAISE_ANY].isSelected())
			if (amttocall >= bank){
				waitGridBox[W_RAISE_ANY].setSelected(false);
				waitGridBox[W_CALL].setSelected(true);
			} else if (minraise <= amttocall){
				waitGridBox[W_RAISE_ANY].setSelected(false);
				waitGridBox[W_RAISE].setSelected(true);}
		if (waitGridBox[W_CALL_ANY] != null && waitGridBox[W_CALL_ANY].isSelected() && amttocall >= bank){
				waitGridBox[W_CALL_ANY].setSelected(false);
				waitGridBox[W_CALL].setSelected(true);}
		lastbet=amttocall;
	}
	
	/**
	 * Sets predefined-action grid for current gamesituation.
	 *   
	 * @param amttocall
	 * @param amtraisable
	 * @param minraise
	 * @param bank
	 */
	
	public void setWaitGrid(double amttocall,double amtraisable,double minraise,double bank){
		if (betmenu!=null)betmenu.setVisible(false);
		waitGridBox[W_FOLD].setVisible(true);		
		waitGridBox[W_CHECK].setVisible(amttocall == 0);
		waitGridBox[W_CHECK_FOLD].setVisible(amttocall == 0);
		waitGridBox[W_RAISE].setVisible(bank >= amttocall && amtraisable !=0);
		if (minraise >= bank)
			waitGridBox[W_RAISE].setText("All-in("+bank+")");
		else if (amttocall == 0)
			waitGridBox[W_RAISE].setText("bet("+minraise+")");
		else waitGridBox[W_RAISE].setText("raise("+minraise+")");
		if (waitGridBox[W_RAISE_ANY]!=null)
			waitGridBox[W_RAISE_ANY].setVisible(minraise < bank && amtraisable !=0);
		
		if (waitGridBox[W_CALL_ANY]!=null)
			waitGridBox[W_CALL_ANY].setVisible(bank >= amttocall);
		waitGridBox[W_CALL].setVisible(amttocall != 0);
		if (amttocall >= bank)
			waitGridBox[W_CALL].setText("All-in("+bank+")");
		else waitGridBox[W_CALL].setText("Call("+amttocall+")");
		grid.removeAll();
		grid.add(waitgrid);
	}
	
	/**
	 * Actionhandler that makes sure that only one or none is selected from waitgrid. 
	 * needed because radiobutton/boxgroup cannot de-select once you have selected once.  
	 * 
	 */
	
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		for (int i = 0; i < waitGridBox.length; i++)
			if (waitGridBox[i] != null)
				if (source == waitGridBox[i])
					waitGridBox[i].setSelected(waitGridBox[i].isSelected());
				else
					waitGridBox[i].setSelected(false);
	}
}
