package com.asdf.plugins.pokergames.gameview;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.URL;



import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;

import javax.swing.JLabel;

import javax.swing.JPanel;
import javax.swing.border.Border;

public class GameViewCheckBoxPanel extends JPanel implements MouseListener {

	private static final long serialVersionUID = 1L;

	private final Border etcBorder = BorderFactory.createEtchedBorder();
	
	private JCheckBox muckW, muckL, autoBlind, sitout, audioDisabled;
	private JLabel leave,lobby;
	private GameView gameview;
	
	/**
	 * Checkboxpanel for general checkboxes related to game.
	 * Contains also Leavegame button and lobbybutton
	 * 
	 * @param gv Reference to GameView that leave game button closes.
	 */
	
	GameViewCheckBoxPanel (GameView gv) {
		gameview = gv;
		ImageIcon image;
		URL url = getClass().getResource("images/checkpanelbutton.png");
		if (url == null)
			image = new ImageIcon("images/checkpanelbutton.png");
		else
			image = new ImageIcon(url);
		
		leave = new JLabel ("Leave",image,JLabel.CENTER);
		lobby = new JLabel ("Lobby",image,JLabel.CENTER);
		
		JPanel buttons = new JPanel();
		buttons.setLayout(new GridLayout());
		
		for (JLabel jlabel : new JLabel[] {leave,lobby}){
			jlabel.setVerticalTextPosition(JLabel.CENTER);
			jlabel.setHorizontalTextPosition(JLabel.CENTER);
			jlabel.setFont(new Font("verdana", Font.PLAIN, 10));
			jlabel.setForeground(Color.white);
			jlabel.setBackground(Color.pink); // pink gives cool etched border!!
			jlabel.setOpaque(true);
			jlabel.setBorder(etcBorder);
			jlabel.addMouseListener(this);
			buttons.add(jlabel);
		}
		
		JPanel checkboxes = new JPanel();
		checkboxes.setLayout(new GridLayout(5,1));
		checkboxes.add(sitout = new JCheckBox("Sit out"));
		checkboxes.add(autoBlind = new JCheckBox("Autopost blinds"));
		checkboxes.add(muckW = new JCheckBox("Muck winning hands"));
		checkboxes.add(muckL = new JCheckBox("Muck Losing hands"));
		//checkboxes.add(audioDisabled = new JCheckBox("Disable sound"));
		
		autoBlind.setSelected(true);
		//audioDisabled.setSelected(true);
		
		for (JCheckBox jbox : new JCheckBox[] {sitout,autoBlind,muckW,muckL,/*audioDisabled*/})
			jbox.setFont(new Font("verdana", Font.PLAIN, 10));
		
		this.setLayout(new BorderLayout(2,2));
		this.add(buttons,BorderLayout.NORTH);
		this.add(checkboxes,BorderLayout.LINE_START);
		buttons.setPreferredSize(new Dimension(40,25));
	}
	
	/**
	 * Toggles sitout button on/off
	 */
	
	public void setSitout(Boolean b){
		sitout.setSelected(b);
	}
	
	/**
	 * @return Is checkbox muck winning hands selected.
	 * 
	 */
	
	public boolean isMuckWinningHands() {
		return muckW.isSelected();
	}
	
	/**
	 * @return Is checkbox muck losing hands selected.
	 * 
	 */
	
	public boolean isMuckLosingHands() {
		return muckL.isSelected();
	}
	
	/**
	 * @return Is checkbox auto post blind selected.
	 * 
	 */
	
	public boolean isAutoPostBlind() {
		return autoBlind.isSelected();
	}

	/**
	 * @return Is checkbox sitout selected.
	 * 
	 */
	
	public boolean isSitOut() {
		return sitout.isSelected();
	}
	
	/**
	 * @return is checkbox audio disabled selected.
	 * 
	 */
	public boolean isAudioDisabled() {
		//return audioDisabled.isSelected();
		return true;
	}
	
	/**
	 * Lobby- and Leavebutton functionality.
	 *  
	 */
	
	public void mouseClicked(MouseEvent e) {
		Object source = e.getSource();
		if (source == leave)
			gameview.closeWindow();
		else if (source == lobby)
			com.asdf.client.LobbyController.lobbyToFront(); 
	}

	public void mouseEntered(MouseEvent e) {
		Object source = e.getSource();
		if (source==leave){
			leave.setForeground(Color.blue);
			leave.setBackground(Color.blue);
		} else if (source==lobby){
			lobby.setForeground(Color.blue);
			lobby.setBackground(Color.blue);
		}
			
	}

	public void mouseExited(MouseEvent e) {
		Object source = e.getSource();
		if (source==leave){
			leave.setForeground(Color.white);
			leave.setBackground(Color.pink);
		} else if (source==lobby){
			lobby.setForeground(Color.white);
			lobby.setBackground(Color.pink);
		}
	}
	
	public void mousePressed(MouseEvent e) { }

	public void mouseReleased(MouseEvent e) { }
}