package com.asdf.plugins.pokergames.gameview;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;


public class GameViewTableCards extends JPanel {
	private static final long serialVersionUID = 1L;
	private final Border grayBorder = BorderFactory.createLineBorder(Color.darkGray);
	private JLayeredPane view;
	private JLabel[] cards = new JLabel[5];
	private String[] tcards = {"H1","H13","H12","H11","H10"};
	private JLabel bet = new JLabel("");
	
	/**
	 *  New table cards, cards are drawn into JLayeredPane given as parameter
	 *  
	 */
	
	public GameViewTableCards(JLayeredPane table){	
		view=table;
		  for (int i=0; i < tcards.length;i++) {
			  cards[i] = new JLabel("");
			  cards[i].setBackground(Color.white);
			  cards[i].setOpaque(true);	
			  cards[i].setBorder(grayBorder);
			  view.add(cards[i]);
			  cards[i].setVisible(false);
			  view.setLayer(cards[i],2000+i);
		  }
		  bet.setFont(new Font("verdana", Font.PLAIN, 10));
		  bet.setForeground(Color.white);
		  view.add(bet);
		  view.setLayer(bet,3000);
	}
	
	public void updateX() {
		for (int i=0; i<cards.length; i++) 
			cards[i].updateUI();
		bet.updateUI();
		view.updateUI();
		updateUI();
	}
	
	
	/**
	 *  Sets pot visible just below table cards.
	 *  
	 */
	
	public void setbet(String s){
		bet.setText(s);
	}
	
	/**
	 * sets this given card as enabled or disabled.
	 * @param nro Card number
	 * @param visible Switch do enabled/disabled.
	 */
	
	public void enableCard(int nro,boolean visible){
		cards[nro].setEnabled(visible);
	}
	
	/**
	 * sets all cards as enabled or disabled.
	 * @param visible Switch do enabled/disabled.
	 */
	
	public void enableCards(boolean visible){
		for (int i=0;i<cards.length;i++)
			cards[i].setEnabled(visible);
	}
	/**
	 * Calculate number of visible cards.
	 * 
	 * @return Number of visible cards 
	 */
	
	public int getVisibleCards() {
		int visibleCards = 0;
		for(int i = 0; i < cards.length; i++)
			if (cards[i].isVisible())
				visibleCards++;
		if (visibleCards==0)
			this.enableCards(true);
		return visibleCards;
	}
	
	/**
	 * Sets cards and loads images to them with given ratio.
	 *  
	 * @param cardlist String list of tablecards "C1","H2" and so on...
	 * @param xratio Width percentage 
	 * @param yratio Height percentage
	 */

	public void setCards (String[] cardlist,double xratio,double yratio) {
		if (cardlist==null) for (int i=0;i<cards.length;i++)
			cards[i].setVisible(false);
		else {
			// set tablecards as given string 
			for (int i=0;i < cardlist.length && i <tcards.length;i++){
				cards[i].setIcon(GameView.getIconFromBufferedimage(cardlist[i],xratio,yratio));
				cards[i].setVisible(true);
				tcards[i]=cardlist[i];
		    }
			// makes sure that there are no extra cards shown 
			for (int i=cardlist.length;i<tcards.length;i++)
				cards[i].setVisible(false);
		}
	}
	
	/**
	 *  Resizes cards and set locations of them in middle of table.
	 *  X and y represents size of gameview window.
	 *  Xratio and pratio are given as how big cards space we give.
	 *  
	 */
	
	public void bounds(int x,int y,double xratio, double pratio) {
		if(x == 0 || y == 0) return;
		int cardX = (int)(GameView.cardsizeX*xratio);
		int cardY = (int)(GameView.cardsizeY*pratio);
		bet.setBounds(
					x/2-cardX-cardX/2-6,
					y/2+cardY/2+cardY/8,
					100,
					12);
		
		for(int i=0; i < cards.length; i++){
		  cards[i].setBounds(
				   x/2+(cardX+3)*i-(cardX*2)-cardX/2-6,
				   y/2-cardY/2+cardY/8,
				   cardX,
				   cardY);
		  cards[i].setIcon(GameView.getIconFromBufferedimage(tcards[i],xratio,pratio)); 
		}
	}
}
