package com.asdf.plugins.pokergames.texasholdem;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

import javax.imageio.metadata.IIOMetadataNode;
import javax.swing.BorderFactory;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

import org.w3c.dom.Element;

import com.asdf.common.PokerGameTypes;
import com.asdf.gui.lobby.CreateGamePanel;
import com.asdf.parser.CreateGameRequest;
import com.asdf.parser.Parameters;

/**
 * Panel used to create Texas holdem games to server. Used in lobby.
 * 
 * @author asdf
 *
 */
public class TexasCreateGamePanel extends CreateGamePanel {
	
	/**
	 * Default serial.
	 */
	private static final long serialVersionUID = 1L;
	
    private JPanel topPanel;
    private JPanel bottomPanel;
    private JButton createButton;
    private JComboBox gameTypeSelector;
    private JLabel gameTypeLabel;
    private JLabel maxPlayersLabel;
    private JLabel minPlayersLabel;
    private JLabel smallBlindLabel;
    private JLabel bigBlindLabel;
    private JCheckBox simulation;
    private JTextField minPlayers;
    private JTextField maxPlayers;
    private JTextField timeToAct;
    private JLabel timeToActLabel;
    private JTextField bigBlind;
    private JTextField smallBlind;
    private JTextField nameField;
    private JLabel name;

    private boolean valid = false;

	{
		//Set Look & Feel
		try {
			javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	* Auto-generated main method to display this 
	* JPanel inside a new JFrame.
	*/
	public static void main(String[] args) {
		JFrame frame = new JFrame();
		frame.getContentPane().add(new TexasCreateGamePanel());
		frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		frame.pack();
		frame.setVisible(true);
	}
	
	/**
	 * Sole constructor which initializes this panel.
	 */
    public TexasCreateGamePanel(){
    	super();
    	// new add
    	Runnable doWorkRunnable = new Runnable() {
		    public void run() { 
		    	initGUI();		    
		    }
		};
		SwingUtilities.invokeLater(doWorkRunnable);
    	//initGUI();
    	setInitialized(true);    
    }	
    /**
     * Initializes this panel.
     */
    public void initGUI() {
        try {
        	GridBagLayout thisLayout = new GridBagLayout();
            thisLayout.rowWeights = new double[] {0.0, 0.0, 0.0, 0.0, 0.0, 0.1};
            thisLayout.rowHeights = new int[] {80, 9, 111, 9, 35, 2};
            thisLayout.columnWeights = new double[] {0.1};
            thisLayout.columnWidths = new int[] {7};
            this.setLayout(thisLayout);
            setPreferredSize(new Dimension(400, 300));
            {
                topPanel = new JPanel();
                GridBagLayout topPanelLayout = new GridBagLayout();
                this.add(topPanel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
                topPanelLayout.rowWeights = new double[] {0.1, 0.1};
                topPanelLayout.rowHeights = new int[] {7, 7};
                topPanelLayout.columnWeights = new double[] {0.0, 0.1};
                topPanelLayout.columnWidths = new int[] {99, 7};
                topPanel.setLayout(topPanelLayout);
                topPanel.setBorder(BorderFactory.createTitledBorder("New Game"));
                {
                    name = new JLabel();
                    topPanel.add(name, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                    name.setText("Name");
                    name.setHorizontalAlignment(JLabel.CENTER);
                }
                {
                    nameField = new JTextField();
                    topPanel.add(nameField, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                }
                {
                	//simulation = new JCheckBox();
                	//topPanel.add(simulation, new GridBagConstraints(0, 2, 2, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 30, 0, 0), 0, 0));
                	//simulation.setText("Simulation game");
                }
                {
                	gameTypeLabel = new JLabel();
                	topPanel.add(gameTypeLabel, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                	gameTypeLabel.setText("Game type");
                	gameTypeLabel.setHorizontalAlignment(JLabel.CENTER);
                }
                {
                	ComboBoxModel gameTypeSelectorModel = 
                		new DefaultComboBoxModel(
                				new String[] { "Limit", "No-Limit", "Pot-Limit" });
                	gameTypeSelector = new JComboBox();
                	topPanel.add(gameTypeSelector, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                	gameTypeSelector.setModel(gameTypeSelectorModel);
                }
            }
            {
                bottomPanel = new JPanel();
                GridBagLayout jPanel1Layout = new GridBagLayout();
                jPanel1Layout.rowHeights = new int[] {7, 7, 7, 7, 7};
                jPanel1Layout.columnWeights = new double[] {0.0, 0.1};
                jPanel1Layout.rowWeights = new double[] {0.1, 0.1, 0.1, 0.1, 0.1};
                jPanel1Layout.columnWidths = new int[] {126};
                this.add(bottomPanel, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
                bottomPanel.setLayout(jPanel1Layout);
                bottomPanel.setBorder(BorderFactory.createTitledBorder("Game Options"));
                {
                	bigBlindLabel = new JLabel();
                	bottomPanel.add(bigBlindLabel, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
                	bigBlindLabel.setText("Big blind");
                	bigBlindLabel.setHorizontalAlignment(JLabel.CENTER);
                }
                {
                	smallBlindLabel = new JLabel();
                	bottomPanel.add(smallBlindLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
                	smallBlindLabel.setText("Small blind");
                	smallBlindLabel.setHorizontalAlignment(JLabel.CENTER);
                }
                {
                	minPlayersLabel = new JLabel();
                	bottomPanel.add(minPlayersLabel, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
                	minPlayersLabel.setText("Minimum players");
                	minPlayersLabel.setHorizontalAlignment(JLabel.CENTER);
                }
                {
                	maxPlayersLabel = new JLabel();
                	bottomPanel.add(maxPlayersLabel, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
                	maxPlayersLabel.setText("Maximum players");
                	maxPlayersLabel.setHorizontalAlignment(JLabel.CENTER);
                }
                {
                	smallBlind = new JTextField();
                	bottomPanel.add(smallBlind, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
                	smallBlind.addFocusListener(new FocusAdapter() {
                        public void focusLost(FocusEvent evt) {
                            verifyAll();
                        }
                    });
                	smallBlind.addKeyListener(new KeyAdapter() {
                        public void keyReleased(KeyEvent evt) {
                            verifyAll();
                        }
                    });
                }
                {
                	minPlayers = new JTextField();
                	bottomPanel.add(minPlayers, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
                	minPlayers.addFocusListener(new FocusAdapter() {
                        public void focusLost(FocusEvent evt) {
                            verifyAll();
                        }
                    });
                	minPlayers.addKeyListener(new KeyAdapter() {
                        public void keyReleased(KeyEvent evt) {
                            verifyAll();
                        }
                    });
                }
                {
                	maxPlayers = new JTextField();
                	bottomPanel.add(maxPlayers, new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
                	maxPlayers.addFocusListener(new FocusAdapter() {
                        public void focusLost(FocusEvent evt) {
                            verifyAll();
                        }
                    });
                	maxPlayers.addKeyListener(new KeyAdapter() {
                        public void keyReleased(KeyEvent evt) {
                            verifyAll();
                        }
                    });
                }
                {
                    bigBlind = new JTextField();
                    bottomPanel.add(bigBlind, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
                    bigBlind.addFocusListener(new FocusAdapter() {
                        public void focusLost(FocusEvent evt) {
                            verifyAll();
                        }
                    });
                    bigBlind.addKeyListener(new KeyAdapter() {
                        public void keyReleased(KeyEvent evt) {
                            verifyAll();
                        }
                    });
                }
                {
                	timeToActLabel = new JLabel();
                	bottomPanel.add(timeToActLabel, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                	timeToActLabel.setText("Time to act");
                	timeToActLabel.setHorizontalAlignment(JLabel.CENTER);
                }
                {
                	timeToAct = new JTextField();
                	bottomPanel.add(timeToAct, new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                	timeToAct.addFocusListener(new FocusAdapter() {
                        public void focusLost(FocusEvent evt) {
                            verifyAll();
                        }
                    });
                	timeToAct.addKeyListener(new KeyAdapter() {
                        public void keyReleased(KeyEvent evt) {  
                            verifyAll();
                        }
                    });
                
                }
            }
            {
                createButton = new JButton();
                this.add(createButton, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 1, 1));
                createButton.setText("Create");
                
                /*  When button is clicked input is verified and message to be
                 *  sent to server is created.
                 */ 
                createButton.addActionListener(new ActionListener() {
                	public void actionPerformed(ActionEvent evt) {
                	    
                	    //boolean valid = true;
                	    bigBlind.setForeground(Color.black);
                	    smallBlind.setForeground(Color.black);
                	    minPlayers.setForeground(Color.black);
                	    maxPlayers.setForeground(Color.black);
                	    
                		CreateGameRequest createReq = new CreateGameRequest();                		
                		createReq.setGameType("TexasHold'em");
                		//createReq.setGameType("com.asdf.plugins.pokergames.texasholdem.TexasHoldemGame");
                		String gameName = nameField.getText();
                		if (gameName.length() == 0)
                			gameName = "Unnamed game";
                		createReq.setName(gameName);
                		Parameters par = new Parameters();                   		
                		
                		Element e = new IIOMetadataNode("any");               		
                		
                		
                		if(verifyD(smallBlind) && !smallBlind.getText().equalsIgnoreCase(""))  
                		    e.setAttribute("smallBlind", ""+new Double(smallBlind.getText()));   
                        else{
                            //smallBlind.setForeground(Color.RED);
                            valid = false;
                        }
                		if(verifyD(bigBlind) && !bigBlind.getText().equalsIgnoreCase(""))              		
            		        e.setAttribute("bigBlind", ""+new Double(bigBlind.getText()));    
                		else{
                		    //bigBlind.setForeground(Color.RED);
                		    valid = false;
                		}
                		if(verify(minPlayers) && !minPlayers.getText().equalsIgnoreCase(""))  
                		    e.setAttribute("minPlayers", ""+new Integer(minPlayers.getText()));   
                        else{
                            //minPlayers.setForeground(Color.RED);
                            valid = false;
                        }
                		if(verify(maxPlayers) && !maxPlayers.getText().equalsIgnoreCase(""))  
                		    e.setAttribute("maxPlayers", ""+new Integer(maxPlayers.getText()));                
                        else{
                            //maxPlayers.setForeground(Color.RED);
                            valid = false;
                        }if(verify(timeToAct) && !timeToAct.getText().equalsIgnoreCase("")){
                        	e.setAttribute("timeToAct", ""+new Integer(timeToAct.getText()));
                        }
                		
                        String s = gameTypeSelector.getSelectedItem().toString();
                        
                        if(s.equalsIgnoreCase("Limit")){
                        	e.setAttribute("limit", "" + PokerGameTypes.HOLDEM_FIXED_LIMIT.ordinal());
                        }else if(s.equalsIgnoreCase("No-Limit")){
                        	e.setAttribute("limit", "" + PokerGameTypes.HOLDEM_NO_LIMIT.ordinal());
                        }else if(s.equalsIgnoreCase("Pot-Limit")){
                        	e.setAttribute("limit", "" + PokerGameTypes.HOLDEM_POT_LIMIT.ordinal());
                        }else{
                        	e.setAttribute("limit", "" + -1);
                        }
	              		
	              		//e.setAttribute("simulation", ""+simulation.isSelected());	       
	              		
	              		par.getAny().add(e);	     
	              			              		
	              		if(verifyAll() && !isEmptyAll()){
    	              		
                            createReq.setParameters(par);
                            getController().requestCreateGame(createReq);

	              		}

                	}
                });
            }
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }
    /**
     * Checks if all the fields are empty.
     * @return true if all are empty, false otherwise
     */
    private boolean isEmptyAll(){
    	if(bigBlind.getText().equalsIgnoreCase(""))
    		return true;
    	else if(smallBlind.getText().equalsIgnoreCase(""))
    		return true;
    	else if(minPlayers.getText().equalsIgnoreCase(""))
    		return true;
    	else if(maxPlayers.getText().equalsIgnoreCase(""))
    		return true;
    	else if(timeToAct.getText().equalsIgnoreCase(""))
    		return true;
    	else
    		return false;    	
    
    }
    /**
     * Verifies that the input in JTexField is a integer
     * @param input JTextField to be checked.
     * @return true if JTextField contains integer, false otherwise
     */
    private boolean verify(JTextField input) {
    
        String text = input.getText().trim();
        int i;
        try {
            if(text == null || text.equalsIgnoreCase(""))
                i = 0;
            else
                i = new Integer(text);            
         } catch (NumberFormatException e){
             return false;
         } 
         if(i >= 0)
             return true;
         else
             return false;
     }
    /**
     * Verifies that the input in JTexField is a double
     * @param input JTextField to be checked.
     * @return true if JTextField contains double, false otherwise
     */
    private boolean verifyD(JTextField input) {
        
        String text = input.getText().trim();
        double i;
        try {
            if(text == null || text.equalsIgnoreCase(""))
                i = 0;
            else
                i = new Double(text);            
         } catch (NumberFormatException e){
             return false;
         } 
         if(i >= 0)
             return true;
         else
             return false;
     }

	/**
	 * Method used to verify that all the input in text fields is correct.
	 * @return true if verified, false otherwise
	 */
	private boolean verifyAll(){
       
		valid = true;
		
        	if(!bigBlind.getText().equalsIgnoreCase("")){
                if(!verifyD(smallBlind)){
                    smallBlind.setForeground(Color.RED);
                    valid = false;
                    
                }else if(!smallBlind.getText().equalsIgnoreCase("") && verifyD(bigBlind) && (new Double(smallBlind.getText()) > new Double(bigBlind.getText()))){
                	smallBlind.setForeground(Color.RED); 
                	valid = false;
                	
            	}else{
                    smallBlind.setForeground(Color.BLACK);
                    //valid = true;
            	}
        	}else{
                if(!verifyD(smallBlind)){
                    smallBlind.setForeground(Color.RED);
                    valid = false;
                   
                }else{
                    smallBlind.setForeground(Color.BLACK);
                    //valid = true;
                }
        	}

        	if(!smallBlind.getText().equalsIgnoreCase("")){
                if(!verifyD(bigBlind)){
                	bigBlind.setForeground(Color.RED);
                	valid = false;
                	
                }else if(!bigBlind.getText().equalsIgnoreCase("") && verifyD(smallBlind) && (new Double(bigBlind.getText()) < new Double(smallBlind.getText()))){
                	bigBlind.setForeground(Color.RED);
                	valid = false;
                	
                }else{
                	bigBlind.setForeground(Color.BLACK);
                	//valid = true;
                }
        	}else{
                if(!verifyD(bigBlind)){
                	bigBlind.setForeground(Color.RED);
                	valid = false;
                	
                }else{
                	bigBlind.setForeground(Color.BLACK);
                	//valid = true;
                }
        	}     
   
        	if(!maxPlayers.getText().equalsIgnoreCase("")){
                if(!verify(minPlayers)){
                	minPlayers.setForeground(Color.RED);
                	valid = false;
                	
                }else if(!minPlayers.getText().equalsIgnoreCase("") && verify(maxPlayers) && (new Integer(minPlayers.getText()) > new Integer(maxPlayers.getText()))){
                	minPlayers.setForeground(Color.RED);
                	valid = false;
                	
                }else{
                	minPlayers.setForeground(Color.BLACK);
                	//valid = true;
                }
        	}else{
                if(!verify(minPlayers)){
                	minPlayers.setForeground(Color.RED);
                	valid = false;
                	
                }else{
                	minPlayers.setForeground(Color.BLACK);
                	//valid = true;
                }
        	}            
 
        	if(!minPlayers.getText().equalsIgnoreCase("")){
                if(!verify(maxPlayers)){
                	maxPlayers.setForeground(Color.RED);
                	valid = false;
                	
                }
                else if(!maxPlayers.getText().equalsIgnoreCase("") && verify(minPlayers) && (new Integer(maxPlayers.getText()) < new Integer(minPlayers.getText()))){
                	maxPlayers.setForeground(Color.RED);
                	valid = false;  
                	
                }else{
                	maxPlayers.setForeground(Color.BLACK);
                	//valid = true;
            	}
        	}else{
                if(!verify(maxPlayers)){
                	maxPlayers.setForeground(Color.RED);
                	valid = false;
                	
                }else{
                	maxPlayers.setForeground(Color.BLACK);
                	//valid = true;
                }
        	}            

	    	if(verify(timeToAct)){
	    		timeToAct.setForeground(Color.BLACK);
	    		//valid = true;
	    	}else{
    			timeToAct.setForeground(Color.RED);	    
    			valid = false;
    		
    		}
	    	
			return valid;
	    	
	    
	}

}

