package com.asdf.plugins.pokergames.texasholdem;

import java.util.SortedSet;
import java.util.TreeSet;

import javax.xml.bind.JAXBException;

import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import com.asdf.common.GameTimerTask;
import com.asdf.common.PokerGameTypes;
import com.asdf.parser.GameRequest;
import com.asdf.parser.OptionalParameters;
import com.asdf.parser.Parameters;
import com.asdf.parser.GameListResponse.Games.Game.AdditionalInfo;
import com.asdf.plugins.pokergames.Card;
import com.asdf.plugins.pokergames.EvalTables;
import com.asdf.plugins.pokergames.GameModel;
import com.asdf.plugins.pokergames.Hand;
import com.asdf.plugins.pokergames.HandEval;
import com.asdf.plugins.pokergames.PlayerPosition;
import com.asdf.plugins.pokergames.PokerGame;
import com.asdf.plugins.pokergames.Stage;
import com.asdf.plugins.pokergames.texasholdem.TexasHoldemStage.TexasHoldemEnumStage;
import com.asdf.utils.BasicElement;
import com.asdf.utils.XMLTools;

public class TexasHoldemGame extends PokerGame {
	
	
	public static EvalTables evalTables = new EvalTables();
	
	public TexasHoldemGame(){
		super.handEval = new HandEval(evalTables);
		try {
			super.xmltools = new XMLTools("com.asdf.plugins.pokergames.texasholdem.parser", "../Protocol/texas_message.xsd");
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/**
	 * 
	 * @param smallBlind
	 * @param bigBlind
	 * @param minPlayers
	 * @param maxPlayers
	 * @param gameID
	 */
	public TexasHoldemGame(double smallBlind, double bigBlind, int minPlayers, int maxPlayers, long gameID, int actionWait) {
		super.handEval = new HandEval(evalTables);
		super.generatorStage = new TexasHoldemStage(TexasHoldemEnumStage.WAIT_PLAYERS);
		super.seatReserveTimerTasks = new GameTimerTask[maxPlayers];
		super.freeSeatTimerTasks = new GameTimerTask[maxPlayers];
		this.gameID = gameID;
		this.gameModel = new GameModel(smallBlind, bigBlind, minPlayers, maxPlayers, gameID, new TexasHoldemStage(TexasHoldemEnumStage.WAIT_PLAYERS));
		this.gameModel.setStage(this.generatorStage);
		this.gameWatchers = new TreeSet<Long>();
		this.superUsers = new TreeSet<Long>();
		this.playerIDs = new TreeSet<Long>();
		this.hands = new Hand[maxPlayers];
		this.PlayerSessionIDs = new long[maxPlayers];
		this.won = new boolean[maxPlayers];
		super.timerSessionBind = new long[maxPlayers];
		ACTION_WAIT = BLIND_WAIT = DISCARD_WAIT = actionWait+2000;
		
	}

	@Override
	protected double getStartMinRaise() {
		if (this.gameModel.isFixedLimit())
			if (TexasHoldemStage.isTurn(this.gameModel.getStage().index()) || TexasHoldemStage.isRiver(this.gameModel.getStage().index())) {
				return 2*this.gameModel.getBigBlind();
			}
		return this.gameModel.getBigBlind();
	}
	
	/**
	 * Returns a description of the texas holdem game.
	 */
	@Override
	protected com.asdf.parser.GameListResponse.Games.Game description() {
		if (this.gameModel == null)
			return null;
		com.asdf.parser.GameListResponse.Games.Game gameDesc = new com.asdf.parser.GameListResponse.Games.Game();
		gameDesc.setGameID(this.getId());
		gameDesc.setGameType("TexasHold'em");
		gameDesc.setName(this.getName());
		gameDesc.setPlayers(this.gameModel.getSittingPlayers());
		gameDesc.setMaxPlayers(this.gameModel.getMaxPlayers());
		String variant;
		switch(PokerGameTypes.values()[this.gameModel.getLimitStyle()]) {
		case HOLDEM_FIXED_LIMIT:
			variant = "limit";
			break;
		case HOLDEM_POT_LIMIT:
			variant = "pot-limit";
			break;
		case HOLDEM_NO_LIMIT:
			variant = "no-limit";
			break;
		default: variant = "VARIANT NOT RECOGNIZED!!";
		}
		gameDesc.setGameVariant(variant);
		
		AdditionalInfo info = new AdditionalInfo();
		OptionalParameters opars = new OptionalParameters();		
		Parameters pars = new Parameters();
		
		
		Element e = new BasicElement("asdf");
		e.setAttribute("Limit", variant);
		e.setAttribute("Blinds", "" + gameModel.getSmallBlind() + " / " + gameModel.getBigBlind());
		//add more if needed...
		
		pars.getAny().add(e);
		opars.setParameters(pars);
		info.setOther(opars);
		
		gameDesc.setAdditionalInfo(info);

		return gameDesc;
	}


	@Override
	protected void handleRequest(long sessionID, GameRequest request) {
		super.handleRequest(sessionID, request);
		
		if(request != null && request.getCreate() != null){
			super.generatorStage = new TexasHoldemStage(TexasHoldemEnumStage.WAIT_PLAYERS);
			Element e = request.getCreate().getParameters().getAny().get(0);
			this.gameModel = new GameModel(
					new Double(e.getAttribute("smallBlind")), 
					new Double(e.getAttribute("bigBlind")), 
					new Integer(e.getAttribute("minPlayers")), 
					new Integer(e.getAttribute("maxPlayers")), 
					this.getId(), 
					new TexasHoldemStage(TexasHoldemEnumStage.WAIT_PLAYERS));
			this.gameWatchers = new TreeSet<Long>();
			this.superUsers = new TreeSet<Long>();
			this.playerIDs = new TreeSet<Long>();
			this.hands = new Hand[new Integer(e.getAttribute("maxPlayers"))];
			this.PlayerSessionIDs = new long[new Integer(e.getAttribute("maxPlayers"))];
			this.won = new boolean[new Integer(e.getAttribute("maxPlayers"))];
			super.seatReserveTimerTasks = new GameTimerTask[new Integer(e.getAttribute("maxPlayers"))];
			super.freeSeatTimerTasks = new GameTimerTask[new Integer(e.getAttribute("maxPlayers"))];
			super.timerSessionBind = new long[new Integer(e.getAttribute("maxPlayers"))];
			this.gameModel.setStage(this.generatorStage);
			ACTION_WAIT = BLIND_WAIT = DISCARD_WAIT = (new Integer(e.getAttribute("timeToAct")))*1000+1500;
			try{
				this.gameModel.setLimitStyle(new Integer(e.getAttribute("limit")));
			}catch(NumberFormatException ex){
				this.gameModel.setLimitStyle(PokerGameTypes.HOLDEM_FIXED_LIMIT.ordinal());
			}

		}else{
		
		}		
	}

	@Override
	protected String[] columnNames() {
		return new String[]{"Blinds", "Limit"}; //add more if necessary
	}

	@Override
	protected void newGame(double smallBlind, double bigBlind, int minPlayers,
			int maxPlayers, int gameID, Stage stage) {
		 {
				this.gameModel = new GameModel(smallBlind, bigBlind, minPlayers, maxPlayers, gameID, stage);
				this.sendUpdatedModelToAll();
			}
		
	}

	@Override
	/**
	 * returns an sorted collection of all players active in pot, starting with the player with the best hand,
	 * for players with equal hands the one who has invested less if first
	 * @return
	 */
	protected SortedSet<PlayerPosition> orderPlayersByHandAndPot() {
		TreeSet<PlayerPosition> handOrder = new TreeSet<PlayerPosition>();
		//add players active in pot to the tree
		int[] hand = new int[7];
		int index = 2;
		for (Card card : this.gameModel.getBoard().getCards())
			hand[index++] = card.getInt();
		for (int i = 0; i < this.gameModel.getMaxPlayers(); i++)
			if (this.gameModel.getPlayer(i) != null && 
					this.gameModel.isActive(i) && 
					this.hands[i] != null && 
					this.hands[i].getCard(0) != null && 
					this.hands[i].getCard(0).isValid() &&
					this.hands[i].getCard(1) != null && 
					this.hands[i].getCard(1).isValid()) {
				hand[0] = this.hands[i].getCard(0).getInt();
				hand[1] = this.hands[i].getCard(1).getInt();
				handOrder.add(new PlayerPosition(i, this.gameModel.getPlayer(i).getAmountInPot(), handEval.eval_7cards(hand)));
			}

		return handOrder;
	}

	@Override
	protected void setMinRaise(double amount) {
		if (this.gameModel.isFixedLimit()) {
			this.gameModel.setMinRaise(this.getStartMinRaise());
		}
		else if (this.gameModel.isPotLimit())
			this.gameModel.setMinRaise(this.gameModel.getBigBlindSize());
		else
			this.gameModel.setMinRaise(amount);
	}


}
