package com.asdf.plugins.pokergames.texasholdem;

import com.asdf.plugins.pokergames.PokerGame;
import com.asdf.plugins.pokergames.Stage;

public class TexasHoldemStage implements Stage {

	public enum TexasHoldemEnumStage {
		WAIT_PLAYERS (PokerGame.WAIT_PLAYERS, 0), 
	    SET_DEALER (PokerGame.SET_DEALER, 0), 
        BLIND_ROUND (PokerGame.BLIND_ROUND, 0), 
        NEW_DEAL (PokerGame.NEW_DEAL, 2), 
        BET_ROUND_PREFLOP (PokerGame.BET_ROUND, 0), 
        FLOP (PokerGame.DEAL_N_CARDS_TO_TABLE, 3), 
        BET_ROUND_FLOP (PokerGame.BET_ROUND, 0), 
        TURN (PokerGame.DEAL_N_CARDS_TO_TABLE, 1), 
        BET_ROUND_TURN (PokerGame.BET_ROUND, 0), 
        RIVER (PokerGame.DEAL_N_CARDS_TO_TABLE, 1), 
        BET_ROUND_RIVER (PokerGame.BET_ROUND, 0),
        LESS_THAN_TWO_PLAYERS (PokerGame.LESS_THAN_TWO_PLAYERS, 0),
        WINNINGS (PokerGame.WINNINGS, 0),
        MUCK_ROUND (PokerGame.MUCK_ROUND, 0),
        SHOWDOWN (PokerGame.SHOWDOWN, 0);

	public final int param;
	public final int correspondingCase;
	
	private TexasHoldemEnumStage(int correspondingCase, int param) {
		this.param = param;
		this.correspondingCase = correspondingCase;
	}
	
	public TexasHoldemEnumStage next() {
		switch (this) {
		case WAIT_PLAYERS: 
			return SET_DEALER;
		case SET_DEALER:
			return BLIND_ROUND;
		case BLIND_ROUND:
			return NEW_DEAL;
		case NEW_DEAL:
			return BET_ROUND_PREFLOP;
		case BET_ROUND_PREFLOP:
			return FLOP;
		case FLOP:
			return BET_ROUND_FLOP;
		case BET_ROUND_FLOP:
			return TURN;
		case TURN:
			return BET_ROUND_TURN;
		case BET_ROUND_TURN:
			return RIVER;
		case RIVER:
			return BET_ROUND_RIVER;
		case BET_ROUND_RIVER:
			return WINNINGS;
		case LESS_THAN_TWO_PLAYERS:
			return SHOWDOWN;
		case WINNINGS:
			return MUCK_ROUND;
		case MUCK_ROUND:
			return SHOWDOWN;
		case SHOWDOWN:
			return SET_DEALER;
		}
		return WAIT_PLAYERS;
		}
	};
	
	
	private final TexasHoldemEnumStage stage;
	
	public TexasHoldemStage(TexasHoldemEnumStage stage) {
		this.stage = stage;
	}
	
	public TexasHoldemStage() {
		this.stage = TexasHoldemEnumStage.WAIT_PLAYERS;
	}
	
	@Override
	public Stage lessThanTwoPlayers() {
		return new TexasHoldemStage(TexasHoldemEnumStage.LESS_THAN_TWO_PLAYERS);
	}

	@Override
	public int correspondingCase() {
		return this.stage.correspondingCase;
	}

	@Override
	public int index() {
		return this.stage.ordinal();
	}

	@Override
	public Stage next() {
		return new TexasHoldemStage(this.stage.next());
	}

	@Override
	public int param() {
		return this.stage.param;
	}

	@Override
	public Stage start() {
		return new TexasHoldemStage(TexasHoldemEnumStage.WAIT_PLAYERS);
	}


	public static boolean isFlop(int stage) {
		return stage == TexasHoldemEnumStage.FLOP.ordinal() || stage == TexasHoldemEnumStage.BET_ROUND_FLOP.ordinal();
	}


	public static boolean isPreFlop(int stage) {
		return stage == TexasHoldemEnumStage.NEW_DEAL.ordinal() || stage == TexasHoldemEnumStage.BET_ROUND_PREFLOP.ordinal();
	}


	public static boolean isRiver(int stage) {
		return stage == TexasHoldemEnumStage.RIVER.ordinal() || stage == TexasHoldemEnumStage.BET_ROUND_RIVER.ordinal();
	}


	public static boolean isTurn(int stage) {
		return stage == TexasHoldemEnumStage.TURN.ordinal() || stage == TexasHoldemEnumStage.BET_ROUND_TURN.ordinal();
	}

	@Override
	public Stage indexToStage(int index) {
		if (index < 0 || index >= TexasHoldemStage.TexasHoldemEnumStage.values().length)
			return null;
		return new TexasHoldemStage(TexasHoldemStage.TexasHoldemEnumStage.values()[index]);
	}

	
	
}
