package com.asdf.plugins.pokergames.texasholdem.bots;

import com.asdf.common.Action;
import com.asdf.plugins.pokergames.*;
import com.asdf.plugins.pokergames.texasholdem.TexasHoldemStage;

/**
 * Simple rule-based bot to show the capabilities of the AI-interface.
 * @author teemu
 *
 */

public class Bananabot implements Player {


   private int ourSeat;       	// our seat for the current hand
   private Card c1, c2;       	// our hole cards
   private GameInfo gi;       	// general game information
   private int state;			// assistance variable to track betting round information 
   private Hand handCards;		// our hand cards
   private HandEvaluator eval; 
   private boolean payedBlind = false;	// blinds not payed yet

   
   public Bananabot() {
   
   state = 0;
   eval = new HandEvaluator();
   
   }

   
   /**
    * Requests an Action from the player
    * Called when it is the Player's turn to act.
    */
   public Action getAction() {
	   
	  //Blind-round:
	  if(ourSeat == gi.getBigBlindSeat() && payedBlind == false) {
			payedBlind = true;
			return Action.bigBlindAction();
	  }else if(ourSeat == gi.getSmallBlindSeat() && payedBlind == false) {
			payedBlind = true;
			return Action.smallBlindAction();
	  }else if(this.gi.getPlayer(ourSeat).hasToPostBlind().toPay(gi.getSmallBlindSize(), gi.getBigBlindSize()) > 0) {
			payedBlind = true;
			return Action.postBlindAction();
	  }
	  
	  // Pre-flop:
	  else if (TexasHoldemStage.isPreFlop(gi.getStage().index())) {
    	 //System.out.println("preflop");
         return preFlopAction();   
      }
	  
	  // Post-flop:
      else  {
    	 //System.out.println("postflop");
         return postFlopAction();
      }
   }


   /**
    * A new betting round has started.
    */
   public void stageEvent(int stage) {
	   //System.out.println("Stage: "+ stage);
   }

   /**
    * A showdown has occurred.
    * @param seat of the player showing
    * @param c1 the first hole card shown
    * @param c2 the second hole card shown
    */
   public void showdownEvent(int seat, Card c1, Card c2) {
	   //System.out.println("showdown");
   }

   /**
    * A new game has been started.
    * @param gInfo the game information
    */
   public void gameStartEvent(GameInfo gInfo, int seat) {
      this.gi = gInfo;
      this.ourSeat = seat;
      //System.out.println("game_start");
   }

   /**
    * An event sent when all players are being dealt their hole cards
    */
   public void dealHoleCardsEvent() {
	   //System.out.println("HoleCardsDealt");
   }

   /**
    * An action has been observed.
    */
   public void actionEvent(int pos, Action act) {
	   //System.out.println("Action"); 
   }

   /**
    * The game info state has been updated
    * Called after an action event has been fully processed
    */
   public void gameStateChanged() {
	   //System.out.println("game_state_changed");
   }

   /**
    * The hand is now over.
    */
   public void gameOverEvent() {
	   payedBlind = false;
	   //System.out.println("hand_over");
   }

   /**
    * A player at seat has won amount with the hand 
    */
   public void winEvent(int seat, double amount, Hand hand) {
	   //System.out.println("winevent");
   }

   @Override
   /**
    * Receiving our hole cards.
    */
	public void receiveHand(Hand hand, int seat) {
		if(hand != null)
			handCards = new Hand(hand);
		this.ourSeat = seat;
		//System.out.println("We got a hand!");
		holeCards(handCards.getCard(0), handCards.getCard(1), ourSeat);
	}
   
   
   /*
    * Insert hole cards and seat
    */
   private void holeCards(Card c1, Card c2, int seat) {
      this.c1 = c1;
      this.c2 = c2;
      this.ourSeat = seat;
   }
   
   /*
    * Our pre-flop action.
	* Simple example.
    */
   private Action preFlopAction() {
 
      double toCall = gi.getAmountToCall(ourSeat);
      double handValue = 50;
      
      handValue = SimplePFTable.calcValue(c1, c2);
      
      double bets = 2*gi.getBetsToCall(ourSeat);
      if (bets < 0.5) bets = 1;
      if (bets > 1.5) state = 1;
      
      
      // Choose betting/raising, calling, checking or folding by the number of player and strength of hand.
      
      // two players:
      if (gi.getNumActivePlayers() == 2) {
	      if (handValue < 50/(1+gi.getNumRaises())) {
		      state = 0;
		      return Action.raiseAction(toCall);
	      }
	      else if (handValue < 150/(1+bets))
		      return Action.callAction();
	      else if (toCall > 0) {
		      if (handValue < 1 - (toCall/gi.getEligiblePot(ourSeat)))
			      return Action.callAction();
		      else return Action.foldAction();
	      }
	      else return Action.checkAction();
      }
      // 3 players
      else if (gi.getNumActivePlayers() == 3) {
	      if (handValue < 33/(1+gi.getNumRaises())) {
		      state = 0;
		      return Action.raiseAction(toCall);
	      }
	      else if (handValue < 65/(1+bets))
		      return Action.callAction();
	      else if (toCall > 0) {
		      if (handValue < 1 - (1.5*toCall/gi.getEligiblePot(ourSeat)))
			      return Action.callAction();
		      else return Action.foldAction();
	      }
	      else return Action.checkAction();
      }
      // 4 players
      else if (gi.getNumActivePlayers() == 4) {
	      if (handValue < 25/(1+gi.getNumRaises())) {
		      state = 0;
		      return Action.raiseAction(toCall);
	      }
	      else if (handValue < 50/(1+bets))
		      return Action.callAction();
	      else if (toCall > 0) {
		      if (handValue < 1 - (2*toCall/gi.getEligiblePot(ourSeat)))
			      return Action.callAction();
		      else return Action.foldAction();
	      }
	      else return Action.checkAction();
      }
      // 5 players
      else if (gi.getNumActivePlayers() == 5) {
	      if (handValue < 20/(1+gi.getNumRaises())) {
		      state = 0;
		      return Action.raiseAction(toCall);
	      }
	      else if (handValue < 50/(1+bets))
		      return Action.callAction();
	      else if (toCall > 0) {
		      if (handValue < 1 - (2.5*toCall/gi.getEligiblePot(ourSeat)))
			      return Action.callAction();
		      else return Action.foldAction();
	      }
	      else return Action.checkAction();
      }
      // 6 players
      else if (gi.getNumActivePlayers() == 6) {
	      if (handValue < 17/(1+gi.getNumRaises())) {
		      state = 0;
		      return Action.raiseAction(toCall);
	      }
	      else if (handValue < 50/(1+bets))
		      return Action.callAction();
	      else if (toCall > 0) {
		      if (handValue < 1 - (3*toCall/gi.getEligiblePot(ourSeat)))
			      return Action.callAction();
		      else return Action.foldAction();
	      }
	      else return Action.checkAction();
      }
      // 7 players
      else if (gi.getNumActivePlayers() == 7) {
	      if (handValue < 14/(1+gi.getNumRaises())) {
		      state = 0;
		      return Action.raiseAction(toCall);
	      }
	      else if (handValue < 50/(1+bets))
		      return Action.callAction();
	      else if (toCall > 0) {
		      if (handValue < 1 - (3.5*toCall/gi.getEligiblePot(ourSeat)))
			      return Action.callAction();
		      else return Action.foldAction();
	      }
	      else return Action.checkAction();
      }
      // 8 players
      else if (gi.getNumActivePlayers() == 8) {
	      if (handValue < 12/(1+gi.getNumRaises())) {
		      state = 0;
		      return Action.raiseAction(toCall);
	      }
	      else if (handValue < 50/(1+bets))
		      return Action.callAction();
	      else if (toCall > 0) {
		      if (handValue < 1 - (4*toCall/gi.getEligiblePot(ourSeat)))
			      return Action.callAction();
		      else return Action.foldAction();
	      }
	      else return Action.checkAction();
      }
      // 9 players
      else if (gi.getNumActivePlayers() == 9) {
	      if (handValue < 11/(1+gi.getNumRaises())) {
		      state = 0;
		      return Action.raiseAction(toCall);
	      }
	      else if (handValue < 50/(1+bets))
		      return Action.callAction();
	      else if (toCall > 0) {
		      if (handValue < 1 - (4.5*toCall/gi.getEligiblePot(ourSeat)))
			      return Action.callAction();
		      else return Action.foldAction();
	      }
	      else return Action.checkAction();
      }
      // 10 players
      else if (gi.getNumActivePlayers() == 10) {
	      if (handValue < 10/(1+gi.getNumRaises())) {
		      state = 0;
		      return Action.raiseAction(toCall);
	      }
	      else if (handValue < 50/(1+bets))
		      return Action.callAction();
	      else if (toCall > 0) {
		      if (handValue < 1 - (5*toCall/gi.getEligiblePot(ourSeat)))
			      return Action.callAction();
		      else return Action.foldAction();
	      }
	      else return Action.checkAction();
      }
      
      // check or call
      return Action.checkAction();
   }




   /*
    * Our post-flop action.
    * Again really simple example.
    */
   private Action postFlopAction() {
      // number of players left in the hand (including us)
      int np = gi.getNumActivePlayers();

      // amount to call
      double toCall = gi.getAmountToCall(ourSeat);
      if (toCall > 0) state = 0;
      
      // immediate pot odds
      double PO = toCall/(double)(gi.getEligiblePot(ourSeat)+toCall);

      // compute our current hand rank
      double HRN = eval.handRank(c1, c2, gi.getBoard(), np-1);
      //System.out.println("HS: "+ HRN);

      // compute a fast approximation of our hand (positive and negative) potential
      double PPOT = 0.0;
      double NPOT = 0.0;
      if (gi.getStage().index() < new TexasHoldemStage(TexasHoldemStage.TexasHoldemEnumStage.RIVER).index()) {
         PPOT = eval.potential(c1, c2, gi.getBoard(), true);
         NPOT = eval.potential(c1, c2, gi.getBoard(), false);
      }
      //System.out.println("PPOT: "+ PPOT);
      //System.out.println("NPOT: "+ NPOT);
      
      // here is an example of how to step through
      // all the opponents at the table:
      int numCommitted = 0;
      for (int i=0; i<gi.getNumSeats(); i++) {
         if (gi.inGame(i)) { // if a player is in seat i
            if (i!= ourSeat && gi.isCommitted(i)) {
               numCommitted++;
            }
         }
      }
      // NUTS:
      if (HRN == 1.0 && state == 0) {
    	  state = 0;
    	  return Action.raiseAction(toCall);
      }
      
      if (gi.getNumToAct()/gi.getNumActivePlayers() < 0.5) {
	      state = 0;
      }
      
      // No bets yet
	  if (toCall == 0) {
	      if (HRN > 0.5 && state == 0) {
		      state = 0;
		      return Action.betAction(toCall);
	      }
	      if (HRN + PPOT > 0.7 && state == 0) {
		      state = 0;
		      return Action.betAction(toCall);
	      }
	      if (TexasHoldemStage.isRiver(gi.getStage().index())) {
		      if (Math.random() < 0.2) {
			      return Action.betAction(toCall);
		      }
	      }
	      state = 0;
	      return Action.checkAction();
      }
	  // Some bets to call
      else {
	      if (HRN > 0.5 + 0.1*gi.getNumRaises()) {
		      state = 0;
		      return Action.raiseAction(toCall);
	      }
	      if (HRN*HRN*gi.getEligiblePot(ourSeat) > toCall || PPOT > PO) {
		      // if we have draw odds or a strong enough hand to call
		      state = 1;
		      return Action.callAction();
	      }
	      state = 0;
	      return Action.checkAction();
      }   
   }  
}

