package com.asdf.plugins.pokergames.texasholdem.bots;

import com.asdf.common.Action;
import com.asdf.plugins.pokergames.BestHand;
import com.asdf.plugins.pokergames.Card;
import com.asdf.plugins.pokergames.Deck;
import com.asdf.plugins.pokergames.EvalTables;
import com.asdf.plugins.pokergames.GameInfo;
import com.asdf.plugins.pokergames.Hand;
import com.asdf.plugins.pokergames.HandEval;
import com.asdf.plugins.pokergames.Player;
import com.asdf.plugins.pokergames.texasholdem.TexasHoldemStage;

public class KillerBot implements Player {
	
	HandEval handEval = new HandEval(new EvalTables());
	boolean payedBlind = false;
	
	GameInfo gi = null;
	int seat = -1;
	Hand handCards = null;

	boolean bluff = false;
	double perhana = 0.0;
	boolean prints = false;
	
	/*
		if (val > 6185)
			return (HIGH_CARD); // 1277 high card
		if (val > 3325)
			return (ONE_PAIR); // 2860 one pair
		if (val > 2467)
			return (TWO_PAIR); //  858 two pair
		if (val > 1609)
			return (THREE_OF_A_KIND); //  858 three-kind
		if (val > 1599)
			return (STRAIGHT); //   10 straights
	 */
	
	
	private int[] valueLegendary5 = {2467, 2200, 2000, 1900, 1800, 1700, 1600, 1500, 1400, 1300};
	private int[] valueLegendary6 = valueLegendary5;

	private int[] valueStrong5 = {4325, 4000, 3800, 3500, 3300, 3100, 3000, 2900, 2700, 2500};
	private int[] valueStrong6 = valueStrong5;
	
	public KillerBot(){
		
	}
	public void setPrints(boolean b){
		prints = b;
	}
	
	public KillerBot(int seat){
		this.seat = seat;
	}

	@Override
	public void actionEvent(int arg0, Action arg1) {
		// TODO Auto-generated method stub

	}

	@Override
	public void dealHoleCardsEvent() {
		// TODO Auto-generated method stub

	}

	@Override
	public void gameOverEvent() {
		handCards = null;
		payedBlind = false;
	}

	@Override
	public void gameStartEvent(GameInfo arg0, int seat) {
		gi = arg0;
		payedBlind = false;		
		this.seat = seat;
	}

	@Override
	public void gameStateChanged() {

	}

	@Override
	public Action getAction() {
		if(prints){
			System.out.println();
			System.out.println("Called getAction on bot");
			System.err.println("(Bot@ " + seat + "): I was asked for an action. I will sleep for one second now, to give you time to understand what's going on.");
		}
		try {
			Thread.sleep(400);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		if(seat == gi.getBigBlindSeat() && payedBlind == false) {
			if(prints)
				System.err.println("(Bot@ " + seat + "): I need to pay big blind. Doing so now.");
			payedBlind = true;
			return Action.bigBlindAction();
		}else if(seat == gi.getSmallBlindSeat() && payedBlind == false) {
			if(prints)
				System.err.println("(Bot@ " + seat + "): I need to pay small blind. Doing so now.");
			payedBlind = true;
			return Action.smallBlindAction();
		}else if(this.gi.getPlayer(seat).hasToPostBlind().toPay(gi.getSmallBlindSize(), gi.getBigBlindSize()) > 0) {
			if(prints)
				System.err.println("(Bot@ " + seat + "): I need to pay penalty blind. Doing so now.");
			payedBlind = true;
			return Action.postBlindAction();
		}
		
		if(handCards == null)
		{
			if(prints)
				System.err.println("(Bot@ " + seat + "): Hand cards are null. Sending minimum raise action.");
			return Action.raiseAction(gi.getMinRaise());
		}
		
		if(handCards.getCard(0) == null)
		{
			if(prints)
				System.err.println("(Bot@ " + seat + "): Hand is not null, but I have no cards. Sending minimum raise.");
			return Action.raiseAction(gi.getMinRaise());
		}
		
		
		if(prints)
			System.err.println(handCards.getCard(0) + ", " + handCards.getCard(1));
		
		if(gi.getStage().index() == TexasHoldemStage.TexasHoldemEnumStage.BLIND_ROUND.ordinal()) {
			if(prints)
				System.err.println("(Bot@ " + seat + "): Blind round. Sending check action.");
			return Action.callAction();
		}
		// hand value / numPlayers > c => raise min
		if(gi.getBoard().size() == 0) {
			
			perhana = Math.random() + 2.0;
			perhana /= 2.0;
			
			if(prints)
				System.err.println("(Bot@ " + seat + "): I have a hand but there are no cards on the table yet. deciding action..");
			
			if(handCards.getCard(0).rank() + handCards.getCard(1).rank() < 10)
			{
//				 Send chat message.
				ChatGenerator.getSentence();
			}
			
			int value = handCards.getCard(0).rank() * handCards.getCard(1).rank();
			int value2 = Math.abs(handCards.getCard(0).rank() - handCards.getCard(1).rank());
			if(value > 15 || value / (value2 + 1) > 5) {
				if(prints)
					System.err.println("(Bot@ " + seat + "): I think my hand is quite good, so I will raise.");
				return Action.callAction();
			}
			if(prints)
				System.err.println("(Bot@ " + seat + "): My hand is shit, I will fold.");
			return Action.foldAction();
		}
		else if(gi.getBoard().size() == 3)
		{
			if(prints)
				System.err.println("(Bot@ " + seat + "): Three cards on the table. Deciding action..");
			
			Hand tmpHand = new Hand();
			
			for(int i=0; i<gi.getBoard().size(); i++)
				tmpHand.addCard(gi.getBoard().getCard(i));
			tmpHand.addCard(handCards.getCard(0));
			tmpHand.addCard(handCards.getCard(1));
			
			int bhand = this.expectedValue5(tmpHand);
			if(prints)
				System.err.println("My hand value is: " + bhand + " limits are, Legendary: " +
					valueLegendary5[gi.getNumActivePlayers()] + ". Strong: " +
					valueStrong5[gi.getNumActivePlayers()] + "."
			);
			
			if(bhand < perhana * valueLegendary5[gi.getNumActivePlayers()]) {
				if(prints)
					System.err.println("(Bot@ " + seat + "): My hand is a fucking legend, I will raise!");
				return Action.raiseAction(gi.getMinRaise() * 2);
			}
			
			if(bhand < perhana * valueStrong5[gi.getNumActivePlayers()]) {
				if(prints)
					System.err.println("(Bot@ " + seat + "): My hand seems quite good, I will raise.");
				return Action.raiseAction(gi.getMinRaise());
			}
			if(prints)
				System.err.println("(Bot@ " + seat + "): My hand is shit, I will fold.");
			return Action.foldAction();
		}	
		else if(gi.getBoard().size() == 4) {
			if(prints)
				System.err.println("(Bot@ " + seat + "): Four cards on table. Deciding action..");
			
			Hand tmpHand = new Hand();
			
			for(int i=0; i<gi.getBoard().size(); i++)
				tmpHand.addCard(gi.getBoard().getCard(i));
			tmpHand.addCard(handCards.getCard(0));
			tmpHand.addCard(handCards.getCard(1));
			
			int handValue = this.expectedValue6(tmpHand);
			if(prints)
				System.err.println("My hand value is: " + handValue + " limits are, Legendary: " +
					valueLegendary5[gi.getNumActivePlayers()] + ". Strong: " +
					valueStrong5[gi.getNumActivePlayers()] + "."
			);
			
			if(handValue < perhana * valueLegendary6[gi.getNumActivePlayers()]) {
				if(prints)
					System.err.println("(Bot): My hand is a fucking legend, I will raise!");
				return Action.raiseAction(gi.getMinRaise() * 2);
			}
			
			if(handValue < perhana * valueStrong6[gi.getNumActivePlayers()]) {
				if(prints)
					System.err.println("(Bot): My hand is very strong. I will raise.");
				return Action.raiseAction(gi.getMinRaise());
			}
			if(prints)
				System.err.println("(Bot@ " + seat + "): My hand is not strong enough. Folding.");
			return Action.foldAction();
		}
		else {
			if(prints)
				System.err.println("(Bot@ " + seat + "): Five cards on the table. Deciding action..");
			
			if(bluff) {
				if(prints)
					System.err.println("(Bot@ " + seat + "): Bluffing, raising.");
				return Action.raiseAction(gi.getMinRaise());
			}
			
			int[] cards = new int[7];
			for(int i=0; i<gi.getBoard().size(); i++) {
				cards[i] = gi.getBoard().getCard(i).getInt();
			}
			cards[5] = handCards.getCard(0).getInt();
			cards[6] = handCards.getCard(1).getInt();
			
			BestHand bhand = handEval.eval_7cards(cards);
			if(prints)
				System.err.println("My hand value is: " + bhand.value + " limits are, Legendary: " +
					valueLegendary5[gi.getNumActivePlayers()] + ". Strong: " +
					valueStrong5[gi.getNumActivePlayers()] + "."
			);
			
			if(bhand.value < perhana * valueLegendary6[gi.getNumActivePlayers()]) {
				if(prints)
					System.err.println("(Bot@ " + seat + "): My hand is a fucking legend, I will raise!");
				return Action.raiseAction(gi.getMinRaise() * 2);
			}
			
			if(bhand.value < perhana * valueStrong6[gi.getNumActivePlayers()]) {
				if(prints)
					System.err.println("(Bot@ " + seat + "): I have super strong hand, raising.");
				return Action.raiseAction(gi.getMinRaise());
			}
			if(prints)
				System.err.println("(Bot@ " + seat + "): My hand is shit, I will fold.");
			return Action.foldAction();
		}
		
	}

	
	
	
	private int expectedValue6(Hand hand) {
		Deck deck = new Deck();
		deck.newDeck();
		
		Card card = deck.dealCard();
		
		int valueSum = 0;
		int valueNum = 0;
		
		int cards[] = new int[7];
		
		for(int i=0; i<6; i++) {
			cards[i] = hand.getCard(i).getInt();
		}
		
		while(card != null) {
			if(!hand.containsCard(card)) {
				cards[6] = card.getInt();
				
				valueSum += handEval.eval_7cards(cards).value;
				valueNum++;
			}
			
			card = deck.dealCard();
		}
		
		return valueSum / valueNum;
	}
	
	
	
	
	private int expectedValue5(Hand hand) {
		Deck deck = new Deck();
		deck.newDeck();
		
		Card card = deck.dealCard();
		
		int valueSum = 0;
		int valueNum = 0;
		
		int cards[] = new int[6];
		
		for(int i=0; i<5; i++) {
			cards[i] = hand.getCard(i).getInt();
		}
		
		while(card != null) {
			if(!hand.containsCard(card)) {
				cards[5] = card.getInt();
				
				valueSum += handEval.eval_6cards(cards).value;
				valueNum++;
			}
			
			card = deck.dealCard();
		}
		
		return valueSum / valueNum;
	}
	
	
	
	
	/*
	@Override
	public void holeCards(Card arg0, Card arg1, int arg2) {
		seat = arg2;
		handCards = new Hand();
		handCards.addCard(arg0);
		handCards.addCard(arg1);
	}*/

	@Override
	public void receiveHand(Hand hand, int seat) {
		if(hand != null)
			handCards = new Hand(hand);
		this.seat = seat;
	}

	@Override
	public void showdownEvent(int arg0, Card arg1, Card arg2) {
		// TODO Auto-generated method stub

	}

	@Override
	public void stageEvent(int arg0) {
		// TODO Auto-generated method stub

	}

	public void winEvent(int arg0, double arg1, String arg2) {
		// TODO Auto-generated method stub
		
	}

	public void winEvent(int seat, double amount, Hand hand) {
		// TODO Auto-generated method stub
		
	}



}
