package com.asdf.plugins.pokergames.texasholdem.bots;

import com.asdf.common.Action;
import com.asdf.plugins.pokergames.BestHand;
import com.asdf.plugins.pokergames.Card;
import com.asdf.plugins.pokergames.EvalTables;
import com.asdf.plugins.pokergames.GameInfo;
import com.asdf.plugins.pokergames.Hand;
import com.asdf.plugins.pokergames.HandEval;
import com.asdf.plugins.pokergames.Player;
import com.asdf.plugins.pokergames.texasholdem.TexasHoldemStage;

public class SuperBot implements Player {
	
	HandEval handEval = new HandEval(new EvalTables());
	boolean payedBlind = false;
	
	GameInfo gi = null;
	int seat = -1;
	Hand handCards = null;

	boolean bluff = false;
	
	public SuperBot(){
		
	}
	
	public SuperBot(int seat){
		this.seat = seat;
	}

	@Override
	public void actionEvent(int arg0, Action arg1) {
		// TODO Auto-generated method stub

	}

	@Override
	public void dealHoleCardsEvent() {
		// TODO Auto-generated method stub

	}

	@Override
	public void gameOverEvent() {
		handCards = null;
		payedBlind = false;
	}

	@Override
	public void gameStartEvent(GameInfo arg0, int seat) {
		gi = arg0;
		payedBlind = false;		
		this.seat = seat;
	}

	@Override
	public void gameStateChanged() {

	}

	@Override
	public Action getAction() {
		System.out.println("Called getAction on bot");
		System.err.println("(Bot): I was asked for an action. I will sleep for one second now, to give you time to understand what's going on.");
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		if(seat == gi.getBigBlindSeat() && payedBlind == false) {
			System.err.println("(Bot): I need to pay big blind. Doing so now.");
			payedBlind = true;
			return Action.bigBlindAction();
		} else if(seat == gi.getSmallBlindSeat() && payedBlind == false) {
			System.err.println("(Bot): I need to pay small blind. Doing so now.");
			payedBlind = true;
			return Action.smallBlindAction();
		} else if(this.gi.getPlayer(seat).hasToPostBlind().toPay(gi.getSmallBlindSize(), gi.getBigBlindSize()) > 0) {
			System.err.println("(Bot@ " + seat + "): I need to pay penalty blind. Doing so now.");
			payedBlind = true;
			return Action.postBlindAction();
		}
		
		if(handCards == null)
		{
			System.err.println("(Bot): Hand cards are null. Sending minimum raise action.");
			return Action.raiseAction(gi.getMinRaise());
		}
		else if(handCards.getCard(0) == null)
		{
			System.err.println("(Bot): Hand is not null, but I have no cards. Sending minimum raise.");
			return Action.raiseAction(gi.getMinRaise());
		}
		
		if(gi.getStage().index() == TexasHoldemStage.TexasHoldemEnumStage.BLIND_ROUND.ordinal()) {
			System.err.println("(Bot): Blind round. Sending check action.");
			return Action.checkAction();
		}
		// hand value / numPlayers > c => raise min
		if(gi.getBoard().size() == 0) {
			
			System.err.println("(Bot): I have a hand but there are no cards on the table yet. deciding action..");
			
			if(handCards.getCard(0).rank() + handCards.getCard(1).rank() < 10)
			{
//				 Send chat message.
				ChatGenerator.getSentence();
			}
			
			if(Math.random() > 0.8) {
				System.err.println("(Bot): I don't care about my hand, I'll just bluff and raise.");
				bluff = true;
				return Action.raiseAction(gi.getMinRaise());
			}

			bluff = false;
			
			if(handCards.getCard(0).rank() == handCards.getCard(1).rank()) {
				System.err.println("(Bot): I have a pair!! Ownage! I will raise now like there is no tomorrow!!1");
				return Action.raiseAction(gi.getBankRoll(seat));
			}
			
			int value = Math.max(handCards.getCard(0).rank(), handCards.getCard(1).rank());
			int value2 = Math.abs(handCards.getCard(0).rank() - handCards.getCard(1).rank());
			if(value > 8 || value2 < 4) {
				System.err.println("(Bot): I think my hand is quite good, so I will raise.");
				return Action.raiseAction(gi.getMinRaise());
			}
			
			System.err.println("(Bot): My hand is shit, I will fold.");
			return Action.foldAction();
		}
		else if(gi.getBoard().size() == 3)
		{
			System.err.println("(Bot): Three cards on the table. Deciding action..");
			
			if(bluff) {
				System.err.println("(Bot): Bluffing, hehe, raising.");
				return Action.raiseAction(gi.getMinRaise());
			}
			
			int c1 = handCards.getCard(0).getInt();
			int c2 = handCards.getCard(1).getInt();
			int c3 = gi.getBoard().getCard(0).getInt();
			int c4 = gi.getBoard().getCard(1).getInt();
			int c5 = gi.getBoard().getCard(2).getInt();
			int bhand = handEval.eval_5cards(c1, c2, c3, c4, c5);
			
			if(bhand > 2000) {
				System.err.println("(Bot): My hand is a fucking legend, I will raise!");
				return Action.raiseAction(gi.getMinRaise() * 2);
			}
			
			if(bhand > 1500) {
				System.err.println("(Bot): My hand seems quite good, I will raise.");
				return Action.raiseAction(gi.getMinRaise());
			}
			
			System.err.println("(Bot): My hand is shit, I will fold.");
			return Action.foldAction();
		}	
		else if(gi.getBoard().size() == 4) {
			System.err.println("(Bot): Four cards on table. Deciding action..");
			
			if(bluff) {
				System.err.println("(Bot): Bluffing, raising.");
				return Action.raiseAction(gi.getMinRaise());
			}
			
			int[] cards = new int[6];
			for(int i=0; i<gi.getBoard().size(); i++) {
				cards[i] = gi.getBoard().getCard(i).getInt();
			}
			cards[4] = handCards.getCard(0).getInt();
			cards[5] = handCards.getCard(1).getInt();
			
			BestHand bhand = handEval.eval_6cards(cards);
			
			if(bhand.value > 2600) {
				System.err.println("(Bot): My hand is a fucking legend, I will raise!");
				return Action.raiseAction(gi.getMinRaise() * 2);
			}
			
			if(bhand.value > 2000) {
				System.err.println("(Bot): My hand is very strong. I will raise.");
				return Action.raiseAction(gi.getMinRaise());
			}
			
			System.err.println("(Bot): My hand is not strong enough. Folding.");
			return Action.foldAction();
		}
		else {
			System.err.println("(Bot): Five cards on the table. Deciding action..");
			
			if(bluff) {
				System.err.println("(Bot): Bluffing, raising.");
				return Action.raiseAction(gi.getMinRaise());
			}
			
			int[] cards = new int[7];
			for(int i=0; i<gi.getBoard().size(); i++) {
				cards[i] = gi.getBoard().getCard(i).getInt();
			}
			cards[5] = handCards.getCard(0).getInt();
			cards[6] = handCards.getCard(1).getInt();
			
			BestHand bhand = handEval.eval_7cards(cards);
			
			if(bhand.value > 3500) {
				System.err.println("(Bot): My hand is a fucking legend, I will raise!");
				return Action.raiseAction(gi.getMinRaise() * 2);
			}
			
			if(bhand.value > 2000) {
				System.err.println("(Bot): I have super strong hand, raising.");
				return Action.raiseAction(gi.getMinRaise());
			}
			
			System.err.println("(Bot): My hand is shit, I will fold.");
			return Action.foldAction();
		}
		
	}

	//@Override
	/*public void holeCards(Card arg0, Card arg1, int arg2) {
		seat = arg2;
		handCards = new Hand();
		handCards.addCard(arg0);
		handCards.addCard(arg1);
	}*/

	@Override
	public void receiveHand(Hand hand, int seat) {
		if(hand != null)
			handCards = new Hand(hand);
		this.seat = seat;
	}

	@Override
	public void showdownEvent(int arg0, Card arg1, Card arg2) {
		// TODO Auto-generated method stub

	}

	@Override
	public void stageEvent(int arg0) {
		// TODO Auto-generated method stub

	}

	public void winEvent(int arg0, double arg1, String arg2) {
		// TODO Auto-generated method stub
		
	}

	public void winEvent(int seat, double amount, Hand hand) {
		// TODO Auto-generated method stub
		
	}



}
