package com.asdf.client;

import java.io.FileInputStream;
import java.util.Properties;

/**
 * Clients configuration.
 * 
 * @author asdf
 *
 */
public class Configuration {
	/** Shared Configuration instance */
	private static Configuration configuration = null;
	
	/** Properties instance */
	private Properties properties;
	
	/**
	 * Returns instance of Configuration and creates
	 * one if it doesn't already exist. Tries 
	 * @return Instance of Configuration
	 */
	public static Configuration getInstance() {
		if (configuration == null) {
			configuration = new Configuration();
		}
		return configuration;
	}
	
	/**
	 * Protected constructor. Can't be used from outside of the
	 * class. Tries to load configuration from client.conf from
	 * working directory.
	 */
	protected Configuration() {
		properties = new Properties();
		try {
			FileInputStream fis = new FileInputStream("client.conf");
			properties.load(fis);
		}
		catch (Exception e) {
			//TODO in case of error?
		}
	}
	
	/**
	 * Plugin classpaths. Paths are went through in order.
	 * Paths are limited by comma.
	 * @return Order of classpaths
	 */
	public static String[] pluginPaths() {
		Configuration c = getInstance();
		String paths = c.properties.getProperty("pluginpaths");
		if (paths != null) {
			return paths.split(",");
		}
		else {
			return new String[]{};
		}
	}

	/**
	 * Bot classpaths. Paths are went through in order.
	 * Paths are limited by comma.
	 * @return Order of classpaths
	 */
	public static String[] botPaths() {
		Configuration c = getInstance();
		String paths = c.properties.getProperty("botpaths");
		if (paths != null) {
			return paths.split(",");
		}
		else {
			return new String[]{};
		}
	}

	
}
