package com.asdf.client;

import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;

import com.asdf.common.GameController;
import com.asdf.common.PluginLoadException;
import com.asdf.gui.lobby.CreateGamePanel;

/**
 * Handles loading of dynamic plugins.
 */
public class PluginLoader {
	
	/**
	 * Constructor for PluginLoader which takes url -locations as parameters to search for plugins.
	 * 
	 * @param urls Locations for plugins.
	 */
	public PluginLoader(URL[] urls) {
		URLClassLoader sysloader = (URLClassLoader)ClassLoader.getSystemClassLoader();
        Class<URLClassLoader> sysclass = URLClassLoader.class;
        try {
        	
            Method method = sysclass.getDeclaredMethod("addURL", new Class[]{URL.class});
            method.setAccessible(true);
            for (int i = 0; i < urls.length; i++) {
            	method.invoke(sysloader,new Object[]{urls[i]});
            }
        } catch (Throwable t) {
            t.printStackTrace(); 
        }
	}
	
	/**
	 * Constructor for PluginLoader which searches for plugins only in location
	 * defined in client.conf file.
	 */
	public PluginLoader() {
		String[] paths = Configuration.pluginPaths();
		String[] botPaths = Configuration.botPaths();
		URL[] urls = new URL[paths.length + botPaths.length];
		int k;
		for (k=0; k<paths.length; k++) {
			try {
				urls[k] = new URL(paths[k]);
			} catch (MalformedURLException e) {
				e.printStackTrace();
			}
		}
		for(int j = 0; j < botPaths.length; j++){
			try {
				urls[k+j] = new URL(botPaths[j]);
			} catch (MalformedURLException e) {
				e.printStackTrace();
			}
		}
		
		URLClassLoader sysloader = (URLClassLoader)ClassLoader.getSystemClassLoader();
        Class<URLClassLoader> sysclass = URLClassLoader.class;
        try {
            Method method = sysclass.getDeclaredMethod("addURL", new Class[]{URL.class});
            method.setAccessible(true);
            for (int i = 0; i < urls.length; i++)
            	method.invoke(sysloader,new Object[]{urls[i]}); 
        } catch (Throwable t) {
            t.printStackTrace(); 
        }

	}

	
	/**
	 * Tries to load and initialize a GameController found in classDir
	 * in package className. Class is casted to GameController before
	 * returning.
	 * 
	 * @param className Fully qualified name of the class
	 * @param gameID ID of the new game
	 * @param name Name of the game
	 * @return Instance of requested Game
	 * @throws PluginLoadException If plugin could not be loaded.
	 */

	@SuppressWarnings("unchecked")
	public GameController loadGame(String className, long gameID, String name) throws PluginLoadException {
		try {			
			return (GameController)Class.forName(className).newInstance();
		}
		catch(Exception e) {
			throw new PluginLoadException(e);
		}		
	}
	
	/**
	 * Tries to load and initialize CreateGamePanel with given className.
	 * @param className Fully qualified name of the class
	 * @param gameID ID of the new game
	 * @param name Name of the panel
	 * @return Instance of requested Game
	 * @throws PluginLoadException If plugin could not be loaded.
	 */
	
	@SuppressWarnings("unchecked")
	public CreateGamePanel loadCreateGamePanel(String className, long gameID, String name) throws PluginLoadException {
		try {			
			Class<CreateGamePanel> cp = (Class<CreateGamePanel>)Class.forName(className);
			CreateGamePanel panel = cp.newInstance();			
			return panel;
		}
		catch(Exception e) {
			throw new PluginLoadException(e);
		}		
	}
	
	/**
	 * Tries to load and initialize an object.
	 * @param className Fully qualified name of the class
	 * @return instance of Object
	 * @throws PluginLoadException If object could not be loaded
	 */
	@SuppressWarnings("unchecked")
	public Object loadObject(String className) throws PluginLoadException {
		try {			
			Class cp = Class.forName(className);		
			return cp.newInstance();
		}
		catch(Exception e) {
			throw new PluginLoadException(e);
		}
	}
	
}
