package com.asdf.client;

import com.asdf.common.AccountType;

/**
 * Class User is used in lobby to keep information about user.
 * Keeps information about user whether he is logged in or out.
 * It can be used to restrict features in LobbyView.
 * 
 * @author louhio
 *
 */
public class User {

    private AccountType type;
    private boolean isLogged = false;
    private String name;
    private double balance;

    public User(){
    	type = AccountType.GUEST;
    }
    /**
     * Sets user logged in.
     * 
     * @param name Account name.
     * @param balance Balance of account.
     * @param type Type of account. See AccountTypes enumeration for details.
     */
    public void setLogged(String name, double balance, AccountType type){
        this.type = type;
        this.balance = balance;
        this.type = type;
        this.name = name;
        
        isLogged = true;
    }
    
    /**
     * Sets user logged out. 
     */
    public void setLoggedOut(){
        isLogged = false;
        this.type = AccountType.GUEST;
        name = null;
        balance = 0.0;        
    }

    /**
     * Returns the type of this users account.
     * 
     * @return AccountType. See AccountType enumeration for details.
     */
	public AccountType getType() {
		return type;
	}

	/**
	 * Checks if user is logged in or out.
	 * @return True if user is logged in, false if user is logged out.
	 */
	public boolean isLogged() {
		return isLogged;
	}
	/**
	 * Returns the name of the account.
	 * @return Name of the account.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Gets the balance of the user.
	 * 
	 * @return Balance of the account.
	 */
	
	public double getBalance() {
		return balance;
	}
    
    
}
