package com.asdf.common;

import java.util.TimerTask;


/**
 * TimerTask wrapper, which synchronizes calling Game
 */
public abstract class GameTimerTask extends TimerTask {

	/** This timer's Game instance */
	protected Game game;
	
	/**
	 * Constructor
	 * @param game Game used for synchronization
	 */
	public GameTimerTask(Game game) {
		super();
		this.game = game;
	}

	/**
	 * Run method that synchronizes game before
	 * calling onTick.
	 */
	@Override
	public void run() {
		synchronized(game) {
			onTick();
		}
	}
	
	/**
	 * Method that is called when the timer fires.
	 */
	public abstract void onTick();

}
