package com.asdf.common;

import java.util.logging.LogManager;
import java.io.*;
import java.net.URL;

/**
 * Logger
 * Provides easy logging mechanisms. Logger may be configured
 * to log to various destinations. Logging.properties file is distributed
 * on the root of Server sources to let users configure destinations and
 * levels. 
 */
public class Logger {
	private static java.util.logging.Logger loggerInstance = null;
	
	/**
	 * Creates new instance when used for a first time and
	 * serves that instance for later calls. Reads configuration file
	 * from logging.properties assumed to be in working directory. If 
	 * file can't be read or it's malformed, logging is done with default
	 * settings.
	 * @return Instance of Logger
	 */
	public static java.util.logging.Logger get() {
		if (loggerInstance == null) {
			try {
				InputStream s = new FileInputStream("logging.properties");
				LogManager.getLogManager().readConfiguration(s);
				loggerInstance = java.util.logging.Logger.getLogger(Logger.class.getName());
				System.out.println("Logging succesfully loaded");
			} catch (Exception e) {
				System.err.println("Couldn't load logging configuration!");
				e.printStackTrace();
				loggerInstance = java.util.logging.Logger.getAnonymousLogger();
			}
			
		}
		return loggerInstance;
	}
	
	public static void logClassPath() {
		for(URL url : ((java.net.URLClassLoader)Thread.currentThread().getContextClassLoader()).getURLs()) {
			get().fine("classpath " + url.toString());
		}
	}
	
	/**
	 * Prevent public instantiation.
	 */
	private Logger() {
	}

}
