package com.asdf.gui.lobby;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import com.asdf.client.LobbyController;

/**
 * This class is intended to be used as a superclass for all implementations
 * of create game panels. These panels are used by the lobby to give a graphical
 * user interface for creating games.
 * 
 * @author asdf
 *
 */
public abstract class CreateGamePanel extends JPanel {
	
	private LobbyController controller = null;
	private boolean initialized = false;
	
	/**
	 * Sole superclass constructor.
	 */
	public CreateGamePanel(){
		this.setBorder(BorderFactory.createTitledBorder("Create Game"));
		//initGUI();
        this.setPreferredSize(new java.awt.Dimension(300, 550));
        this.setSize(300, 550);
		//setInitialized(true);
	}
	/**
	 * This method is called as a constructor to make sure the game is actually
	 * initialized properly. Note that this is called after the panel is created
	 * only if isInitialized() is false.
	 */
	public abstract void initGUI();
	
	/**
	 * Sets the lobby controller.
	 * @param ctrl Lobby controller
	 */
	public void setController(LobbyController ctrl){
		controller = ctrl;
	}

	/**
	 * Returns the lobby controller used by this panel.
	 * 
	 * @return Lobby Controller
	 */
	public LobbyController getController(){
		return controller;
	}
	/**
	 * Checks if this panel is already initialized. If it is not, then initGUI()
	 * method is called when this panel is set showing. 
	 * 
	 * @return true if this is initialized, false otherwise
	 */
	public boolean isInitialized(){
		return initialized;
	}
	/**
	 * Use this method to set if this panel is initialized. If initialized is set
	 * to true, then initGUI() method is not called when panel is set to showing,
	 * since it is then assumed that the panel is already initialized.
	 * 
	 * @param b
	 */
	public void setInitialized(boolean b){
		initialized = b;
	}

}
