package com.asdf.gui.lobby;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTextField;

import com.asdf.client.LobbyController;

/**
 * Bottom panel in LobbyFrame containing gamelist filters.
 * 
 * @author asdf
 * 
 */
public class LobbyBottomPanel extends javax.swing.JPanel implements KeyListener, ActionListener{
    
	/**
	 * Default serial.
	 */
	private static final long serialVersionUID = 1L;
	
    {
        //Set Look & Feel
        try {
            javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
        } catch(Exception e) {
            e.printStackTrace();
        }
    }    
    
    private JCheckBox hideFull;
    private JTextField maxPlayers;
    private JLabel filterPlayersByNameLabel;
    private JTextField playerNameFilter;
    private JTextField gameNameFilter;
    private JLabel filterPlayersByName;
    private JTextField minPlayers;
    private JLabel minPlayersLabel;
    private JLabel maxPlayersLabel;
    private JCheckBox hideEmpty;

    private LobbyController controller;

    /**
     * Sole constructor.
     *       
     * @param controller Lobby controller which controls this component.
     */
    public LobbyBottomPanel(LobbyController controller) {
        super();  
        this.controller = controller;        
        initGUI();
    }
    
    /**
     * Initializes GUI for this panel. 
     */
    private void initGUI() {
        try {
            GridBagLayout thisLayout = new GridBagLayout();
            this.setPreferredSize(new java.awt.Dimension(500, 40));
            thisLayout.rowWeights = new double[] {0.0, 0.1, 0.0, 0.1, 0.0};
            thisLayout.rowHeights = new int[] {2, 20, 5, 20, 2};
            thisLayout.columnWeights = new double[] {0.2, 0.2, 0.1, 0.0, 0.3, 0.6};
            thisLayout.columnWidths = new int[] {70, 70, 24, 12, 100, 120};
            this.setLayout(thisLayout);
            this.setSize(500, 40);
            this.setMaximumSize(new java.awt.Dimension(400, 40));
            this.setMinimumSize(new java.awt.Dimension(400, 20));
            {
                hideFull = new JCheckBox();
                this.add(hideFull, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                hideFull.setText("Hide full");
                hideFull.addActionListener(this);
            }
            {
                hideEmpty = new JCheckBox();
                this.add(hideEmpty, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                hideEmpty.setText("Hide Empty");
                hideEmpty.addActionListener(this);                
            }
            {
                maxPlayersLabel = new JLabel();
                this.add(maxPlayersLabel, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                maxPlayersLabel.setText("Max players");
   
            }
            {
                minPlayersLabel = new JLabel();
                this.add(minPlayersLabel, new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                minPlayersLabel.setText("Min players");
            }
            {
                maxPlayers = new JTextField();
                this.add(maxPlayers, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                maxPlayers.addKeyListener(this);
            }
            {
                minPlayers = new JTextField();
                this.add(minPlayers, new GridBagConstraints(2, 3, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                minPlayers.addKeyListener(this);
            }
            {
                filterPlayersByNameLabel = new JLabel();
                this.add(filterPlayersByNameLabel, new GridBagConstraints(4, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                filterPlayersByNameLabel.setText("Find games by name");
            }
            {
                filterPlayersByName = new JLabel();
                this.add(filterPlayersByName, new GridBagConstraints(4, 3, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                filterPlayersByName.setText("Find players by name");
            }
            {
                gameNameFilter = new JTextField();
                this.add(gameNameFilter, new GridBagConstraints(5, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                gameNameFilter.addKeyListener(this);
            }
            {
            	
                playerNameFilter = new JTextField();
                this.add(playerNameFilter, new GridBagConstraints(5, 3, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
                playerNameFilter.addKeyListener(this);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
  /**
   * Not used.
   */
    @Override
    public void keyPressed(KeyEvent e) {    	

    }

    /**
     * When key is released we check the input in JTextFields and filter the results
     * in gamelist. 
     */
    @Override
    public void keyReleased(KeyEvent e) {
        if(e.getSource().equals(playerNameFilter)){
            
        }else if(e.getSource().equals(gameNameFilter)){
            controller.showGamesWithNameContaining(gameNameFilter.getText());
        }else if(e.getSource().equals(minPlayers) || e.getSource().equals(maxPlayers)){
            int min = 0;
            int max = Integer.MAX_VALUE;
            if(verify(minPlayers)){
            	minPlayers.setForeground(Color.BLACK);
            	if(!minPlayers.getText().equalsIgnoreCase(""))
            		min = new Integer(minPlayers.getText());
            }else{
            	minPlayers.setForeground(Color.RED);
            }
            if(verify(maxPlayers)){
            	maxPlayers.setForeground(Color.BLACK);
            	if(!maxPlayers.getText().equalsIgnoreCase(""))
            		max = new Integer(maxPlayers.getText());
            }else{
            	maxPlayers.setForeground(Color.RED);
            }            
                        
           controller.showGamesWithPlayers(min, max);            
        }    
    }

    /**
     * Not used.
     */
    @Override
    public void keyTyped(KeyEvent e) {    	

    }

    /**
     * ActionListener for checkboxes.
     * 
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        
        if(e.getSource().equals(this.hideFull)){
            if(this.hideFull.isSelected()){
                controller.setFullGamesHidden(true);                        
            }else{
                controller.setFullGamesHidden(false);                
            }
            
        }else if(e.getSource().equals(this.hideEmpty)){
            if(this.hideEmpty.isSelected()){
                controller.setEmptyGamesHidden(true);                          
            }else{
                controller.setEmptyGamesHidden(false);               
            }
        }
        
    }
    
    /**
     * Verifies if the input in a JTextField is integer.
     * @param input JTextField to be checked
     * @return true if field contains integer, false otherwise
     */
    public boolean verify(JTextField input) {
        
        String text = input.getText().trim();
        int i;
        try {
            if(text == null || text.equalsIgnoreCase(""))
                i = 0;
            else
                i = new Integer(text);            
         } catch (NumberFormatException e){
             return false;
         } 
         if(i >= 0)
             return true;
         else
             return false;
     }
    /**
	 * Verifies if the input in a JTextField is double.	
     * @param input JTextField to be checked
     * @return true if field contains double, false otherwise
     */
    public boolean verifyD(JTextField input) {
        
        String text = input.getText().trim();
        double i;
        try {
            if(text == null || text.equalsIgnoreCase(""))
                i = 0;
            else
                i = new Double(text);            
         } catch (NumberFormatException e){
             return false;
         } 
         if(i >= 0)
             return true;
         else
             return false;
     }

}
