package com.asdf.gui.lobby;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JFrame;
import javax.swing.JPanel;

import com.asdf.client.LobbyController;
import com.asdf.parser.GameDetailsResponse;

/**
 * The class which actually shows the lobby view and collects all the panels in this
 * package under one frame.
 * 
 * @author asdf
 *
 */
public class LobbyFrame extends javax.swing.JFrame implements WindowListener{
    
	/**
	 * Default serial.
	 */
	private static final long serialVersionUID = 1L;
	
    {
        //Set Look & Feel
        try {
            javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
        } catch(Exception e) {
            e.printStackTrace();
        }
    }
    
    private JPanel topPanel;
    private JPanel leftPanel;
    private JPanel rightPanel;
    private JPanel bottomPanel;
    private JPanel middlePanel;
    
    private LobbyController controller = null;

    /**
     * Sole constructor.
     * @param controller Sets the LobbyController for this frame.
     */
    public LobbyFrame(LobbyController controller) {
        super();
        this.controller = controller;
        this.setTitle("ASDF - Game browser");
        initGUI();
    }
	
    /**
     * Initializes GUI for lobby. 
     */
	private void initGUI() {
		try {
		    GridBagLayout thisLayout = new GridBagLayout();
            thisLayout.rowWeights = new double[] {0.0, 0.2, 0.2, 0.2, 0.2, 0.0, 0.2, 0.1};
            thisLayout.rowHeights = new int[] {22, 7, 7, 7, 7, 89, 7, 7};
            thisLayout.columnWeights = new double[] {0.1, 0.1, 0.1, 0.1, 0.1, 0.1};
            thisLayout.columnWidths = new int[] {7, 7, 7, 7, 7, 7};
			getContentPane().setLayout(thisLayout);
			setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);;
			{
			    topPanel = new LobbyTopPanel(this.controller);
			    getContentPane().add(topPanel, new GridBagConstraints(0, 0, 6, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
			}
			{
			    leftPanel = new LobbyLeftPanel(this.controller);
			    getContentPane().add(leftPanel, new GridBagConstraints(0, 1, 1, 7, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
                leftPanel.setMinimumSize(new java.awt.Dimension(100, 100));
                leftPanel.setMaximumSize(new java.awt.Dimension(200, -1));
			}
			{
			    middlePanel = new LobbyMiddlePanel(this.controller);
			    getContentPane().add(middlePanel, new GridBagConstraints(1, 1, 3, 6, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
                middlePanel.setMinimumSize(new java.awt.Dimension(200, 200));
                middlePanel.setMaximumSize(new java.awt.Dimension(-1, -1));
                middlePanel.setPreferredSize(new java.awt.Dimension(550, 500));  
			}
			{
				showEmptyRightPanel();
			}
			{
			    bottomPanel = new LobbyBottomPanel(this.controller);
			    getContentPane().add(bottomPanel, new GridBagConstraints(1, 7, 3, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
                bottomPanel.setMaximumSize(new java.awt.Dimension(500, 30));
                bottomPanel.setMinimumSize(new java.awt.Dimension(300, 20));
                bottomPanel.setSize(500, 20);
                bottomPanel.setPreferredSize(new java.awt.Dimension(500, 30));
			}

			this.pack();
			this.setSize(1024, 768);
			this.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Returns the panel containing tree of the available games.
	 * @return LobbyLeftPanel
	 */
	public LobbyLeftPanel getGameTreePanel(){
	    return (LobbyLeftPanel) leftPanel;
	}
	/**
	 * Returns the panel containing table of listed games.
	 * @return LobbyMiddlePanel
	 */
	public LobbyMiddlePanel getGameListPanel(){
        return (LobbyMiddlePanel) middlePanel;
    }
	/**
	 * Returns the login panel.
	 * @return LobbyTopPanel
	 */
    public LobbyTopPanel getLoginPanel(){
        return (LobbyTopPanel) topPanel;
    }
    
    /**
     * Sets minimum and maximum sizes of rightPanel.
     */
    protected void setRightPanelDimensions() {
    	rightPanel.setMinimumSize(new java.awt.Dimension(150, -1));
        rightPanel.setMaximumSize(new java.awt.Dimension(300, -1));
        rightPanel.setPreferredSize(new java.awt.Dimension(200, -1));
        this.validate();
    }
    
    /**
     * Show empty panel on the right.
     */
    public void showEmptyRightPanel() {
    	if (rightPanel != null)
    		getContentPane().remove(rightPanel);
    	rightPanel = new JPanel();
    	getContentPane().add(rightPanel, new GridBagConstraints(4, 1, 2, 7, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
    	setRightPanelDimensions();
    }
    
    /**
     * Show "Create game" panel on the right
     * @param panel Panel to show
     */
    public void showCreateGamePanel(CreateGamePanel panel){
    	if (rightPanel != null)
    		getContentPane().remove(rightPanel);
	    rightPanel = panel;
	    panel.setController(this.controller);
	    getContentPane().add(rightPanel, new GridBagConstraints(4, 1, 2, 7, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));    
	    setRightPanelDimensions();
    }
    
    /**
     * Shows game details panel on the right. Panel shows
     * player with their seats and possible optional information.
     * @param response Protocol response object containing data to show on panel
     */
    public void showGameDetailsPanel(GameDetailsResponse response){
    	if (rightPanel != null)
    		getContentPane().remove(rightPanel);
	    rightPanel = new LobbyRightPanel(this.controller);
	    ((LobbyRightPanel) rightPanel).showGameDetails(response);
	    getContentPane().add(rightPanel, new GridBagConstraints(4, 1, 2, 7, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
	    setRightPanelDimensions();
    }

	@Override
	public void windowActivated(WindowEvent e) {
	}

	@Override
	public void windowClosed(WindowEvent e) {
		controller.requestLogOut();
		System.exit(0);		
	}

	@Override
	public void windowClosing(WindowEvent e) {		
	}

	@Override
	public void windowDeactivated(WindowEvent e) {		
	}

	@Override
	public void windowDeiconified(WindowEvent e) {		
	}

	@Override
	public void windowIconified(WindowEvent e) {		
	}

	@Override
	public void windowOpened(WindowEvent e) {		
	}


}
