package com.asdf.gui.lobby;

import javax.swing.table.DefaultTableModel;

import com.asdf.parser.GameDetailsResponse.Players.Player;
import com.asdf.parser.GameDetailsResponse;
import com.asdf.parser.GameListResponse;
import com.asdf.parser.GameListResponse.Games.Game;
import com.asdf.utils.Filter;

/**
 * Table model for various tables in lobby's GUI.
 * 
 * @author asdf
 *
 */
public class LobbyGameTableModel extends DefaultTableModel {
  
	/**
	 * Default serial.
	 */
	private static final long serialVersionUID = 1L;
	
    private GameListResponse listData = null;
    private GameDetailsResponse detailsData = null;
    private Filter gameListFilter = null;
    
    /**
     * Empty constructor which only calls superclass constructor.
     */
    public LobbyGameTableModel(){
        super();
    }

    /**
     * Cells are not editable.
     */
    public boolean isCellEditable(int rowIndex, int colIndex) {                 
        return false;          
    }
       
    /**
     * Constructor for game details table.
     * 
     * @param detailsData Response from server containing game details.
     */
    public LobbyGameTableModel(GameDetailsResponse detailsData){
        super();
        this.detailsData = detailsData;
        setGameDetailsList(this.detailsData);
    }
    
    /**
     * Constructor for game list table.
     * 
     * @param listData Response from server containing game list.
     */
    public LobbyGameTableModel(GameListResponse listData){
        super();
        this.listData = listData;
        setDataGameList(this.listData, null);     
    }
    
    /**
     * Constructor for game list table.
     * 
     * @param listData Response from server containing game list.
     */
    public LobbyGameTableModel(GameListResponse listData, Filter gameListFilter){
        super();
        this.listData = listData;
        this.gameListFilter = gameListFilter;
        setDataGameList(this.listData, this.gameListFilter);     
    }

    /**
     * Sets game list as a data for this table.
     * @param data Response from server containing game list.
     * @param filter Rules based on which shown games are filtered.
     */
    public void setDataGameList(GameListResponse data, Filter filter){
        listData = data;
        if(data!=null){
            String[] colData = null;
            String[][] rowData = null;
            if(data.getColumnNames()!=null){
                colData = new String[data.getColumnNames().getString().size()+3];     
                rowData = new String[data.getGames().getGame().size()][data.getColumnNames().getString().size()+3];
                int j = 3;
                for(String s : data.getColumnNames().getString()){
                    colData[j] = s; 
                    j++;
                }
            }else{
                colData = new String[3];  
                rowData = new String[data.getGames().getGame().size()][3];
            }
            colData[0] = "ID";
            colData[1] = "Name";
            colData[2] = "Players";

            int n = 0, m = 1, skipped = 0;
            
            for(Game g : data.getGames().getGame()){             	
            
        		if(g != null){
        			/* Monstrous filter checking which simply checks 
        			 * all the filter rules. These are is the game empty or full,
        			 * how many players it has and is the game of the type we
        			 * wanted to list. If one of these rules is not met, then the
        			 * game is not put to this table and thus does not show in
        			 * lobby's game browser.
        			 */
        			if((filter != null) && !(filter.hideFull() && (g.getMaxPlayers() == g.getPlayers()))
        					&& !(filter.hideEmpty() && (g.getPlayers() == 0)) && filter.players(g.getPlayers())
        					&& filter.nameContains(g.getName())
        					&& (data.getGames().getRequestedGameVariant() == null || (g.getGameVariant() != null 
        					&& g.getGameVariant().equalsIgnoreCase(data.getGames().getRequestedGameVariant())))){
        				
		                rowData[n][0] = ""+g.getGameID();
		                rowData[n][1] = ""+g.getName();
		                rowData[n][2] = ""+g.getPlayers()+" / "+g.getMaxPlayers();
		                m = 3;
		                if(data.getColumnNames() != null && g.getAdditionalInfo() != null && g.getAdditionalInfo().getOther().getParameters() != null){
		                    for(String s : data.getColumnNames().getString()){	  
		                       rowData[n][m] = g.getAdditionalInfo().getOther().getParameters().getAny().get(0).getAttribute(s);
		                       m++;
		                       if(m >= rowData[n].length)
		                           break;
		                    }
		                }
		                n++;
        		
        			}else{
        				skipped++;
        			}
            	}
            }
            String[][] tmp = new String[data.getGames().getGame().size() - skipped][data.getColumnNames().getString().size()+3];
            for(int i = 0; i < tmp.length; i++){
            	tmp[i] = rowData[i];
            }
            
            super.setDataVector(tmp, colData);        
        }
    }      
     
    /**
     * Sets game details as a data for this table.
     * @param data Response from server containing game details.
     */
    public void setGameDetailsList(GameDetailsResponse data) {
        detailsData = data;

		if (data != null && data.getColumnNames() != null) {

			// Get number of columns and rows
			int nrColumns = data.getColumnNames().getString().size();
			int nrRows = data.getPlayers().getPlayer().size();

			// Initialize data structures
			String[][] rowData = new String[nrRows][nrColumns];
			String[] colData = new String[nrColumns];

			// Populate column names
			for (int i = 0; i < nrColumns; i++)
				colData[i] = data.getColumnNames().getString().get(i);

			// Populate rows
			int currentRow = 0;
			for (Player p : data.getPlayers().getPlayer()) {
				if(p!=null){
					int j = 0;
					for(String s : colData){					
						rowData[currentRow][j] = p.getInfo().getParameters().getAny().get(0).getAttribute(s);
						j++;
					}
					currentRow++;
				}
			}

			super.setDataVector(rowData, colData);
		}
    }  
    

    
}
