package com.asdf.gui.lobby;

import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.JTree;
import javax.swing.border.BevelBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreeSelectionModel;

import com.asdf.client.LobbyController;
import com.asdf.parser.AvailableGamesResponse;
import com.asdf.parser.AvailableGamesResponse.Game;

/**
 * Panel used by LobbyFrame to show available games.
 * 
 * @author asdf
 *
 */
public class LobbyLeftPanel extends javax.swing.JPanel implements TreeSelectionListener {

	/**
	 * Default serial.
	 */
	private static final long serialVersionUID = 1L;
	
	
	{
		//Set Look & Feel
		try {
			javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
		} catch(Exception e) {
			e.printStackTrace();
		}
	}

    private JTree gameTree;
    private LobbyController controller = null;
   
    /**
     * Sole constructor.
     *       
     * @param controller Lobby controller which controls this component.
     */
	public LobbyLeftPanel(LobbyController controller) {
	    super();    
	    this.controller = controller;        	    
    }
	
	
    /**
     * Initializes GUI for this panel. 
     * @param games containing games available on server.
     */
   public void initGUI(AvailableGamesResponse games) {
       this.removeAll();       
       this.repaint();
        try {
            this.setPreferredSize(new java.awt.Dimension(150, 550));
            GridLayout thisLayout = new GridLayout(1, 1);
            thisLayout.setColumns(1);
            thisLayout.setHgap(5);
            thisLayout.setVgap(5);
            this.setLayout(thisLayout);
            this.setSize(150, 550);
            
            {               
                DefaultMutableTreeNode top =
                    new DefaultMutableTreeNode("Available games");
                
                if(games != null && games.getGame().size() > 0)
                    createNodes(top, games);     
                
                gameTree = new JTree(top);
              
                this.add(gameTree);
                
                gameTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
                gameTree.setShowsRootHandles(false);
                gameTree.setBorder(BorderFactory.createEtchedBorder(BevelBorder.LOWERED));
                gameTree.addTreeSelectionListener(this);                
            }
            
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

   /**
    * Creates tree nodes.
    * 
    * @param top Root node of the tree.
    * @param games AvailableGamesResponse containing games available on server.
    */
	private void createNodes(DefaultMutableTreeNode top, AvailableGamesResponse games) {
        DefaultMutableTreeNode game = null;
        DefaultMutableTreeNode setting = null;
        
        for(Game g : games.getGame()){           
            game = new DefaultMutableTreeNode(g.getType());
            top.add(game);
            if(g.getVariants()!=null){
                for(String s : g.getVariants().getString()){
                    setting = new DefaultMutableTreeNode(s);
                    game.add(setting);
                }
            }
        }
    }

    @Override    
    /**
     * Action listener for tree events.
     */    
    public void valueChanged(TreeSelectionEvent e) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)gameTree.getLastSelectedPathComponent();
        
        if (node != null) {   
            String selected = null;
            TreeNode parent = node.getParent();
            String gameVariant = null;
            
            if(parent != null && !parent.toString().equalsIgnoreCase("Available Games")){               
            	selected = parent.toString();
                gameVariant = (String) node.getUserObject();     
                controller.requestGameList(selected, gameVariant);                
            } else if (parent != null) {        
            	selected = (String) node.getUserObject();    
                controller.requestGameList(selected, null);                
            }
            
        }
    }

}
