package com.asdf.gui.lobby;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.table.TableModel;

import com.asdf.client.LobbyController;
import com.asdf.parser.GameListResponse;
import com.asdf.utils.Filter;

/**
 * Middle panel containing running games list.
 */
public class LobbyMiddlePanel extends javax.swing.JPanel {
    
	/**
	 * Default serial.
	 */
	private static final long serialVersionUID = 1L;

	/**
     * Can be used to control the distribution of tables width amongst columns.
     * Higher value should give more width to columns needing it more...
     */
    public static final double COL_WIDTH_MULTIPLIER = 100;

    {
        //Set Look & Feel
        try {
            javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    private JScrollPane jScrollPane;
    private LobbyTable gamesTable;
    private ListSelectionModel listSelectionModel;
    private LobbyController controller = null;

    /**
     * Sole constructor.
     *       
     * @param controller Lobby controller which controls this component.
     */
    public LobbyMiddlePanel(LobbyController controller) {
        super();
        this.controller = controller;
        initGUI();
    }
    
    /**
     * Sets game list data to be shown on this panel.
     * 
     * @param gameList GameListResponse from server containing list of games.
     * @param filter Filter to be used for filtering the gameList
     */
    public void setData(GameListResponse gameList, Filter filter){

		if(gameList != null){
	        TableModel jTableModel = new LobbyGameTableModel(gameList, filter);   
	
	        gamesTable.setModel(jTableModel);
	        		       
	        // Set preferred column widths
	        for (int i = 0; i < gamesTable.getColumnCount(); i++) {
	            int length = 0;
	            int tmp = 0;
	            for (int j = 0 ; j < gamesTable.getModel().getRowCount(); j++){
	            	if(gamesTable.getModel().getValueAt(j, i) != null)		            		
	            		tmp = gamesTable.getModel().getValueAt(j, i).toString().length();
	                if(tmp > length)
	                    length = tmp;
	            }
	            gamesTable.getColumnModel().getColumn(i).setPreferredWidth((int)(length*COL_WIDTH_MULTIPLIER)); 
	        }
		}else{
			//System.out.println("gamelist was empty");
		}
    	
    }

    /**
     * Initializes GUI.
     */
    public void initGUI() {
        this.removeAll();        
        try {
            GridLayout thisLayout = new GridLayout(1, 1);
            thisLayout.setColumns(1);
            thisLayout.setHgap(5);
            thisLayout.setVgap(5);         
            this.setLayout(thisLayout);
            this.setPreferredSize(new java.awt.Dimension(550, 500));           
            this.setSize(550, 500);        
            this.setBackground(Color.WHITE);       
            {
                jScrollPane = new JScrollPane();                
                this.add(jScrollPane);

                {                                      
                    gamesTable = new LobbyTable();
                    jScrollPane.setViewportView(gamesTable);      

                    gamesTable.setRowHeight(20);
                    gamesTable.setRowMargin(5);

                    gamesTable.setShowVerticalLines(false);

                    gamesTable.getTableHeader().setVisible(true);
                    gamesTable.setAutoCreateRowSorter(true);        
                    gamesTable.setRowSelectionAllowed(true);
                    gamesTable.setColumnSelectionAllowed(false);
                    gamesTable.setFillsViewportHeight(true);
                    gamesTable.setFocusable(false);


                    listSelectionModel = gamesTable.getSelectionModel();
                    listSelectionModel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
                    
                    gamesTable.addMouseListener(new MouseAdapter() {
                    	public void mouseClicked(MouseEvent evt) {
                    		
                    		if(evt.getButton() != MouseEvent.BUTTON1) {
                    			gamesTable.clearSelection();
                    			controller.setGameDetails(null);                                
                    			
                    		}else{                               
                    			if(!gamesTable.getSelectionModel().isSelectionEmpty())
                    				controller.requestGameDetails(
                    						gamesTable.getValueAt(
                    								gamesTable.getSelectionModel().getMinSelectionIndex(), 0).toString());
                    			
                    		}
                    		
                    	}
                    });
        

                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        this.repaint();
    }
   
}
