package com.asdf.gui.lobby;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableModel;

import com.asdf.client.LobbyController;
import com.asdf.parser.GameDetailsResponse;

/**
 * RightPanel is used by LobbyFrame to show game's details.
 * @author asdf
 *
 */
public class LobbyRightPanel extends javax.swing.JPanel {

	private final class DefaultTableCellRendererExtension extends
			DefaultTableCellRenderer {
		
		/**
		 * Default serial.
		 */
		private static final long serialVersionUID = 1L;
		
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int col) {
			Component comp = super.getTableCellRendererComponent(table, value,
					isSelected, hasFocus, row, col);
			String s = "";
			for(int i = 0; i < table.getColumnCount(); i++)
				if(table.getColumnName(i).equalsIgnoreCase("Name")){
					s = table.getValueAt(row, i).toString();
				}

			if (s.equalsIgnoreCase("Reserved")) {
				if (isSelected)
					comp.setForeground(Color.black);
				else {
					comp.setBackground(new Color(237,37,37));
					comp.setForeground(Color.white);
				}
			} else if (s.equalsIgnoreCase("Free")) {
				comp.setForeground(Color.black);
				if (!isSelected)
					comp.setBackground(new Color(205,255,162));
			} else {
				comp.setForeground(Color.black);
				if (!isSelected)
					comp.setBackground(new Color(252,208,132));
			}
			comp.setFont(new Font("verdana", Font.PLAIN, 10));
			return (comp);
		}
	}

	/**
	 * Default serial.
	 */
	private static final long serialVersionUID = 1L;

	{
		// Set Look & Feel
		try {
			javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	/**
	 * Can be used to control the distribution of tables width amongst columns.
	 * Higher value should give more width to columns needing it more...
	 */
	public final static double COL_WIDTH_MULTIPLIER = 3.5;
	private JButton joinButton;
	private JPanel bottomPanel;
	private JTable gameDetails;
	private JScrollPane topPanel;
	private JTextArea gameRules;
	
	private ListSelectionModel listSelectionModel;
	
	private LobbyController controller;

	/**
     * Sole constructor.
     *       
     * @param controller Lobby controller which controls this component.
     */
	public LobbyRightPanel(LobbyController controller) {
        super();
        this.controller = controller;
        initGUI();
    }
	
	/**
	 * Initializes top part of this panel.
	 */
    private void initTopPanel(){
    
        topPanel = new JScrollPane();            
        this.add(topPanel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
        		GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        topPanel.setBorder(BorderFactory.createTitledBorder("Game details"));
        {

            gameDetails = new JTable();
            topPanel.setViewportView(gameDetails);
            
            gameDetails.setPreferredSize(new java.awt.Dimension(390, 64));
            gameDetails.setOpaque(false);
            gameDetails.setAutoCreateRowSorter(true);      
            gameDetails.setRowSelectionAllowed(true);
            gameDetails.setColumnSelectionAllowed(false);
            gameDetails.setFillsViewportHeight(true);
            gameDetails.setFocusable(false);
            
            gameDetails.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
            
            gameDetails.addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent evt) {
                    if(evt.getButton() != MouseEvent.BUTTON1)
                        gameDetails.clearSelection();
                }
            });

            gameDetails.setDefaultRenderer(Object.class, new DefaultTableCellRendererExtension());
            
            listSelectionModel = gameDetails.getSelectionModel();
            listSelectionModel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
           
            /*listSelectionModel.addListSelectionListener(new ListSelectionListener(){
                public void valueChanged(ListSelectionEvent e) {
                    ListSelectionModel lsm = (ListSelectionModel) e.getSource();
                       
                   /if(lsm.isSelectionEmpty()){
                        System.out.println("empty");                                
                    }else{
                        System.out.println(lsm.getMinSelectionIndex());                      
                    }

                }                        
            });*/

        }
        
	}
    /**
     * Initializes bottom part of this panel.
     */
	private void initBottomPanel() {
        bottomPanel = new JPanel();
        GridLayout bottomPanelLayout = new GridLayout(1, 1);
        bottomPanelLayout.setHgap(5);
        bottomPanelLayout.setVgap(5);
        bottomPanelLayout.setColumns(1);
        bottomPanel.setLayout(bottomPanelLayout);
        this.add(bottomPanel, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
        bottomPanel.setBorder(BorderFactory.createTitledBorder("Game rules"));
        {
            gameRules = new JTextArea();
            bottomPanel.add(gameRules);
            gameRules.setEditable(false);
        }
	}
	
	/**
	 * Sets details of a game showing on this panel.
	 * 
	 * @param details GameDetailsResponse from server containing details of a game.
	 */
	public void showGameDetails(GameDetailsResponse details){
	    
        TableModel jTableModel = new LobbyGameTableModel(details);
		gameDetails.setModel(jTableModel);

		// Set preferred column widths
		for (int i = 0; i < gameDetails.getModel().getColumnCount(); i++) {
			int length = 0;
			int tmp = 0;
			if (gameDetails.getModel().getRowCount() < 1)
				continue;
			for (int j = 0; j < gameDetails.getModel().getRowCount(); j++) {
				tmp = gameDetails.getModel().getValueAt(j, i).toString().length();
				if (tmp > length)
					length = tmp;
			}
			gameDetails.getColumnModel().getColumn(i).setPreferredWidth(
					(int) (length * COL_WIDTH_MULTIPLIER));
		}
		if (details != null)
			setRuleText(details.getRules().getString());
	}

	/**
	 * Sets rule text of this panel.
	 * 
	 * @param rules List of rules.
	 */
	public void setRuleText(List<String> rules){
	    gameRules.setText("");
        for(String s : rules)
            gameRules.append(s+"\n");     
	}
	
	/**
	 * Initializes GUI.
	 *
	 */
	public void initGUI() {
		try {
			GridBagLayout thisLayout = new GridBagLayout();
			thisLayout.rowWeights = new double[] {0.1, 0.0, 0.1, 0.0, 0.0, 0.0};
			thisLayout.rowHeights = new int[] {245, 7, 139, 7, 0, 7};
			thisLayout.columnWeights = new double[] {0.1};
			thisLayout.columnWidths = new int[] {7};
			this.setLayout(thisLayout);
			this.setPreferredSize(new java.awt.Dimension(300, 550));
            this.setSize(300, 550);
			{
				joinButton = new JButton();
								
				this.add(joinButton, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 0), 0, 0));
				joinButton.setText("Join");
				joinButton.setMaximumSize(new java.awt.Dimension(200, 40));
				joinButton.setMinimumSize(new java.awt.Dimension(80, 25));
				joinButton.setSize(14, 7);
				
				joinButton.addActionListener(new ActionListener() {
				    public void actionPerformed(ActionEvent evt) {
				        controller.requestJoinGame();
				    }
				});
				
			}
			
		    initBottomPanel();
			
			initTopPanel();
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
