package com.asdf.gui.lobby;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import javax.swing.border.BevelBorder;

import com.asdf.client.LobbyController;
import com.asdf.common.AccountType;

/**
 * TopPanel used by LobbyFrame to show account and login information.
 * 
 * @author louhio
 *
 */
public class LobbyTopPanel extends javax.swing.JPanel {
    
	/**
	 * Default serial.
	 */
	private static final long serialVersionUID = 1L;
	
    {
        //Set Look & Feel
        try {
            javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
        } catch(Exception e) {
            e.printStackTrace();
        }
    }    
    
    private JButton loginButton;
    private JPasswordField passwdField;
    private JTextField accountField;
    private boolean logged = false;
    private JLabel balanceLabel;
    private JLabel accountLabel;
    private JLabel serverLabel;
    private JButton createButton;

    private LobbyController controller;

    /**
     * Sole constructor.
     *       
     * @param controller LobbyController which controls this component.
     */
    public LobbyTopPanel(LobbyController controller) {
        super();
        this.controller = controller;
        initGUI();
    }

    /**
     * Checks if user is logged in.
     * @return true if logged in, false if logged out
     */
    public boolean isLogged(){
        return logged;
    }
    /**
     * Sets success of create account operation.
     * 
     * @param success true if account was created successfully, false if not.
     * @param description String containing error message.
     */
    public void createAccountSuccess(boolean success, String description) {
    	if(success){
    		setMessage("Account created successfully.", false);
    	}else{
    		String s = (description == null) ? "unknown reason" : description;
    		setMessage("Create failed: " + s, true);
    	}
    }
    
    /**
     * Sets panel to logged in state.
     * 
     * @param name Account name.
     * @param balance Account balance.
     * @param accountType Type of account. This can be AccountType.SUPER_USER, AccountType.BOT_USER, AccountType.REGULAR_USER.
     * See AccountType for more details.
     */
    public void setLoggedIn(String name, double balance, AccountType accountType){        
        
    	logged = true;

        if(accountType == AccountType.SUPER_USER)
            initLoggedGUI("[S] " + name, balance);
        else if(accountType == AccountType.BOT_USER)
            initLoggedGUI("[B] " + name, balance);
        else if(accountType == AccountType.REGULAR_USER)
            initLoggedGUI(name, balance);
        
        //setMessage("Logged in.", false);
    }
    
//    /**
//     * Initializes GUI for bot user. Currently not used.
//     * 
//     * @param name Account name.
//     * @param balance Account balance.
//     */
//    private void initBotGUI(String name, double balance) {
//        this.removeAll();
//        this.repaint();
//        try {
//            //Set layout
//            GridBagLayout thisLayout = new GridBagLayout();
//            this.setPreferredSize(new java.awt.Dimension(1000, 30));
//            thisLayout.rowWeights = new double[] {0.1};
//            thisLayout.rowHeights = new int[] {7};
//            //thisLayout.columnWeights = new double[] {0.1, 0.0, 0.0, 0.0, 0.0};
//            //thisLayout.columnWidths = new int[] {400, 160, 120, 50, 40};
//            thisLayout.columnWeights = new double[] {0.1, 0.0, 0.0, 0.0};
//            thisLayout.columnWidths = new int[] {450, 160, 120, 40};
//            this.setLayout(thisLayout);
//            this.setSize(1000, 30);
//            this.setBorder(BorderFactory.createEtchedBorder(BevelBorder.LOWERED));
//            
//            {
//                loginButton = new JButton();
//                this.add(loginButton, new GridBagConstraints(4, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
//                loginButton.setText("Logout");
//                
//                loginButton.addActionListener(new ActionListener() {
//                    public void actionPerformed(ActionEvent evt) {       
//                        //controller.requestLogOut();
//                    }
//                });
//            }
//            {
//                balanceLabel = new JLabel();
//                this.add(balanceLabel, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
//                balanceLabel.setText("Balance: " + balance + "$");
//                balanceLabel.setHorizontalAlignment(SwingConstants.CENTER);
//
//            }
//            {
//                accountLabel = new JLabel();
//                this.add(accountLabel, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
//                accountLabel.setText(name);
//                accountLabel.setHorizontalAlignment(SwingConstants.RIGHT);
//   
//            }
//            /*{
//                botButton = new JButton();
//                this.add(botButton, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
//                botButton.setText("Bot menu");
//                
//                botButton.addActionListener(new ActionListener() {
//                    public void actionPerformed(ActionEvent evt) {       

//                        //controller.requestLogOut();
//                    }
//                });
//            }*/
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        
//    }
    
//    /**
//     * Initializes GUI for super user. Currently not used.
//     * 
//     * @param name Account name.
//     * @param balance Account balance.
//     */
//    private void initSuperGUI(String name, double balance) {
//        this.removeAll();
//        this.repaint();
//        try {
//            //Set layout
//            GridBagLayout thisLayout = new GridBagLayout();
//            this.setPreferredSize(new java.awt.Dimension(1000, 30));
//            thisLayout.rowWeights = new double[] {0.1};
//            thisLayout.rowHeights = new int[] {7};
//            //thisLayout.columnWeights = new double[] {0.1, 0.0, 0.0, 0.0, 0.0};
//            //thisLayout.columnWidths = new int[] {400, 160, 120, 50, 40};
//            thisLayout.columnWeights = new double[] {0.1, 0.0, 0.0, 0.0};
//            thisLayout.columnWidths = new int[] {450, 160, 120, 40};
//            this.setLayout(thisLayout);
//            this.setSize(1000, 30);
//            this.setBorder(BorderFactory.createEtchedBorder(BevelBorder.LOWERED));
//            
//  
//            
//            {
//                loginButton = new JButton();
//                this.add(loginButton, new GridBagConstraints(4, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
//                loginButton.setText("Logout");
//                
//                loginButton.addActionListener(new ActionListener() {
//                    public void actionPerformed(ActionEvent evt) {       
//                        //controller.requestLogOut();
//                    }
//                });
//            }
//            {
//                balanceLabel = new JLabel();
//                this.add(balanceLabel, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
//                balanceLabel.setText("Balance: " + balance + "$");
//                balanceLabel.setHorizontalAlignment(SwingConstants.CENTER);
//
//            }
//            {
//                accountLabel = new JLabel();
//                this.add(accountLabel, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
//                accountLabel.setText(name);
//                accountLabel.setHorizontalAlignment(SwingConstants.RIGHT);
//   
//            }
//            /*{
//                superButton = new JButton();
//                this.add(superButton, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
//                superButton.setText("Super menu");
//                
//                superButton.addActionListener(new ActionListener() {
//                    public void actionPerformed(ActionEvent evt) {       
//                        
//                        //controller.requestLogOut();
//                    }
//                });
//            }*/
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        
//    }

    /**
     * Initializes GUI for regular user when logged in.
     * 
     * @param name Account name.
     * @param balance Account balance.
     */
    private void initLoggedGUI(String name, double balance) {
        this.removeAll();
        this.repaint();
        try {
            //Set layout
            GridBagLayout thisLayout = new GridBagLayout();
            this.setPreferredSize(new java.awt.Dimension(1000, 30));
            thisLayout.rowWeights = new double[] {0.1};
            thisLayout.rowHeights = new int[] {7};
            thisLayout.columnWeights = new double[] {0.1, 0.0, 0.0, 0.0};
            thisLayout.columnWidths = new int[] {450, 160, 120, 40};
            this.setLayout(thisLayout);
            this.setSize(1000, 30);
            this.setBorder(BorderFactory.createEtchedBorder(BevelBorder.LOWERED));
            
            serverLabel = new JLabel();
            this.add(serverLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
            
            {
                loginButton = new JButton();
                this.add(loginButton, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
                loginButton.setText("Logout");
                
                loginButton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent evt) {       
                        controller.requestLogOut();
                    }
                });
            }
//            {
//                createButton = new JButton();
//                this.add(createButton, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
//                createButton.setText("Create new account");
//                createButton.addActionListener(new ActionListener() {
//                    public void actionPerformed(ActionEvent evt) {   
//                    	createButton.setText("Create account");
//                    	createButton.setForeground(Color.BLACK);
//                        JFrame frame = new LobbyCreateAccountPanel(controller);                       
//                        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
//                        frame.setTitle("Create a new account.");
//                        frame.pack();
//                        frame.setVisible(true);
//                    }
//                });
//            }
            {
                balanceLabel = new JLabel();
                this.add(balanceLabel, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
                balanceLabel.setText("Balance: " + balance + "$");
                balanceLabel.setHorizontalAlignment(SwingConstants.CENTER);

            }
            {
                accountLabel = new JLabel();
                this.add(accountLabel, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
                accountLabel.setText("Logged in as " + name + ".");
                accountLabel.setHorizontalAlignment(SwingConstants.RIGHT);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        
    }

    
    public String getAccountName() {
    	if(accountLabel == null)
    		return "";
    	return accountLabel.getText();
    }
    
    /**
     * Sets this panel to logged out state.
     */
    public void setLoggedOut() {
        logged = false;
        initGUI();
        setMessage("Logged out", false);
    }

    /**
     * Initializes this GUI when logged out.
     */
    private void initGUI() {
        this.removeAll();
        this.repaint();
        try {
        	GridBagLayout thisLayout = new GridBagLayout();
            this.setPreferredSize(new java.awt.Dimension(1000, 30));
            thisLayout.rowWeights = new double[] {0.1};
            thisLayout.rowHeights = new int[] {7};
            thisLayout.columnWeights = new double[] {0.1, 0.0, 0.0, 0.0, 0.0};
            thisLayout.columnWidths = new int[] {450, 40, 160, 100, 40};
            this.setLayout(thisLayout);
            this.setSize(1000, 30);
            this.setBorder(BorderFactory.createEtchedBorder(BevelBorder.LOWERED));
            
            serverLabel = new JLabel();
            this.add(serverLabel, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
                        
            {
                loginButton = new JButton();
                this.add(loginButton, new GridBagConstraints(4, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
                loginButton.setText("Login");
                loginButton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent evt) {                          
                        controller.requestLogIn(accountField.getText(), new String(passwdField.getPassword()));                        
                    }
                });
            }
           
            {
                accountField = new JTextField();
                this.add(accountField, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
                accountField.setText("account");
                accountField.addFocusListener(new FocusListener() {

					public void focusGained(FocusEvent e) {
						accountField.selectAll();
					}

					public void focusLost(FocusEvent e) {						
					}
                	
                });

                accountField.addKeyListener(new KeyListener() {
        			public void keyPressed(KeyEvent k) {
        				if (k.getKeyCode() == KeyEvent.VK_ENTER ||
        					k.getKeyCode() == KeyEvent.VK_TAB) {
        					passwdField.requestFocus();
        				}
        			};
        			public void keyReleased(KeyEvent k) {};
        			public void keyTyped(KeyEvent k) {};
        		});
            }
            {
                createButton = new JButton();
                this.add(createButton, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
                createButton.setText("Create new account");
                createButton.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent evt) {   
                    	createButton.setText("Create account");
                    	createButton.setForeground(Color.BLACK);
                        JFrame frame = new LobbyCreateAccountPanel(controller);                       
                        frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
                        frame.setTitle("Create a new account.");
                        frame.pack();
                        frame.setVisible(true);
                    }
                });
            }
            {
                passwdField = new JPasswordField();
                this.add(passwdField, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(2, 2, 2, 2), 0, 0));
                passwdField.setText("password");
                passwdField.addFocusListener(new FocusListener() {

					public void focusGained(FocusEvent e) {
						passwdField.selectAll();
					}

					public void focusLost(FocusEvent e) {						
					}
                	
                });

                passwdField.addKeyListener(new KeyListener() {
        			public void keyPressed(KeyEvent k) {
        				if (k.getKeyCode() == KeyEvent.VK_ENTER) {
        					loginButton.getActionListeners()[0].actionPerformed(null);
        				}
        			};
        			public void keyReleased(KeyEvent k) {};
        			public void keyTyped(KeyEvent k) {};
        		});
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Set message displayed in top panel
     * @param text Message
     */
    public void setMessage(String text, boolean red) {
    	if (red)
    		serverLabel.setForeground(Color.RED);
    	else
    		serverLabel.setForeground(Color.BLACK);
    	
    	serverLabel.setText(text);
    }

}
