package com.asdf.server;

import java.util.TreeMap;
import java.util.Random;

import com.asdf.common.AccountType;
import com.asdf.common.Game;

/**
 * Stores account details like name and type.
 * List of associated sessions is also stored.
 */
public class Account {
	
	private static Random rnd = new Random();
	private String name;
	private TreeMap<Long, Game> sessions;
	private AccountType type;
	private double balance;
	
	public Statistics scores;
	
	/**
	 * Constructor. Creates a new account with a given account name.
	 * @param name Account name
	 */
	public Account(String name, AccountType type) {
		this.name = name;
		this.type = type;
		sessions = new TreeMap<Long, Game>();
		
		scores = new Statistics();
		
	}

	/**
	 * Getter for account name
	 * @return Name of the account
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Return the type of the user
	 * @return Type as short
	 */
	public AccountType getType() {
		return type;
	}

	/**
	 * Creates a new session for this account
	 * @param id Session ID
	 * @param game Game with which this session is associated with
	 * @return returns true on successful operation, false otherwise.
	 */
	public boolean insertSession(Long id, Game game) {
		if (sessions.get(id) == null) {
			sessions.put(id, game);
			return true;
		}
		return false;
	}

	/**
	 * Creates a new session for this account. Success is guaranteed but time consumption
	 * is a stochastic random variable.
	 * @param game Game with which this session is associated with
	 */
	
	public void insertSession(Game game) {
		Long id = Long.valueOf(rnd.nextLong());
		while (sessions.get(id) != null) {
			id = Long.valueOf(rnd.nextLong());
		}
		sessions.put(id, game);
	}

	/**
	 * Removes a session from the account.
	 * @param id ID of the session to be removed.
	 */
	public void removeSession(Long id) {
		sessions.remove(id);
	}

	/**
	 * Getter for game instance.
	 * @param id Session ID associated with the game that we want to get.
	 * @return Game associated with the given session ID. If there exists no such game, null is returned.
	 */
	public Game getSession(Long id) {
		return sessions.get(id);
	}

	/**
	 * @return the balance
	 */
	public double getBalance() {
		return balance;
	}

	/**
	 * @param balance the balance to set
	 */
	public void setBalance(double balance) {
		this.balance = balance;
	}

}