package com.asdf.server;

import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;

import com.asdf.common.Game;
import com.asdf.common.Logger;
import com.asdf.common.PluginLoadException;

/**
 * Handles dynamic loading and initializing of new games.
 */
public class GameFactory {
	
	// Determines if classloader's classpath is already h4x0red
	private static boolean initialized = false;
	
	/**
	 * Tries to load and initialize a class found in classDir
	 * in package className. Class is casted to Game before
	 * returning.
	 * 
	 * @param classDirs Directory from where the class is loaded from.
	 * @param className Fully qualified name of the class
	 * @param server Server instance
	 * @param gameID ID of the new game
	 * @param name Name of the game
	 * @return Instance of requested Game
	 * @throws PluginLoadException
	 */
	@SuppressWarnings("unchecked")
	public static Game createGame(String[] classDirs, String className, Server server, long gameID, String name) throws PluginLoadException {
		try {
			
			URLClassLoader sysloader = (URLClassLoader)ClassLoader.getSystemClassLoader();
			
			if (!initialized) {
				Class<URLClassLoader> sysclass = URLClassLoader.class;
		        try {
		        	
		            Method method = sysclass.getDeclaredMethod("addURL", new Class[]{URL.class});
		            method.setAccessible(true);
		            for (int i = 0; i < classDirs.length; i++) {
		            	method.invoke(sysloader,new Object[]{new URL(classDirs[i])});
		            }
		        } catch (Throwable t) {
		            t.printStackTrace(); 
		        }
			}
			
			
			Logger.get().info("Trying to create game of type " + className);

			ClassLoader cl = sysloader;
			// This can't be done more type safely since the class isn't available
			// at compile time so ignore the warning.
			Class<Game> cp = (Class<Game>)cl.loadClass(className);
			Game game = cp.newInstance();
			game.initialize(server, gameID, name);
			Logger.get().info("Loaded new instance of game " + className);
			return game;
		}
		catch(Exception e) {
			throw new PluginLoadException(e);
		}
		
	}
	
	/**
	 * Tries to load and initialize a class found in classDir
	 * in package className. Class is casted to Game before
	 * returning.
	 * 
	 * @param className Fully qualified name of the class
	 * @param server Server instance
	 * @param gameID ID of the new game
	 * @param name Name of the game
	 * @return Instance of requested Game
	 * @throws PluginLoadException
	 */
	public static Game createGame(String className, Server server, long gameID, String name) throws PluginLoadException {
		return createGame(Configuration.pluginPaths(), className, server, gameID, name);
	}
}
