package com.asdf.server;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.util.Date;
import java.util.logging.Formatter;
import java.util.logging.LogRecord;

/**
 * Custom formatter for PokeriPalvelin log messages.
 */
public class ServerFormatter extends Formatter {

	public ServerFormatter() {
		super();
	}
	
	@SuppressWarnings("deprecation")
	private String formatDate(Date date) {
		StringBuffer sb = new StringBuffer();
		sb.append(String.format("%04d:%02d:%02d %02d:%02d:%02d",
				date.getYear() + 1900,
				date.getMonth() + 1,
				date.getDate(),
				date.getHours(),
				date.getMinutes(),
				date.getSeconds()));
		return sb.toString();
	}
	
	@Override
	public String format(LogRecord record) {
		//Create a StringBuffer to contain the formatted record
		// start with the date.
		StringBuilder sb = new StringBuilder();
		
		// Get the date from the LogRecord and add it to the buffer
		Date date = new Date(record.getMillis());
		sb.append(formatDate(date));
		sb.append(" ");
		
		// Get the level name and add it to the buffer
		sb.append(String.format("%-8s", record.getLevel().getName()));
		sb.append(" ");
		 
		// Get thread id
		sb.append("TID:" + Thread.currentThread().getId() + " ");
		
		// Get the formatted message (includes localization 
		// and substitution of paramters) and add it to the buffer
		sb.append(formatMessage(record));
		sb.append("\n");
		if (record.getThrown() != null) {
			sb.append("Class: " + record.getSourceClassName());
			sb.append(" Method: " + record.getSourceMethodName());
			sb.append(" Msg: " + record.getThrown().getMessage() + "\n");
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			record.getThrown().printStackTrace(new PrintStream(bos));
			sb.append(bos.toString());
		}
		

		return sb.toString();
	}

}
