package com.asdf.test;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.net.Socket;
import java.net.SocketException;
import java.util.concurrent.Semaphore;

import org.junit.Test;
import org.junit.Assert;

import com.asdf.server.Server;
import com.asdf.utils.ClientServerConstants;

/**
 * Tries to get client banned by using same sessionID
 * on two different socket connections.
 */
public class ServerBanTest extends ServerTest {

	protected Socket socket;
	protected DataInputStream input;
	protected DataOutputStream output;
	
	protected Socket socket2;
	protected DataInputStream input2;
	protected DataOutputStream output2;
	
	protected Socket socket3;
	
	/**
	 * Semaphore for determining when server has processed
	 * connecting requests.
	 */
	protected Semaphore serverSemaphore;
	protected Semaphore testSemaphore;
	
	private static final String LOGIN_MSG_PATH = "testmaterial/ServerGameTest/login.xml";
	
	public ServerBanTest() {
		
		serverSemaphore = new Semaphore(0);
		testSemaphore = new Semaphore(0);
	}
	
	@Override
	protected void onServerLoop(Server server) {
		if (serverSemaphore.tryAcquire())
			testSemaphore.release();
	}
	
	/**
	 * Used to synchronize test and server operation. onServerLoop
	 * is called when server have run one loop.
	 * @throws InterruptedException
	 */
	private void sync() throws InterruptedException {
		serverSemaphore.release();
		testSemaphore.acquire();
		System.err.println("Synchronized");
	}
	
	/**
	 * Test if socket is alive. Only way to do this is
	 * to try to send something to socket and wait if
	 * remote sends connection reset. It is assumed that
	 * no firewall is dropping packets in between as is the
	 * case usually in localhost connections.
	 * @param s Socket to test
	 * @return Alive or not
	 */
	private boolean socketAlive(Socket s) throws Exception {
		try {
			s.getOutputStream().write(0xff);
			sync();
			s.getOutputStream().write(0xff);
			return true;
		} catch (SocketException e) {
			return false;
		}
	}
	
	@Override
	@Test(timeout=3000)
	public void testServer() throws Exception {
		startServer();
		
		socket = new Socket("localhost", ClientServerConstants.PORT);
		input = new DataInputStream(socket.getInputStream());
		output = new DataOutputStream(socket.getOutputStream());

		socket2 = new Socket("localhost", ClientServerConstants.PORT);
		input2 = new DataInputStream(socket2.getInputStream());
		output2 = new DataOutputStream(socket2.getOutputStream());
		
		// First session
		sendMessage(LOGIN_MSG_PATH, output);
		receiveResponse(input);
		sync();
		
		// Second session which uses same sessionID as previous
		sendMessage(LOGIN_MSG_PATH, output2);
		// Receive response containing banned message
		receiveResponse(input2);
		
		Assert.assertEquals(false, socketAlive(socket2));

		// Try to connect with new socket to determine that connections
		// are instantly rejected.
		socket3 = new Socket("localhost", ClientServerConstants.PORT);
		Assert.assertEquals(false, socketAlive(socket3));
		
		// First session should still work
		sendMessage(LOGIN_MSG_PATH, output);
		receiveResponse(input);
		
		System.out.println("Closing socket");
		socket.close();
		System.out.println("Stopping server");
		stopServer();
	}

}
