package com.asdf.test;

import java.net.Socket;
import java.io.DataOutputStream;
import java.io.DataInputStream;
import org.junit.Test;
import org.junit.Assert;
import com.asdf.parser.Message;
import com.asdf.parser.CreateGameResponse;
import com.asdf.utils.ClientServerConstants;
import com.asdf.server.Server;


/**
 * Tries to create and join a game.
 */
public class ServerGameTest extends ServerTest {

	private static final String CREATE_MSG_PATH = "testmaterial/ServerGameTest/create.xml";
	private static final String JOIN_MSG_PATH = "testmaterial/ServerGameTest/join.xml";
	private static final String LOGIN_MSG_PATH = "testmaterial/ServerGameTest/login.xml";
	
	@Override
	protected void onServerLoop(Server server) {
		
	}
	
	@Override
	@Test(timeout=4000)
	public void testServer() throws Exception {
		startServer();
		
		try {
			socket = new Socket("localhost", ClientServerConstants.PORT);
			input = new DataInputStream(socket.getInputStream());
			output = new DataOutputStream(socket.getOutputStream());
			
			Message msg;
			sendMessage(LOGIN_MSG_PATH);
			msg = receiveResponse();
			Assert.assertEquals(true, validLoginResponse(msg));
			
			sendMessage(CREATE_MSG_PATH);
			msg = receiveResponse();
			Assert.assertEquals(true, validCreateResponse(msg));
			
			sendMessage(JOIN_MSG_PATH);
			msg = receiveResponse();
			Assert.assertEquals(true, validJoinResponse(msg));
	
			System.out.println("Closing socket");
			socket.close();
			System.out.println("Stopping server");
		} finally {
			stopServer();
		}
	}
	
	/**
	 * Validate response for login request.
	 * @param msg Message containing the response
	 * @return Valid or not
	 */
	private boolean validLoginResponse(Message msg) {
		return msg.getResponse().getAuthentication().getLogin().isSuccess();
	}
	
	/**
	 * Validate response received for the
	 * create game message.
	 * @param msg Message containing the response
	 * @return Valid or not
	 */
	private boolean validCreateResponse(Message msg) {
		CreateGameResponse cgr = msg.getResponse().getGame().getCreate();
		//this.gameID = cgr.getGameID();
		return cgr.isSuccess();
	}
	
	/**
	 * Validate response for joining game. 
	 * @param msg Message containing the response
	 * @return Valid or not
	 */
	private boolean validJoinResponse(Message msg) {
		return msg.getResponse().getGame().getJoin().isSuccess();
	}
	


}
