package com.asdf.utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;

public class ByteUtils {
	
	/**
	 * Returns given data chopped to BYTE_CHUNK_SIZE sized
	 * ByteBuffer pieces. Data is prepended with the size of
	 * the data.
	 * @param data to chop
	 * @return Array of bytebuffers containing pieces
	 */
	public static ByteBuffer[] brokeToPieces(byte[] data, int chunkSize) {
		
		ByteBuffer buffers[];
		int length = data.length + 4;
		int numberOfPieces = length / chunkSize;; 
		if (length % chunkSize != 0)
			numberOfPieces++;
		buffers = new ByteBuffer[numberOfPieces];
		
		ByteBuffer buffer = ByteBuffer.allocate(length);
		buffer.putInt(data.length);
		buffer.put(data);
		buffer.rewind();
		byte[] sizedData = buffer.array();
		
		for(int i = 0; i < numberOfPieces; i++) {
			int startIndex = chunkSize * i;
			int thisChunkSize = chunkSize;
			if (i == (numberOfPieces - 1))
				thisChunkSize = length - (i * chunkSize);
			
			buffers[i] = ByteBuffer.allocate(thisChunkSize);
			buffers[i].put(sizedData, startIndex, thisChunkSize);
			buffers[i].rewind();
		}
		
		return buffers;
	}
	
	/**
	 * Converts byte array to java integer.
	 * @param data Byte array
	 * @return Integer
	 * @throws IOException
	 */
	public static int bytesToInt(byte[] data) throws IOException {
		ByteArrayInputStream byteStream = new ByteArrayInputStream(data);
		DataInputStream dataStream = new DataInputStream(byteStream);
		return dataStream.readInt();
	}
	
	/**
	 * Converts int to byte array.
	 * @param integer Integer to convert
	 * @return Byte array containing given integer
	 * @throws IOException
	 */
	public static byte[] intToBytes(int integer) throws IOException {
		ByteArrayOutputStream byteStream = new ByteArrayOutputStream(4);
		DataOutputStream dataStream = new DataOutputStream(byteStream);
		dataStream.writeInt(integer);
		return byteStream.toByteArray();
	}
}
