package com.asdf.utils;

/**
 * Used by lobby to filter game lists. Filter is set filter rules and 
 * these rules are then checked when game list is drawn.
 * 
 * @author asdf
 *
 */
public class Filter {

	private boolean hideFull = false;
	private boolean hideEmpty = false;
	private int maxPlayers = Integer.MAX_VALUE;
	private int minPlayers = 0;
	private String s = null;
	
	/**
	 * Sets name filter rule.
	 * 
	 * @param s String to be used to filter game names.
	 */
	public void setNameContains(String s){
		if(s.equalsIgnoreCase(""))
			this.s = null;
		else
			this.s = s;
	}
	/**
	 * Sets filter rule.
	 * @param b
	 */
	public void setHideFull(boolean b){
		hideFull = b;
	}
	/**
	 * Sets filter rule.
	 * @param b
	 */
	public void setHideEmpty(boolean b){
		hideEmpty = b;
	}
	/**
	 * Checks if full games are hidden.
	 */
	public boolean hideFull(){
		return hideFull;
	}
	/**
	 * Checks if empty games are hidden.
	 */
	public boolean hideEmpty(){
		return hideEmpty;
	}
	/**
	 * Checks if name contains set name filter String.
	 * 
	 * @param name String to be checked.
	 * @return true if passes filter, false if not.
	 */
	public boolean nameContains(String name){
		if(s == null)
			return true;
		else if(name.contains(s))
			return true;
		else
			return false;
	}
	/**
	 * Sets player amount filter rule.
	 * 
	 * @param min Minimum players.
	 * @param max Maximum players.
	 */
	public void setPlayers(int min, int max){
		minPlayers = min;
		maxPlayers = max;
	}
	/**
	 * Checks if player amount is within the bounds of this filter.
	 * @param players Amount of players.
	 * @return is within the bounds
	 */
	public boolean players(int players){
		if(players <= maxPlayers && players >= minPlayers){
			return true;
		}else{
			return false;
		}
	}
	
}
